
/* EG: minicpp.g -> CMM.hpp */

#ifndef __PROG_CMM_HPP__
#define __PROG_CMM_HPP__

#include <cassert>
#include <string>
#include <antlr/TokenStreamSelector.hpp>
using namespace std;

namespace gibbon
{

extern antlr::TokenStreamSelector * selector;

enum CmmAccess
{
  PrivateAccess, 
  ProtectedAccess, 
  PublicAccess, 
  NoAccess
};

enum CmmSpecKind
{
  a_inline, 
  a_virtual, 
  a_explicit, 
  a_auto, 
  a_register, 
  a_static, 
  a_extern, 
  a_mutable, 
  a_friend, 
  a_typedef, 
  a_const, 
  a_volatile
};

enum CmmTypeKind
{
  a_char, 
  a_wchar, 
  a_bool, 
  a_short, 
  a_int, 
  a_long, 
  a_signed, 
  a_unsigned, 
  a_float, 
  a_double, 
  a_void
};

enum CmmDefineItemMode
{
  ident_mode, 
  text_mode, 
  string_mode, 
  expr_mode, 
  expr_list_mode, 
  stat_mode, 
  decl_mode, 
  init_item_mode, 
  init_list_mode
};

enum CmmValueExpr_kind
{
  ValueExp
};

enum CmmModernCastExpr_kind
{
  DynamicCast, 
  StaticCast, 
  ConstCast, 
  ReinterpreterCast
};

enum CmmBinaryExpr_kind
{
  DotMemberExp, 
  ArrowMemverExp, 
  MulExp, 
  DivExp, 
  ModExp, 
  AddExp, 
  SubExp, 
  ShlExp, 
  ShrExp, 
  LtExp, 
  GtExp, 
  LeExp, 
  GeExp, 
  EqExp, 
  NeExp, 
  BitAndExp, 
  BitXorExp, 
  BitOrExp, 
  LogAndExp, 
  LogOrExp, 
  AssignExp, 
  AddAssignExp, 
  SubAssignExp, 
  MulAssignExp, 
  DivAssignExp, 
  ModAssignExp, 
  ShlAssignExp, 
  ShrAssignExp, 
  AndAssignExp, 
  XorAssignExp, 
  OrAssignExp, 
  CommaExp
};

enum CmmClassDecl_style
{
  ClassStyle, 
  StructStyle, 
  UnionStyle
};

class CmmBasic;
class CmmMemberVisibility;
class CmmBaseItem;
class CmmSpec;
class CmmType;
class CmmDecl;
class CmmCompoundStat;
class CmmDefineSect;
class CmmDefineItem;
class CmmSuperStat;
class CmmTextSect;
class CmmContName;
class CmmSimpName;
class CmmNameExpr;
class CmmExpr;
class CmmValueExpr;
class CmmThisExpr;
class CmmSubexprExpr;
class CmmModernCastExpr;
class CmmTypeidExpr;
class CmmIndexExpr;
class CmmCallExpr;
class CmmFieldExpr;
class CmmPtrFieldExpr;
class CmmPostIncExpr;
class CmmPostDecExpr;
class CmmExprList;
class CmmIncExpr;
class CmmDecExpr;
class CmmDerefExpr;
class CmmAddrExpr;
class CmmPlusExpr;
class CmmMinusExpr;
class CmmBitNotExpr;
class CmmLogNotExpr;
class CmmSizeofExp;
class CmmNewExpr;
class CmmNewArrayLimit;
class CmmDeleteExpr;
class CmmCastFormula;
class CmmBinaryExpr;
class CmmCondExpr;
class CmmStatSect;
class CmmStat;
class CmmDeclStat;
class CmmLabeledStat;
class CmmCaseStat;
class CmmDefaultStat;
class CmmExprStat;
class CmmEmptyStat;
class CmmCondition;
class CmmIfStat;
class CmmSwitchStat;
class CmmWhileStat;
class CmmDoStat;
class CmmForStat;
class CmmBreakStat;
class CmmContinueStat;
class CmmReturnStat;
class CmmGotoStat;
class CmmDeclSect;
class CmmCtorHead;
class CmmCtorDecl;
class CmmSpecSect;
class CmmFuncHead;
class CmmFuncDecl;
class CmmSimpleDecl;
class CmmInitDeclarator;
class CmmEmptyDecl;
class CmmDeclSpec;
class CmmTypeSect;
class CmmAllocTypeSpecSect;
class CmmAllocTypeSpec;
class CmmEnumDecl;
class CmmEnumSect;
class CmmEnumItem;
class CmmNamespace;
class CmmUsingDecl;
class CmmExternDecl;
class CmmAsmDecl;
class CmmTypeId;
class CmmCommonTypeId;
class CmmDeclarator;
class CmmAbstractDeclarator;
class CmmCommonDeclarator;
class CmmContDeclaratorSect;
class CmmContDeclarator;
class CmmContFunctionDeclarator;
class CmmContArrayDeclarator;
class CmmPtrSpecifierSect;
class CmmPtrSpecifier;
class CmmCvSpecifierSect;
class CmmCvSpecifier;
class CmmParamSect;
class CmmParamItem;
class CmmInitializer;
class CmmInitItem;
class CmmInitSimple;
class CmmInitList;
class CmmClassDecl;
class CmmBaseSect;
class CmmCtorInitializer;
class CmmMemberInitializer;
class CmmSpecialFuction;
class CmmTemplateDecl;
class CmmTemplateParamSect;
class CmmTemplateParam;
class CmmTemplateTypeParam;
class CmmTemplateExtraParam;
class CmmTemplateValueParam;
class CmmTemplateArgSect;
class CmmTemplateArg;
class CmmTemplateTypeArg;
class CmmTemplateValueArg;
class CmmTryStat;
class CmmHandlerSect;
class CmmHandlerItem;
class CmmExceptionDecl;
class CmmThrowExpr;
class CmmExceptionSpec;
class CmmIncludeName;
class CmmDirective;
class CmmDirectiveStat;
class CmmNamespaceExtension;
class CmmClusterStat;
class CmmIntoStat;
class CmmTextItem;
class CmmDefineStat;
class CmmGroupStat;
class CmmIndentStat;
class CmmWhenStat;
class CmmWhenItem;
class CmmCallStat;
class CmmSpaceStat;
class CmmLocationStat;
class AltName;
class AltSimple;
class AltBranch;
class AltExpr;
class DefItem;
class DefList;
class LogFactor;
class LogTerm;
class LogExpr;
class LogStat;
class NumStatSect;
class NumStat;
class NumCode;
class NumIndent;
class NumUnindent;
class NumWhen;
class NumStatAlternative;
class NumAssign;
class NumSelection;
class NumExprAlternative;
class NumCond;
class NumAndCond;
class NumCompCond;

class CmmBasic
{
  public:
    virtual string produce ();
    virtual string produce_subst () { return ""; }
    virtual void produce_attributes () {}
    virtual void produce_sequence () {}
    virtual string getTypeName () { return "CmmBasic"; }
    virtual int getFirstLocation () { return 0; }
    virtual bool isCustomized () { return false; }
    virtual bool isSimple () { return true; }
    virtual bool haveSubitems () { return false; }
    virtual bool isBinary (CmmBinaryExpr_kind param) { return false; }
    virtual ~ CmmBasic ();
};

class CmmDecl : public CmmBasic
{
  public:
    string comment;
    int group;
    bool one_line_comment;
    bool insert_empty_line;
    bool ins_empty_line;
    bool one_line_function;
    bool save_newlines;
    bool redefine;
    CmmAccess redefine_access;
    CmmAccess required_access;
    string group_comment;
    
    virtual CmmCtorDecl * conv_CmmCtorDecl () { return NULL; }
    virtual CmmFuncDecl * conv_CmmFuncDecl () { return NULL; }
    virtual CmmSimpleDecl * conv_CmmSimpleDecl () { return NULL; }
    virtual CmmEmptyDecl * conv_CmmEmptyDecl () { return NULL; }
    virtual CmmNamespace * conv_CmmNamespace () { return NULL; }
    virtual CmmUsingDecl * conv_CmmUsingDecl () { return NULL; }
    virtual CmmAsmDecl * conv_CmmAsmDecl () { return NULL; }
    virtual CmmTemplateDecl * conv_CmmTemplateDecl () { return NULL; }
    virtual CmmExternDecl * conv_CmmExternDecl () { return NULL; }
    virtual CmmDirective * conv_CmmDirective () { return NULL; }
    virtual CmmNamespaceExtension * conv_CmmNamespaceExtension () { return NULL; }
    virtual CmmMemberVisibility * conv_CmmMemberVisibility () { return NULL; }
    virtual CmmClassDecl * conv_CmmClassDecl () { return NULL; }
    virtual CmmEnumDecl * conv_CmmEnumDecl () { return NULL; }
    
    CmmDecl *next;
    
    CmmDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDecl"; }
    virtual bool isSimple () { return false; }
};

class CmmMemberVisibility : public CmmDecl
{
  public:
    CmmAccess access;
    
    virtual CmmMemberVisibility * conv_CmmMemberVisibility () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmMemberVisibility ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmMemberVisibility"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmBaseItem : public CmmBasic
{
  public:
    CmmAccess access;
    bool a_virtual;
    CmmNameExpr * from;
    CmmTemplateArgSect * template_args;
    
    CmmBaseItem *next;
    
    int location [4];
    string start_note [4];
    
    CmmBaseItem ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmBaseItem"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmSpec : public CmmBasic
{
  public:
    CmmSpecKind kind;
    
    CmmSpec *next;
    
    int location [15];
    string start_note [15];
    
    CmmSpec ();
    explicit CmmSpec (CmmSpecKind p_kind);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSpec"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmType : public CmmBasic
{
  public:
    CmmTypeKind kind;
    
    CmmType *next;
    
    int location [11];
    string start_note [11];
    
    CmmType ();
    explicit CmmType (CmmTypeKind p_kind);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmType"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmStat : public CmmBasic
{
  public:
    virtual CmmExprStat * conv_CmmExprStat () { return NULL; }
    virtual CmmDeclStat * conv_CmmDeclStat () { return NULL; }
    virtual CmmCompoundStat * conv_CmmCompoundStat () { return NULL; }
    virtual CmmLabeledStat * conv_CmmLabeledStat () { return NULL; }
    virtual CmmCaseStat * conv_CmmCaseStat () { return NULL; }
    virtual CmmDefaultStat * conv_CmmDefaultStat () { return NULL; }
    virtual CmmEmptyStat * conv_CmmEmptyStat () { return NULL; }
    virtual CmmIfStat * conv_CmmIfStat () { return NULL; }
    virtual CmmSwitchStat * conv_CmmSwitchStat () { return NULL; }
    virtual CmmWhileStat * conv_CmmWhileStat () { return NULL; }
    virtual CmmDoStat * conv_CmmDoStat () { return NULL; }
    virtual CmmForStat * conv_CmmForStat () { return NULL; }
    virtual CmmBreakStat * conv_CmmBreakStat () { return NULL; }
    virtual CmmContinueStat * conv_CmmContinueStat () { return NULL; }
    virtual CmmReturnStat * conv_CmmReturnStat () { return NULL; }
    virtual CmmGotoStat * conv_CmmGotoStat () { return NULL; }
    virtual CmmTryStat * conv_CmmTryStat () { return NULL; }
    virtual CmmDirectiveStat * conv_CmmDirectiveStat () { return NULL; }
    virtual CmmIntoStat * conv_CmmIntoStat () { return NULL; }
    virtual CmmDefineStat * conv_CmmDefineStat () { return NULL; }
    virtual CmmGroupStat * conv_CmmGroupStat () { return NULL; }
    virtual CmmIndentStat * conv_CmmIndentStat () { return NULL; }
    virtual CmmWhenStat * conv_CmmWhenStat () { return NULL; }
    virtual CmmCallStat * conv_CmmCallStat () { return NULL; }
    virtual CmmSpaceStat * conv_CmmSpaceStat () { return NULL; }
    virtual CmmLocationStat * conv_CmmLocationStat () { return NULL; }
    virtual CmmSuperStat * conv_CmmSuperStat () { return NULL; }
    virtual CmmClusterStat * conv_CmmClusterStat () { return NULL; }
    virtual CmmStat * conv_CmmStat () { return this; }
    
    CmmStat *next;
    
    CmmStat ();
    
    virtual string getTypeName () { return "CmmStat"; }
};

class CmmCompoundStat : public CmmStat
{
  public:
    CmmCompoundStat * link;
    CmmDefineSect * actual_dictionary;
    CmmExpr * actual_group;
    CmmExpr * actual_indent;
    CmmStatSect * body;
    
    virtual CmmCompoundStat * conv_CmmCompoundStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCompoundStat ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCompoundStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmDefineSect : public CmmBasic
{
  public:
    CmmDefineSect * link;
    
    CmmDefineItem *first;
    CmmDefineItem *last;
    void add (CmmDefineItem * item);
    void insert_first (CmmDefineItem * item);
    
    CmmDefineSect ();
    explicit CmmDefineSect (CmmDefineSect * p_link);
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmDefineSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmDefineItem : public CmmBasic
{
  public:
    CmmDefineItemMode mode;
    string name;
    CmmExpr * value;
    
    CmmDefineItem *next;
    
    int location [13];
    string start_note [13];
    
    CmmDefineItem ();
    CmmDefineItem (CmmDefineItemMode p_mode, string p_name, CmmExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDefineItem"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSuperStat : public CmmStat
{
  public:
    CmmClassDecl * cls;
    CmmExpr * param;
    
    virtual CmmSuperStat * conv_CmmSuperStat () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmSuperStat ();
    CmmSuperStat (CmmClassDecl * p_cls, CmmExpr * p_param);
    
    virtual string produce ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSuperStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isCustomized () { return true; }
};

class CmmTextSect : public CmmBasic
{
  public:
    int keyword_column;
    CmmExpr * into_param;
    CmmExpr * indent_expr;
    int shift_value;
    int first_column;
    bool start_line;
    string acc;
    int spaces;
    CmmExpr * indent;
    string shift;
    
    void store_position ();
    
    CmmTextItem *first;
    CmmTextItem *last;
    void add (CmmTextItem * item);
    void insert_first (CmmTextItem * item);
    
    int location [4];
    string start_note [4];
    
    CmmTextSect ();
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmTextSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmContName : public CmmBasic
{
  public:
    bool destructor_name;
    bool template_name;
    string id;
    
    CmmContName *next;
    
    int location [4];
    string start_note [4];
    
    CmmContName ();
    CmmContName (bool p_destructor_name, bool p_template_name, string p_id);
    
    virtual string produce_subst ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmContName"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSimpName : public CmmBasic
{
  public:
    string id;
    CmmSpecialFuction * special;
    
    int location [1];
    string start_note [1];
    
    CmmSimpName ();
    CmmSimpName (string p_id, CmmSpecialFuction * p_special);
    
    virtual string produce_subst ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSimpName"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmExpr : public CmmBasic
{
  public:
    virtual CmmValueExpr * conv_CmmValueExpr () { return NULL; }
    virtual CmmThisExpr * conv_CmmThisExpr () { return NULL; }
    virtual CmmNameExpr * conv_CmmNameExpr () { return NULL; }
    virtual CmmSubexprExpr * conv_CmmSubexprExpr () { return NULL; }
    virtual CmmModernCastExpr * conv_CmmModernCastExpr () { return NULL; }
    virtual CmmTypeidExpr * conv_CmmTypeidExpr () { return NULL; }
    virtual CmmExpr * conv_CmmExpr () { return this; }
    virtual CmmIndexExpr * conv_CmmIndexExpr () { return NULL; }
    virtual CmmCallExpr * conv_CmmCallExpr () { return NULL; }
    virtual CmmFieldExpr * conv_CmmFieldExpr () { return NULL; }
    virtual CmmPtrFieldExpr * conv_CmmPtrFieldExpr () { return NULL; }
    virtual CmmPostIncExpr * conv_CmmPostIncExpr () { return NULL; }
    virtual CmmPostDecExpr * conv_CmmPostDecExpr () { return NULL; }
    virtual CmmIncExpr * conv_CmmIncExpr () { return NULL; }
    virtual CmmDecExpr * conv_CmmDecExpr () { return NULL; }
    virtual CmmDerefExpr * conv_CmmDerefExpr () { return NULL; }
    virtual CmmAddrExpr * conv_CmmAddrExpr () { return NULL; }
    virtual CmmPlusExpr * conv_CmmPlusExpr () { return NULL; }
    virtual CmmMinusExpr * conv_CmmMinusExpr () { return NULL; }
    virtual CmmBitNotExpr * conv_CmmBitNotExpr () { return NULL; }
    virtual CmmLogNotExpr * conv_CmmLogNotExpr () { return NULL; }
    virtual CmmSizeofExp * conv_CmmSizeofExp () { return NULL; }
    virtual CmmNewExpr * conv_CmmNewExpr () { return NULL; }
    virtual CmmDeleteExpr * conv_CmmDeleteExpr () { return NULL; }
    virtual CmmCastFormula * conv_CmmCastFormula () { return NULL; }
    virtual CmmBinaryExpr * conv_CmmBinaryExpr () { return NULL; }
    virtual CmmCondExpr * conv_CmmCondExpr () { return NULL; }
    virtual CmmThrowExpr * conv_CmmThrowExpr () { return NULL; }
    
    CmmExpr *next;
    
    int location [1];
    string start_note [1];
    
    CmmExpr ();
    
    virtual string getTypeName () { return "CmmExpr"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmNameExpr : public CmmExpr
{
  public:
    CmmSimpName * head;
    
    virtual CmmNameExpr * conv_CmmNameExpr () { return this; }
    
    CmmContName *first;
    CmmContName *last;
    void add (CmmContName * item);
    void insert_first (CmmContName * item);
    
    CmmNameExpr ();
    explicit CmmNameExpr (CmmSimpName * p_head);
    
    virtual string produce_subst ();
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmNameExpr"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmValueExpr : public CmmExpr
{
  public:
    CmmValueExpr_kind kind;
    string value;
    
    virtual CmmValueExpr * conv_CmmValueExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmValueExpr ();
    CmmValueExpr (CmmValueExpr_kind p_kind, string p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmValueExpr"; }
};

class CmmThisExpr : public CmmExpr
{
  public:
    virtual CmmThisExpr * conv_CmmThisExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmThisExpr ();
    
    virtual string getTypeName () { return "CmmThisExpr"; }
};

class CmmSubexprExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmSubexprExpr * conv_CmmSubexprExpr () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmSubexprExpr ();
    explicit CmmSubexprExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSubexprExpr"; }
};

class CmmModernCastExpr : public CmmExpr
{
  public:
    CmmModernCastExpr_kind kind;
    CmmTypeId * type;
    CmmExpr * param;
    
    virtual CmmModernCastExpr * conv_CmmModernCastExpr () { return this; }
    
    int location [8];
    string start_note [8];
    
    CmmModernCastExpr ();
    CmmModernCastExpr (CmmModernCastExpr_kind p_kind, CmmTypeId * p_type, CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmModernCastExpr"; }
};

class CmmTypeidExpr : public CmmExpr
{
  public:
    CmmExpr * param1;
    CmmTypeId * param2;
    
    virtual CmmTypeidExpr * conv_CmmTypeidExpr () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmTypeidExpr ();
    CmmTypeidExpr (CmmExpr * p_param1, CmmTypeId * p_param2);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTypeidExpr"; }
};

class CmmIndexExpr : public CmmExpr
{
  public:
    CmmExpr * left;
    CmmExpr * right;
    
    virtual CmmIndexExpr * conv_CmmIndexExpr () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmIndexExpr ();
    CmmIndexExpr (CmmExpr * p_left, CmmExpr * p_right);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIndexExpr"; }
};

class CmmCallExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    CmmExprList * list;
    
    virtual CmmCallExpr * conv_CmmCallExpr () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCallExpr ();
    CmmCallExpr (CmmExpr * p_param, CmmExprList * p_list);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCallExpr"; }
};

class CmmFieldExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    bool destructor_name;
    bool template_name;
    CmmNameExpr * name;
    
    virtual CmmFieldExpr * conv_CmmFieldExpr () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmFieldExpr ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmFieldExpr"; }
    virtual bool isSimple () { return false; }
};

class CmmPtrFieldExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    bool destructor_name;
    bool template_name;
    CmmNameExpr * name;
    
    virtual CmmPtrFieldExpr * conv_CmmPtrFieldExpr () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmPtrFieldExpr ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmPtrFieldExpr"; }
    virtual bool isSimple () { return false; }
};

class CmmPostIncExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmPostIncExpr * conv_CmmPostIncExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmPostIncExpr ();
    explicit CmmPostIncExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmPostIncExpr"; }
};

class CmmPostDecExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmPostDecExpr * conv_CmmPostDecExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmPostDecExpr ();
    explicit CmmPostDecExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmPostDecExpr"; }
};

class CmmExprList : public CmmBasic
{
  public:
    CmmExpr *first;
    CmmExpr *last;
    void add (CmmExpr * item);
    void insert_first (CmmExpr * item);
    
    int location [1];
    string start_note [1];
    
    CmmExprList ();
    
    virtual string produce_subst ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmExprList"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmIncExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmIncExpr * conv_CmmIncExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmIncExpr ();
    explicit CmmIncExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIncExpr"; }
};

class CmmDecExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmDecExpr * conv_CmmDecExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmDecExpr ();
    explicit CmmDecExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDecExpr"; }
};

class CmmDerefExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmDerefExpr * conv_CmmDerefExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmDerefExpr ();
    explicit CmmDerefExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDerefExpr"; }
};

class CmmAddrExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmAddrExpr * conv_CmmAddrExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmAddrExpr ();
    explicit CmmAddrExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmAddrExpr"; }
};

class CmmPlusExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmPlusExpr * conv_CmmPlusExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmPlusExpr ();
    explicit CmmPlusExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmPlusExpr"; }
};

class CmmMinusExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmMinusExpr * conv_CmmMinusExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmMinusExpr ();
    explicit CmmMinusExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmMinusExpr"; }
};

class CmmBitNotExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmBitNotExpr * conv_CmmBitNotExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmBitNotExpr ();
    explicit CmmBitNotExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmBitNotExpr"; }
};

class CmmLogNotExpr : public CmmExpr
{
  public:
    CmmExpr * param;
    
    virtual CmmLogNotExpr * conv_CmmLogNotExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmLogNotExpr ();
    explicit CmmLogNotExpr (CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmLogNotExpr"; }
};

class CmmSizeofExp : public CmmExpr
{
  public:
    CmmExpr * param1;
    CmmTypeId * param2;
    
    virtual CmmSizeofExp * conv_CmmSizeofExp () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmSizeofExp ();
    CmmSizeofExp (CmmExpr * p_param1, CmmTypeId * p_param2);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSizeofExp"; }
};

class CmmNewExpr : public CmmExpr
{
  public:
    CmmAllocTypeSpecSect * spec;
    CmmTypeId * type2;
    CmmExprList * init;
    
    virtual CmmNewExpr * conv_CmmNewExpr () { return this; }
    
    CmmNewArrayLimit *first;
    CmmNewArrayLimit *last;
    void add (CmmNewArrayLimit * item);
    void insert_first (CmmNewArrayLimit * item);
    
    int location [5];
    string start_note [5];
    
    CmmNewExpr ();
    CmmNewExpr (CmmAllocTypeSpecSect * p_spec, CmmTypeId * p_type2, CmmExprList * p_init);
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmNewExpr"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmNewArrayLimit : public CmmBasic
{
  public:
    CmmExpr * value;
    
    CmmNewArrayLimit *next;
    
    int location [2];
    string start_note [2];
    
    CmmNewArrayLimit ();
    explicit CmmNewArrayLimit (CmmExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmNewArrayLimit"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDeleteExpr : public CmmExpr
{
  public:
    bool a_array;
    CmmExpr * param;
    
    virtual CmmDeleteExpr * conv_CmmDeleteExpr () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmDeleteExpr ();
    CmmDeleteExpr (bool p_a_array, CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDeleteExpr"; }
};

class CmmCastFormula : public CmmExpr
{
  public:
    CmmTypeId * type;
    CmmExpr * param;
    
    virtual CmmCastFormula * conv_CmmCastFormula () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCastFormula ();
    CmmCastFormula (CmmTypeId * p_type, CmmExpr * p_param);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCastFormula"; }
};

class CmmBinaryExpr : public CmmExpr
{
  public:
    CmmBinaryExpr_kind kind;
    CmmExpr * left;
    CmmExpr * right;
    
    virtual CmmBinaryExpr * conv_CmmBinaryExpr () { return this; }
    
    CmmBinaryExpr ();
    CmmBinaryExpr (CmmBinaryExpr_kind p_kind, CmmExpr * p_left, CmmExpr * p_right);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmBinaryExpr"; }
    virtual bool isBinary (CmmBinaryExpr_kind param) { return kind == param; }
};

class CmmCondExpr : public CmmExpr
{
  public:
    CmmExpr * cond_expr;
    CmmExpr * then_expr;
    CmmExpr * else_expr;
    
    virtual CmmCondExpr * conv_CmmCondExpr () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCondExpr ();
    CmmCondExpr (CmmExpr * p_cond_expr, CmmExpr * p_then_expr, CmmExpr * p_else_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCondExpr"; }
};

class CmmStatSect : public CmmBasic
{
  public:
    CmmStat *first;
    CmmStat *last;
    void add (CmmStat * item);
    void insert_first (CmmStat * item);
    
    CmmStatSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmStatSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmDeclStat : public CmmStat
{
  public:
    CmmDecl * decl;
    
    virtual CmmDeclStat * conv_CmmDeclStat () { return this; }
    
    CmmDeclStat ();
    explicit CmmDeclStat (CmmDecl * p_decl);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDeclStat"; }
};

class CmmLabeledStat : public CmmStat
{
  public:
    string lab;
    CmmStat * body;
    
    virtual CmmLabeledStat * conv_CmmLabeledStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmLabeledStat ();
    CmmLabeledStat (string p_lab, CmmStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmLabeledStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmCaseStat : public CmmStat
{
  public:
    CmmExpr * case_expr;
    CmmStat * body;
    
    virtual CmmCaseStat * conv_CmmCaseStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCaseStat ();
    CmmCaseStat (CmmExpr * p_case_expr, CmmStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCaseStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDefaultStat : public CmmStat
{
  public:
    CmmStat * body;
    
    virtual CmmDefaultStat * conv_CmmDefaultStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmDefaultStat ();
    explicit CmmDefaultStat (CmmStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDefaultStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmExprStat : public CmmStat
{
  public:
    CmmExpr * inner_expr;
    
    virtual CmmExprStat * conv_CmmExprStat () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmExprStat ();
    explicit CmmExprStat (CmmExpr * p_inner_expr);
    
    virtual string produce_subst ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmExprStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmEmptyStat : public CmmStat
{
  public:
    virtual CmmEmptyStat * conv_CmmEmptyStat () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmEmptyStat ();
    
    virtual string getTypeName () { return "CmmEmptyStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmCondition : public CmmBasic
{
  public:
    CmmExpr * cond_expr;
    
    CmmCondition ();
    explicit CmmCondition (CmmExpr * p_cond_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCondition"; }
};

class CmmIfStat : public CmmStat
{
  public:
    CmmCondition * cond;
    CmmStat * then_stat;
    CmmStat * else_stat;
    
    virtual CmmIfStat * conv_CmmIfStat () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmIfStat ();
    CmmIfStat (CmmCondition * p_cond, CmmStat * p_then_stat, CmmStat * p_else_stat);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIfStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSwitchStat : public CmmStat
{
  public:
    CmmCondition * cond;
    CmmStat * body;
    
    virtual CmmSwitchStat * conv_CmmSwitchStat () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmSwitchStat ();
    CmmSwitchStat (CmmCondition * p_cond, CmmStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSwitchStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmWhileStat : public CmmStat
{
  public:
    CmmCondition * cond;
    CmmStat * body;
    
    virtual CmmWhileStat * conv_CmmWhileStat () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmWhileStat ();
    CmmWhileStat (CmmCondition * p_cond, CmmStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmWhileStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDoStat : public CmmStat
{
  public:
    CmmStat * body;
    CmmExpr * cond_expr;
    
    virtual CmmDoStat * conv_CmmDoStat () { return this; }
    
    int location [5];
    string start_note [5];
    
    CmmDoStat ();
    CmmDoStat (CmmStat * p_body, CmmExpr * p_cond_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDoStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmForStat : public CmmStat
{
  public:
    CmmSimpleDecl * from_decl;
    CmmExpr * from_expr;
    bool empty_param;
    CmmExpr * to_expr;
    CmmExpr * cond_expr;
    CmmStat * body;
    
    virtual CmmForStat * conv_CmmForStat () { return this; }
    
    int location [6];
    string start_note [6];
    
    CmmForStat ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmForStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmBreakStat : public CmmStat
{
  public:
    virtual CmmBreakStat * conv_CmmBreakStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmBreakStat ();
    
    virtual string getTypeName () { return "CmmBreakStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmContinueStat : public CmmStat
{
  public:
    virtual CmmContinueStat * conv_CmmContinueStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmContinueStat ();
    
    virtual string getTypeName () { return "CmmContinueStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmReturnStat : public CmmStat
{
  public:
    CmmExpr * return_expr;
    
    virtual CmmReturnStat * conv_CmmReturnStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmReturnStat ();
    explicit CmmReturnStat (CmmExpr * p_return_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmReturnStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmGotoStat : public CmmStat
{
  public:
    string goto_lab;
    
    virtual CmmGotoStat * conv_CmmGotoStat () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmGotoStat ();
    explicit CmmGotoStat (string p_goto_lab);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmGotoStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDeclSect : public CmmBasic
{
  public:
    CmmDecl *first;
    CmmDecl *last;
    void add (CmmDecl * item);
    void insert_first (CmmDecl * item);
    
    CmmDeclSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmDeclSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmCtorHead : public CmmBasic
{
  public:
    CmmSpecSect * spec;
    bool destructor;
    CmmNameExpr * name;
    CmmContFunctionDeclarator * cont;
    CmmCtorInitializer * init;
    bool body;
    bool try_block;
    bool empty;
    
    int location [5];
    string start_note [5];
    
    CmmCtorHead ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCtorHead"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmCtorDecl : public CmmDecl
{
  public:
    CmmSpecSect * spec;
    bool destructor;
    CmmDeclarator * decl;
    CmmCtorInitializer * init;
    CmmCompoundStat * statements;
    CmmHandlerSect * handlers;
    bool empty;
    
    virtual CmmCtorDecl * conv_CmmCtorDecl () { return this; }
    
    int location [5];
    string start_note [5];
    
    CmmCtorDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCtorDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSpecSect : public CmmBasic
{
  public:
    CmmSpec *first;
    CmmSpec *last;
    void add (CmmSpec * item);
    void insert_first (CmmSpec * item);
    
    CmmSpecSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmSpecSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmFuncHead : public CmmBasic
{
  public:
    CmmDeclSpec * spec;
    CmmPtrSpecifierSect * ptr;
    CmmNameExpr * name;
    CmmContFunctionDeclarator * params;
    CmmContDeclaratorSect * cont;
    bool body;
    bool try_block;
    bool empty;
    
    int location [3];
    string start_note [3];
    
    CmmFuncHead ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmFuncHead"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmFuncDecl : public CmmDecl
{
  public:
    CmmDeclSpec * spec;
    CmmDeclarator * decl;
    CmmCompoundStat * statements;
    CmmHandlerSect * handlers;
    bool empty;
    
    virtual CmmFuncDecl * conv_CmmFuncDecl () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmFuncDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmFuncDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSimpleDecl : public CmmDecl
{
  public:
    CmmDeclSpec * spec;
    
    virtual CmmSimpleDecl * conv_CmmSimpleDecl () { return this; }
    
    CmmInitDeclarator *first;
    CmmInitDeclarator *last;
    void add (CmmInitDeclarator * item);
    void insert_first (CmmInitDeclarator * item);
    
    int location [4];
    string start_note [4];
    
    CmmSimpleDecl ();
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmSimpleDecl"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmInitDeclarator : public CmmBasic
{
  public:
    CmmDeclarator * decl;
    CmmInitializer * init;
    CmmExpr * width;
    
    CmmInitDeclarator *next;
    
    int location [1];
    string start_note [1];
    
    CmmInitDeclarator ();
    CmmInitDeclarator (CmmDeclarator * p_decl, CmmInitializer * p_init, CmmExpr * p_width);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmInitDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmEmptyDecl : public CmmDecl
{
  public:
    virtual CmmEmptyDecl * conv_CmmEmptyDecl () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmEmptyDecl ();
    
    virtual string getTypeName () { return "CmmEmptyDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDeclSpec : public CmmBasic
{
  public:
    CmmSpecSect * mode;
    CmmNameExpr * name;
    CmmTemplateArgSect * template_args;
    CmmClassDecl * class_spec;
    CmmEnumDecl * enum_spec;
    CmmTypeSect * type;
    CmmSpecSect * mode2;
    
    CmmDeclSpec ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDeclSpec"; }
    virtual bool isSimple () { return false; }
};

class CmmTypeSect : public CmmBasic
{
  public:
    CmmType *first;
    CmmType *last;
    void add (CmmType * item);
    void insert_first (CmmType * item);
    
    CmmTypeSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmTypeSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmAllocTypeSpecSect : public CmmBasic
{
  public:
    CmmAllocTypeSpec *first;
    CmmAllocTypeSpec *last;
    void add (CmmAllocTypeSpec * item);
    void insert_first (CmmAllocTypeSpec * item);
    
    CmmAllocTypeSpecSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmAllocTypeSpecSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmAllocTypeSpec : public CmmBasic
{
  public:
    bool a_const;
    bool a_volatile;
    CmmNameExpr * name;
    CmmClassDecl * class_spec;
    CmmEnumDecl * enum_spec;
    CmmType * simple;
    
    CmmAllocTypeSpec *next;
    
    int location [2];
    string start_note [2];
    
    CmmAllocTypeSpec ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmAllocTypeSpec"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmEnumDecl : public CmmDecl
{
  public:
    CmmNameExpr * name;
    CmmEnumSect * items;
    
    virtual CmmEnumDecl * conv_CmmEnumDecl () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmEnumDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmEnumDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmEnumSect : public CmmBasic
{
  public:
    CmmEnumItem *first;
    CmmEnumItem *last;
    void add (CmmEnumItem * item);
    void insert_first (CmmEnumItem * item);
    
    int location [1];
    string start_note [1];
    
    CmmEnumSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmEnumSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmEnumItem : public CmmBasic
{
  public:
    string name;
    CmmExpr * init;
    
    CmmEnumItem *next;
    
    int location [2];
    string start_note [2];
    
    CmmEnumItem ();
    CmmEnumItem (string p_name, CmmExpr * p_init);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmEnumItem"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmNamespace : public CmmDecl
{
  public:
    CmmDeclSect * unnamed_body;
    string namespace_name;
    CmmNameExpr * value;
    CmmDeclSect * body;
    
    virtual CmmNamespace * conv_CmmNamespace () { return this; }
    
    int location [8];
    string start_note [8];
    
    CmmNamespace ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmNamespace"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmUsingDecl : public CmmDecl
{
  public:
    bool a_namespace;
    bool a_typename;
    CmmNameExpr * name;
    
    virtual CmmUsingDecl * conv_CmmUsingDecl () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmUsingDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmUsingDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmExternDecl : public CmmDecl
{
  public:
    string language;
    CmmDeclSect * decl_list;
    CmmDecl * decl;
    
    virtual CmmExternDecl * conv_CmmExternDecl () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmExternDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmExternDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmAsmDecl : public CmmDecl
{
  public:
    string instruction;
    
    virtual CmmAsmDecl * conv_CmmAsmDecl () { return this; }
    
    int location [5];
    string start_note [5];
    
    CmmAsmDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmAsmDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmTypeId : public CmmBasic
{
  public:
    CmmDeclSpec * spec;
    CmmAbstractDeclarator * decl;
    
    CmmTypeId *next;
    
    CmmTypeId ();
    CmmTypeId (CmmDeclSpec * p_spec, CmmAbstractDeclarator * p_decl);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTypeId"; }
};

class CmmCommonTypeId : public CmmBasic
{
  public:
    CmmDeclSpec * spec;
    CmmCommonDeclarator * decl;
    
    CmmCommonTypeId ();
    CmmCommonTypeId (CmmDeclSpec * p_spec, CmmCommonDeclarator * p_decl);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCommonTypeId"; }
};

class CmmDeclarator : public CmmBasic
{
  public:
    CmmPtrSpecifierSect * ptr;
    CmmNameExpr * name;
    CmmDeclarator * inner;
    CmmContDeclaratorSect * cont;
    
    int location [2];
    string start_note [2];
    
    CmmDeclarator ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmAbstractDeclarator : public CmmBasic
{
  public:
    CmmPtrSpecifierSect * ptr;
    CmmAbstractDeclarator * inner;
    CmmContDeclaratorSect * cont;
    
    int location [2];
    string start_note [2];
    
    CmmAbstractDeclarator ();
    CmmAbstractDeclarator (CmmPtrSpecifierSect * p_ptr, CmmAbstractDeclarator * p_inner, CmmContDeclaratorSect * p_cont);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmAbstractDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmCommonDeclarator : public CmmBasic
{
  public:
    CmmPtrSpecifierSect * ptr;
    CmmNameExpr * name;
    CmmCommonDeclarator * inner;
    CmmContDeclaratorSect * cont;
    
    int location [2];
    string start_note [2];
    
    CmmCommonDeclarator ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCommonDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmContDeclaratorSect : public CmmBasic
{
  public:
    CmmContDeclarator *first;
    CmmContDeclarator *last;
    void add (CmmContDeclarator * item);
    void insert_first (CmmContDeclarator * item);
    
    CmmContDeclaratorSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmContDeclaratorSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmContDeclarator : public CmmBasic
{
  public:
    virtual CmmContFunctionDeclarator * conv_CmmContFunctionDeclarator () { return NULL; }
    virtual CmmContArrayDeclarator * conv_CmmContArrayDeclarator () { return NULL; }
    
    CmmContDeclarator *next;
    
    CmmContDeclarator ();
    
    virtual string getTypeName () { return "CmmContDeclarator"; }
};

class CmmContFunctionDeclarator : public CmmContDeclarator
{
  public:
    CmmParamSect * parameters;
    CmmCvSpecifierSect * cv_spec;
    CmmExceptionSpec * exception_spec;
    
    virtual CmmContFunctionDeclarator * conv_CmmContFunctionDeclarator () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmContFunctionDeclarator ();
    CmmContFunctionDeclarator (CmmParamSect * p_parameters, CmmCvSpecifierSect * p_cv_spec, CmmExceptionSpec * p_exception_spec);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmContFunctionDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmContArrayDeclarator : public CmmContDeclarator
{
  public:
    CmmExpr * lim;
    
    virtual CmmContArrayDeclarator * conv_CmmContArrayDeclarator () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmContArrayDeclarator ();
    explicit CmmContArrayDeclarator (CmmExpr * p_lim);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmContArrayDeclarator"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmPtrSpecifierSect : public CmmBasic
{
  public:
    CmmPtrSpecifier *first;
    CmmPtrSpecifier *last;
    void add (CmmPtrSpecifier * item);
    void insert_first (CmmPtrSpecifier * item);
    
    CmmPtrSpecifierSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmPtrSpecifierSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmPtrSpecifier : public CmmBasic
{
  public:
    CmmCvSpecifierSect * cv_spec;
    bool pointer;
    bool reference;
    
    CmmPtrSpecifier *next;
    
    int location [2];
    string start_note [2];
    
    CmmPtrSpecifier ();
    CmmPtrSpecifier (CmmCvSpecifierSect * p_cv_spec, bool p_pointer, bool p_reference);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmPtrSpecifier"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmCvSpecifierSect : public CmmBasic
{
  public:
    CmmCvSpecifier *first;
    CmmCvSpecifier *last;
    void add (CmmCvSpecifier * item);
    void insert_first (CmmCvSpecifier * item);
    
    CmmCvSpecifierSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmCvSpecifierSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmCvSpecifier : public CmmBasic
{
  public:
    bool a_const;
    bool a_volatile;
    
    CmmCvSpecifier *next;
    
    int location [2];
    string start_note [2];
    
    CmmCvSpecifier ();
    CmmCvSpecifier (bool p_a_const, bool p_a_volatile);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCvSpecifier"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmParamSect : public CmmBasic
{
  public:
    CmmParamItem *first;
    CmmParamItem *last;
    void add (CmmParamItem * item);
    void insert_first (CmmParamItem * item);
    
    int location [1];
    string start_note [1];
    
    CmmParamSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmParamSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmParamItem : public CmmBasic
{
  public:
    CmmCommonTypeId * type;
    CmmExpr * init;
    bool dots;
    
    CmmParamItem *next;
    
    int location [2];
    string start_note [2];
    
    CmmParamItem ();
    CmmParamItem (CmmCommonTypeId * p_type, CmmExpr * p_init, bool p_dots);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmParamItem"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmInitializer : public CmmBasic
{
  public:
    CmmInitItem * value;
    
    int location [1];
    string start_note [1];
    
    CmmInitializer ();
    explicit CmmInitializer (CmmInitItem * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmInitializer"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmInitItem : public CmmBasic
{
  public:
    virtual CmmInitSimple * conv_CmmInitSimple () { return NULL; }
    virtual CmmInitList * conv_CmmInitList () { return NULL; }
    
    CmmInitItem *next;
    
    CmmInitItem ();
    
    virtual string getTypeName () { return "CmmInitItem"; }
};

class CmmInitSimple : public CmmInitItem
{
  public:
    CmmExpr * expr;
    
    virtual CmmInitSimple * conv_CmmInitSimple () { return this; }
    
    CmmInitSimple ();
    explicit CmmInitSimple (CmmExpr * p_expr);
    
    virtual string produce_subst ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmInitSimple"; }
};

class CmmInitList : public CmmInitItem
{
  public:
    virtual CmmInitList * conv_CmmInitList () { return this; }
    
    CmmInitItem *first;
    CmmInitItem *last;
    void add (CmmInitItem * item);
    void insert_first (CmmInitItem * item);
    
    int location [3];
    string start_note [3];
    
    CmmInitList ();
    
    virtual string produce_subst ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmInitList"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmClassDecl : public CmmDecl
{
  public:
    CmmClassDecl_style style;
    CmmBaseSect * base_list;
    CmmDeclSect * members;
    bool anonymous;
    CmmNameExpr * name;
    
    virtual CmmClassDecl * conv_CmmClassDecl () { return this; }
    
    int location [12];
    string start_note [12];
    
    CmmClassDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmClassDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmBaseSect : public CmmBasic
{
  public:
    CmmBaseItem *first;
    CmmBaseItem *last;
    void add (CmmBaseItem * item);
    void insert_first (CmmBaseItem * item);
    
    int location [1];
    string start_note [1];
    
    CmmBaseSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmBaseSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmCtorInitializer : public CmmBasic
{
  public:
    CmmMemberInitializer *first;
    CmmMemberInitializer *last;
    void add (CmmMemberInitializer * item);
    void insert_first (CmmMemberInitializer * item);
    
    int location [1];
    string start_note [1];
    
    CmmCtorInitializer ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmCtorInitializer"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmMemberInitializer : public CmmBasic
{
  public:
    CmmNameExpr * name;
    CmmExprList * params;
    
    CmmMemberInitializer *next;
    
    int location [2];
    string start_note [2];
    
    CmmMemberInitializer ();
    CmmMemberInitializer (CmmNameExpr * p_name, CmmExprList * p_params);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmMemberInitializer"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmSpecialFuction : public CmmBasic
{
  public:
    bool spec_new;
    bool spec_delete;
    bool spec_add;
    bool spec_sub;
    bool spec_mul;
    bool spec_div;
    bool spec_mod;
    bool spec_xor;
    bool spec_and;
    bool spec_or;
    bool spec_not;
    bool spec_log_not;
    bool spec_eq;
    bool spec_lt;
    bool spec_gt;
    bool spec_add_assign;
    bool spec_sub_assign;
    bool spec_mul_assign;
    bool spec_div_assign;
    bool spec_mod_assign;
    bool spec_xor_assign;
    bool spec_and_assign;
    bool spec_or_assign;
    bool spec_shl;
    bool spec_shr;
    bool spec_shl_assign;
    bool spec_shr_assign;
    bool spec_ne;
    bool spec_le;
    bool spec_ge;
    bool spec_log_and;
    bool spec_log_or;
    bool spec_inc;
    bool spec_dec;
    bool spec_comma;
    bool spec_mebber_deref;
    bool spec_deref;
    bool spec_call;
    bool spec_index;
    
    int location [43];
    string start_note [43];
    
    CmmSpecialFuction ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmSpecialFuction"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmTemplateDecl : public CmmDecl
{
  public:
    bool export_template;
    CmmTemplateParamSect * params;
    bool explicit_specialization;
    bool parameter_block;
    bool explicit_instatiation;
    CmmDecl * body;
    
    virtual CmmTemplateDecl * conv_CmmTemplateDecl () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmTemplateDecl ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmTemplateParamSect : public CmmBasic
{
  public:
    CmmTemplateParam *first;
    CmmTemplateParam *last;
    void add (CmmTemplateParam * item);
    void insert_first (CmmTemplateParam * item);
    
    int location [1];
    string start_note [1];
    
    CmmTemplateParamSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmTemplateParamSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmTemplateParam : public CmmBasic
{
  public:
    virtual CmmTemplateExtraParam * conv_CmmTemplateExtraParam () { return NULL; }
    virtual CmmTemplateTypeParam * conv_CmmTemplateTypeParam () { return NULL; }
    virtual CmmTemplateValueParam * conv_CmmTemplateValueParam () { return NULL; }
    
    CmmTemplateParam *next;
    
    CmmTemplateParam ();
    
    virtual string getTypeName () { return "CmmTemplateParam"; }
};

class CmmTemplateTypeParam : public CmmTemplateParam
{
  public:
    bool a_class;
    bool a_typename;
    string name;
    CmmTypeId * value;
    
    virtual CmmTemplateTypeParam * conv_CmmTemplateTypeParam () { return this; }
    
    int location [4];
    string start_note [4];
    
    CmmTemplateTypeParam ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateTypeParam"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmTemplateExtraParam : public CmmTemplateParam
{
  public:
    CmmTemplateParamSect * params;
    string name;
    CmmNameExpr * value;
    
    virtual CmmTemplateExtraParam * conv_CmmTemplateExtraParam () { return this; }
    
    int location [6];
    string start_note [6];
    
    CmmTemplateExtraParam ();
    CmmTemplateExtraParam (CmmTemplateParamSect * p_params, string p_name, CmmNameExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateExtraParam"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmTemplateValueParam : public CmmTemplateParam
{
  public:
    CmmCommonTypeId * type;
    CmmExpr * init;
    
    virtual CmmTemplateValueParam * conv_CmmTemplateValueParam () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmTemplateValueParam ();
    CmmTemplateValueParam (CmmCommonTypeId * p_type, CmmExpr * p_init);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateValueParam"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmTemplateArgSect : public CmmBasic
{
  public:
    CmmTemplateArg *first;
    CmmTemplateArg *last;
    void add (CmmTemplateArg * item);
    void insert_first (CmmTemplateArg * item);
    
    int location [3];
    string start_note [3];
    
    CmmTemplateArgSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmTemplateArgSect"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmTemplateArg : public CmmBasic
{
  public:
    virtual CmmTemplateTypeArg * conv_CmmTemplateTypeArg () { return NULL; }
    virtual CmmTemplateValueArg * conv_CmmTemplateValueArg () { return NULL; }
    
    CmmTemplateArg *next;
    
    CmmTemplateArg ();
    
    virtual string getTypeName () { return "CmmTemplateArg"; }
};

class CmmTemplateTypeArg : public CmmTemplateArg
{
  public:
    CmmTypeId * type;
    
    virtual CmmTemplateTypeArg * conv_CmmTemplateTypeArg () { return this; }
    
    CmmTemplateTypeArg ();
    explicit CmmTemplateTypeArg (CmmTypeId * p_type);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateTypeArg"; }
};

class CmmTemplateValueArg : public CmmTemplateArg
{
  public:
    CmmExpr * value;
    
    virtual CmmTemplateValueArg * conv_CmmTemplateValueArg () { return this; }
    
    CmmTemplateValueArg ();
    explicit CmmTemplateValueArg (CmmExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTemplateValueArg"; }
};

class CmmTryStat : public CmmStat
{
  public:
    CmmCompoundStat * body;
    CmmHandlerSect * handlers;
    
    virtual CmmTryStat * conv_CmmTryStat () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmTryStat ();
    CmmTryStat (CmmCompoundStat * p_body, CmmHandlerSect * p_handlers);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTryStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmHandlerSect : public CmmBasic
{
  public:
    CmmHandlerItem *first;
    CmmHandlerItem *last;
    void add (CmmHandlerItem * item);
    void insert_first (CmmHandlerItem * item);
    
    CmmHandlerSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmHandlerSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmHandlerItem : public CmmBasic
{
  public:
    CmmExceptionDecl * exception;
    CmmCompoundStat * body;
    
    CmmHandlerItem *next;
    
    int location [3];
    string start_note [3];
    
    CmmHandlerItem ();
    CmmHandlerItem (CmmExceptionDecl * p_exception, CmmCompoundStat * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmHandlerItem"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmExceptionDecl : public CmmBasic
{
  public:
    CmmCommonTypeId * type;
    bool dots;
    
    int location [1];
    string start_note [1];
    
    CmmExceptionDecl ();
    CmmExceptionDecl (CmmCommonTypeId * p_type, bool p_dots);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmExceptionDecl"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmThrowExpr : public CmmExpr
{
  public:
    CmmExpr * expr;
    
    virtual CmmThrowExpr * conv_CmmThrowExpr () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmThrowExpr ();
    explicit CmmThrowExpr (CmmExpr * p_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmThrowExpr"; }
};

class CmmExceptionSpec : public CmmBasic
{
  public:
    CmmTypeId *first;
    CmmTypeId *last;
    void add (CmmTypeId * item);
    void insert_first (CmmTypeId * item);
    
    int location [4];
    string start_note [4];
    
    CmmExceptionSpec ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmExceptionSpec"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmIncludeName : public CmmBasic
{
  public:
    string include_q_str;
    string include_h_str;
    string include_id;
    
    int location [3];
    string start_note [3];
    
    CmmIncludeName ();
    CmmIncludeName (string p_include_q_str, string p_include_h_str, string p_include_id);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIncludeName"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDirective : public CmmDecl
{
  public:
    string d1;
    bool include_directive;
    CmmIncludeName * include_name;
    bool if_directive;
    string if_expr;
    bool ifdef_directive;
    string ifdef_name;
    bool ifndef_directive;
    string ifndef_name;
    bool else_directive;
    bool endif_directive;
    bool define_directive;
    string define_name;
    bool undef_directive;
    string undef_name;
    string d2;
    
    virtual CmmDirective * conv_CmmDirective () { return this; }
    
    int location [15];
    string start_note [15];
    
    CmmDirective ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDirective"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmDirectiveStat : public CmmStat
{
  public:
    string d1;
    bool include_directive;
    CmmIncludeName * include_name;
    bool if_directive;
    string if_expr;
    bool ifdef_directive;
    string ifdef_name;
    bool ifndef_directive;
    string ifndef_name;
    bool else_directive;
    bool endif_directive;
    bool define_directive;
    string define_name;
    bool undef_directive;
    string undef_name;
    string d2;
    
    virtual CmmDirectiveStat * conv_CmmDirectiveStat () { return this; }
    
    int location [15];
    string start_note [15];
    
    CmmDirectiveStat ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDirectiveStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmNamespaceExtension : public CmmDecl
{
  public:
    bool open;
    
    virtual CmmNamespaceExtension * conv_CmmNamespaceExtension () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmNamespaceExtension ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmNamespaceExtension"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmClusterStat : public CmmStat
{
  public:
    CmmStatSect * body;
    
    virtual CmmClusterStat * conv_CmmClusterStat () { return this; }
    
    CmmClusterStat ();
    explicit CmmClusterStat (CmmStatSect * p_body);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmClusterStat"; }
};

class CmmIntoStat : public CmmStat
{
  public:
    CmmExpr * into_param;
    CmmExpr * store_param;
    bool with_into;
    CmmExpr * group;
    CmmExpr * comment;
    CmmDefineSect * def;
    bool prior;
    CmmExpr * expr;
    CmmExprList * expr_list;
    CmmStat * stat;
    CmmStatSect * code;
    CmmDecl * decl;
    CmmDeclSect * decl_list;
    CmmParamSect * param_list;
    CmmEnumDecl * enum_type;
    CmmEnumSect * enum_items;
    CmmClassDecl * cls;
    CmmBaseSect * base_list;
    bool a_public;
    bool a_protected;
    bool a_private;
    CmmExpr * access;
    CmmDecl * member;
    CmmDeclSect * members;
    CmmCtorInitializer * ctor_init;
    CmmInitItem * init_item;
    CmmInitList * init_list;
    CmmTextSect * text_sect;
    
    virtual CmmIntoStat * conv_CmmIntoStat () { return this; }
    
    int location [28];
    string start_note [28];
    
    CmmIntoStat ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIntoStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmTextItem : public CmmBasic
{
  public:
    string token_value;
    bool token_flag;
    string space_value;
    bool white_space_flag;
    string new_line_value;
    bool new_line_flag;
    string other_value;
    bool other_flag;
    
    CmmTextItem *next;
    
    int location [4];
    string start_note [4];
    
    CmmTextItem ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmTextItem"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class CmmDefineStat : public CmmStat
{
  public:
    CmmDefineSect * def;
    
    virtual CmmDefineStat * conv_CmmDefineStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmDefineStat ();
    explicit CmmDefineStat (CmmDefineSect * p_def);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmDefineStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmGroupStat : public CmmStat
{
  public:
    CmmExpr * value;
    
    virtual CmmGroupStat * conv_CmmGroupStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmGroupStat ();
    explicit CmmGroupStat (CmmExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmGroupStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmIndentStat : public CmmStat
{
  public:
    CmmExpr * value;
    
    virtual CmmIndentStat * conv_CmmIndentStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmIndentStat ();
    explicit CmmIndentStat (CmmExpr * p_value);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmIndentStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmWhenStat : public CmmStat
{
  public:
    CmmExpr * cond_expr;
    CmmStatSect * then_body;
    CmmStatSect * else_body;
    
    virtual CmmWhenStat * conv_CmmWhenStat () { return this; }
    
    CmmWhenItem *first;
    CmmWhenItem *last;
    void add (CmmWhenItem * item);
    void insert_first (CmmWhenItem * item);
    
    int location [4];
    string start_note [4];
    
    CmmWhenStat ();
    CmmWhenStat (CmmExpr * p_cond_expr, CmmStatSect * p_then_body, CmmStatSect * p_else_body);
    
    virtual string produce ();
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "CmmWhenStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isCustomized () { return true; }
    virtual bool haveSubitems () { return first != NULL; }
};

class CmmWhenItem : public CmmBasic
{
  public:
    CmmExpr * sub_expr;
    CmmStatSect * sub_body;
    
    CmmWhenItem *next;
    
    int location [2];
    string start_note [2];
    
    CmmWhenItem ();
    CmmWhenItem (CmmExpr * p_sub_expr, CmmStatSect * p_sub_body);
    
    virtual string getTypeName () { return "CmmWhenItem"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmCallStat : public CmmStat
{
  public:
    CmmStatSect * body;
    
    virtual CmmCallStat * conv_CmmCallStat () { return this; }
    
    int location [2];
    string start_note [2];
    
    CmmCallStat ();
    explicit CmmCallStat (CmmStatSect * p_body);
    
    virtual string produce ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmCallStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isCustomized () { return true; }
};

class CmmSpaceStat : public CmmStat
{
  public:
    virtual CmmSpaceStat * conv_CmmSpaceStat () { return this; }
    
    int location [1];
    string start_note [1];
    
    CmmSpaceStat ();
    
    virtual string getTypeName () { return "CmmSpaceStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class CmmLocationStat : public CmmStat
{
  public:
    CmmExpr * place;
    
    virtual CmmLocationStat * conv_CmmLocationStat () { return this; }
    
    int location [3];
    string start_note [3];
    
    CmmLocationStat ();
    explicit CmmLocationStat (CmmExpr * p_place);
    
    virtual string produce ();
    virtual void produce_attributes ();
    virtual string getTypeName () { return "CmmLocationStat"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isCustomized () { return true; }
};

class AltName : public CmmBasic
{
  public:
    string id;
    string txt;
    
    int location [2];
    string start_note [2];
    
    AltName ();
    AltName (string p_id, string p_txt);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "AltName"; }
    virtual int getFirstLocation () { return location [0]; }
};

class AltSimple : public CmmBasic
{
  public:
    AltName * name;
    AltExpr * opt;
    AltExpr * sub_expr;
    
    AltSimple *next;
    
    int location [4];
    string start_note [4];
    
    AltSimple ();
    AltSimple (AltName * p_name, AltExpr * p_opt, AltExpr * p_sub_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "AltSimple"; }
    virtual int getFirstLocation () { return location [0]; }
};

class AltBranch : public CmmBasic
{
  public:
    AltBranch *next;
    AltSimple *first;
    AltSimple *last;
    void add (AltSimple * item);
    void insert_first (AltSimple * item);
    
    AltBranch ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "AltBranch"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class AltExpr : public CmmBasic
{
  public:
    AltBranch *first;
    AltBranch *last;
    void add (AltBranch * item);
    void insert_first (AltBranch * item);
    
    int location [1];
    string start_note [1];
    
    AltExpr ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "AltExpr"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class DefItem : public CmmBasic
{
  public:
    AltName * name;
    
    DefItem *next;
    
    DefItem ();
    explicit DefItem (AltName * p_name);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "DefItem"; }
};

class DefList : public CmmBasic
{
  public:
    DefItem *first;
    DefItem *last;
    void add (DefItem * item);
    void insert_first (DefItem * item);
    
    int location [1];
    string start_note [1];
    
    DefList ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "DefList"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class LogFactor : public CmmBasic
{
  public:
    AltName * name;
    LogFactor * not_factor;
    LogExpr * sub_expr;
    
    LogFactor *next;
    
    int location [3];
    string start_note [3];
    
    LogFactor ();
    LogFactor (AltName * p_name, LogFactor * p_not_factor, LogExpr * p_sub_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "LogFactor"; }
    virtual int getFirstLocation () { return location [0]; }
};

class LogTerm : public CmmBasic
{
  public:
    LogTerm *next;
    LogFactor *first;
    LogFactor *last;
    void add (LogFactor * item);
    void insert_first (LogFactor * item);
    
    int location [1];
    string start_note [1];
    
    LogTerm ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "LogTerm"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class LogExpr : public CmmBasic
{
  public:
    LogTerm *first;
    LogTerm *last;
    void add (LogTerm * item);
    void insert_first (LogTerm * item);
    
    int location [1];
    string start_note [1];
    
    LogExpr ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "LogExpr"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class LogStat : public CmmBasic
{
  public:
    AltName * name;
    LogExpr * expr;
    
    int location [1];
    string start_note [1];
    
    LogStat ();
    LogStat (AltName * p_name, LogExpr * p_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "LogStat"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumStatSect : public CmmBasic
{
  public:
    NumStat *first;
    NumStat *last;
    void add (NumStat * item);
    void insert_first (NumStat * item);
    
    NumStatSect ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "NumStatSect"; }
    virtual bool haveSubitems () { return first != NULL; }
};

class NumStat : public CmmBasic
{
  public:
    virtual NumAssign * conv_NumAssign () { return NULL; }
    virtual NumCode * conv_NumCode () { return NULL; }
    virtual NumWhen * conv_NumWhen () { return NULL; }
    virtual NumIndent * conv_NumIndent () { return NULL; }
    virtual NumUnindent * conv_NumUnindent () { return NULL; }
    
    NumStat *next;
    
    NumStat ();
    
    virtual string getTypeName () { return "NumStat"; }
};

class NumCode : public NumStat
{
  public:
    string code;
    
    virtual NumCode * conv_NumCode () { return this; }
    
    int location [3];
    string start_note [3];
    
    NumCode ();
    explicit NumCode (string p_code);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "NumCode"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumIndent : public NumStat
{
  public:
    virtual NumIndent * conv_NumIndent () { return this; }
    
    int location [1];
    string start_note [1];
    
    NumIndent ();
    
    virtual string getTypeName () { return "NumIndent"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumUnindent : public NumStat
{
  public:
    virtual NumUnindent * conv_NumUnindent () { return this; }
    
    int location [1];
    string start_note [1];
    
    NumUnindent ();
    
    virtual string getTypeName () { return "NumUnindent"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumWhen : public NumStat
{
  public:
    NumCond * cond;
    NumStatSect * when_stat;
    NumStatSect * otherwise_stat;
    
    virtual NumWhen * conv_NumWhen () { return this; }
    
    NumStatAlternative *first;
    NumStatAlternative *last;
    void add (NumStatAlternative * item);
    void insert_first (NumStatAlternative * item);
    
    int location [4];
    string start_note [4];
    
    NumWhen ();
    NumWhen (NumCond * p_cond, NumStatSect * p_when_stat, NumStatSect * p_otherwise_stat);
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "NumWhen"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class NumStatAlternative : public CmmBasic
{
  public:
    NumCond * cond;
    NumStatSect * stat;
    
    NumStatAlternative *next;
    
    int location [2];
    string start_note [2];
    
    NumStatAlternative ();
    NumStatAlternative (NumCond * p_cond, NumStatSect * p_stat);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "NumStatAlternative"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumAssign : public NumStat
{
  public:
    string id;
    CmmExprList * list;
    CmmExpr * expr;
    NumSelection * selection;
    
    virtual NumAssign * conv_NumAssign () { return this; }
    
    int location [5];
    string start_note [5];
    
    NumAssign ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "NumAssign"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool isSimple () { return false; }
};

class NumSelection : public CmmBasic
{
  public:
    NumCond * cond;
    CmmExpr * when_expr;
    CmmExpr * otherwise_expr;
    
    NumExprAlternative *first;
    NumExprAlternative *last;
    void add (NumExprAlternative * item);
    void insert_first (NumExprAlternative * item);
    
    int location [4];
    string start_note [4];
    
    NumSelection ();
    NumSelection (NumCond * p_cond, CmmExpr * p_when_expr, CmmExpr * p_otherwise_expr);
    
    virtual void produce_attributes ();
    virtual void produce_sequence ();
    virtual string getTypeName () { return "NumSelection"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class NumExprAlternative : public CmmBasic
{
  public:
    NumCond * cond;
    CmmExpr * expr;
    
    NumExprAlternative *next;
    
    int location [2];
    string start_note [2];
    
    NumExprAlternative ();
    NumExprAlternative (NumCond * p_cond, CmmExpr * p_expr);
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "NumExprAlternative"; }
    virtual int getFirstLocation () { return location [0]; }
};

class NumCond : public CmmBasic
{
  public:
    NumAndCond *first;
    NumAndCond *last;
    void add (NumAndCond * item);
    void insert_first (NumAndCond * item);
    
    int location [1];
    string start_note [1];
    
    NumCond ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "NumCond"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class NumAndCond : public CmmBasic
{
  public:
    NumAndCond *next;
    NumCompCond *first;
    NumCompCond *last;
    void add (NumCompCond * item);
    void insert_first (NumCompCond * item);
    
    int location [1];
    string start_note [1];
    
    NumAndCond ();
    
    virtual void produce_sequence ();
    virtual string getTypeName () { return "NumAndCond"; }
    virtual int getFirstLocation () { return location [0]; }
    virtual bool haveSubitems () { return first != NULL; }
};

class NumCompCond : public CmmExpr
{
  public:
    string left_id;
    bool equal;
    bool not_equal;
    string right_id;
    string right_val;
    bool relation;
    
    NumCompCond *next;
    
    int location [5];
    string start_note [5];
    
    NumCompCond ();
    
    virtual void produce_attributes ();
    virtual string getTypeName () { return "NumCompCond"; }
    virtual bool isSimple () { return false; }
};

void send_identifier (string p_item);
void send_cont_name (CmmContName * p_item);
void send_simple_name (CmmSimpName * p_item);
void send_id_expr (CmmNameExpr * p_item);
void send_primary_expr (CmmExpr * p_item);
void send_literal_expr (CmmValueExpr * p_item);
void send_literal (string p_item);
void send_this_expr (CmmThisExpr * p_item);
void send_subexpr_expr (CmmSubexprExpr * p_item);
void send_postfix_start (CmmExpr * p_item);
void send_modern_cast_expr (CmmModernCastExpr * p_item);
void send_typeid_expr (CmmTypeidExpr * p_item);
void send_postfix_expr (CmmExpr * p_item);
void send_index_expr (CmmIndexExpr * p_item);
void send_call_expr (CmmCallExpr * p_item);
void send_field_expr (CmmFieldExpr * p_item);
void send_ptr_field_expr (CmmPtrFieldExpr * p_item);
void send_post_inc_expr (CmmPostIncExpr * p_item);
void send_post_dec_expr (CmmPostDecExpr * p_item);
void send_expr_list (CmmExprList * p_item);
void send_unary_expr (CmmExpr * p_item);
void send_inc_expr (CmmIncExpr * p_item);
void send_dec_expr (CmmDecExpr * p_item);
void send_deref_expr (CmmDerefExpr * p_item);
void send_addr_expr (CmmAddrExpr * p_item);
void send_plus_expr (CmmPlusExpr * p_item);
void send_minus_expr (CmmMinusExpr * p_item);
void send_bit_not_expr (CmmBitNotExpr * p_item);
void send_log_not_expr (CmmLogNotExpr * p_item);
void send_sizeof_expr (CmmSizeofExp * p_item);
void send_allocation_expr (CmmNewExpr * p_item);
void send_allocation_array_limit (CmmNewArrayLimit * p_item);
void send_deallocation_expr (CmmDeleteExpr * p_item);
void send_cast_expr (CmmExpr * p_item);
void send_cast_formula (CmmCastFormula * p_item);
void send_pm_expr (CmmExpr * p_item);
void send_multiplicative_expr (CmmExpr * p_item);
void send_additive_expr (CmmExpr * p_item);
void send_shift_expr (CmmExpr * p_item);
void send_relational_expr (CmmExpr * p_item);
void send_equality_expr (CmmExpr * p_item);
void send_and_expr (CmmExpr * p_item);
void send_exclusive_or_expr (CmmExpr * p_item);
void send_inclusive_or_expr (CmmExpr * p_item);
void send_logical_and_expr (CmmExpr * p_item);
void send_logical_or_expr (CmmExpr * p_item);
void send_conditional_expr (CmmExpr * p_item);
void send_conditional_cont_expr (CmmCondExpr * p_item);
void send_assignment_expr (CmmExpr * p_item);
void send_simple_assignment_expr (CmmExpr * p_item);
void send_expr (CmmExpr * p_item);
void send_const_expr (CmmExpr * p_item);
void send_stat_list (CmmStatSect * p_item);
void send_stat (CmmStat * p_item);
void send_inner_stat (CmmStat * p_item);
void send_tail_stat (CmmStat * p_item);
void send_declaration_stat (CmmDeclStat * p_item);
void send_labeled_stat (CmmLabeledStat * p_item);
void send_case_stat (CmmCaseStat * p_item);
void send_default_stat (CmmDefaultStat * p_item);
void send_expression_stat (CmmExprStat * p_item);
void send_empty_stat (CmmEmptyStat * p_item);
void send_compound_stat (CmmCompoundStat * p_item);
void send_condition (CmmCondition * p_item);
void send_if_stat (CmmIfStat * p_item);
void send_switch_stat (CmmSwitchStat * p_item);
void send_while_stat (CmmWhileStat * p_item);
void send_do_stat (CmmDoStat * p_item);
void send_for_stat (CmmForStat * p_item);
void send_break_stat (CmmBreakStat * p_item);
void send_continue_stat (CmmContinueStat * p_item);
void send_return_stat (CmmReturnStat * p_item);
void send_goto_stat (CmmGotoStat * p_item);
void send_declaration_list (CmmDeclSect * p_item);
void send_declaration (CmmDecl * p_item);
void send_block_declaration (CmmDecl * p_item);
void send_constructor_declaration (CmmCtorDecl * p_item);
void send_constructor_decl_specifiers (CmmSpecSect * p_item);
void send_constructor_decl_specifier (CmmSpec * p_item);
void send_function_declaration (CmmFuncDecl * p_item);
void send_simple_declaration (CmmSimpleDecl * p_item);
void send_init_declarator (CmmInitDeclarator * p_item);
void send_empty_declaration (CmmEmptyDecl * p_item);
void send_decl_specifiers (CmmDeclSpec * p_item);
void send_simple_decl_specifiers (CmmSpecSect * p_item);
void send_simple_decl_specifier (CmmSpec * p_item);
void send_simple_type_specifiers (CmmTypeSect * p_item);
void send_simple_type_specifier (CmmType * p_item);
void send_alloc_type_specifier_list (CmmAllocTypeSpecSect * p_item);
void send_alloc_type_specifier (CmmAllocTypeSpec * p_item);
void send_enum_specifier (CmmEnumDecl * p_item);
void send_enum_list (CmmEnumSect * p_item);
void send_enumerator (CmmEnumItem * p_item);
void send_namespace_definition (CmmNamespace * p_item);
void send_using_declaration (CmmUsingDecl * p_item);
void send_linkage_specification (CmmExternDecl * p_item);
void send_string_literal (string p_item);
void send_asm_declaration (CmmAsmDecl * p_item);
void send_type_id (CmmTypeId * p_item);
void send_common_type_id (CmmCommonTypeId * p_item);
void send_declarator (CmmDeclarator * p_item);
void send_abstract_declarator (CmmAbstractDeclarator * p_item);
void send_common_declarator (CmmCommonDeclarator * p_item);
void send_cont_declarator_list (CmmContDeclaratorSect * p_item);
void send_cont_declarator (CmmContDeclarator * p_item);
void send_cont_function_declarator (CmmContFunctionDeclarator * p_item);
void send_cont_array_declarator (CmmContArrayDeclarator * p_item);
void send_declarator_id (CmmNameExpr * p_item);
void send_ptr_specifier_list (CmmPtrSpecifierSect * p_item);
void send_ptr_specifier (CmmPtrSpecifier * p_item);
void send_cv_specifier_list (CmmCvSpecifierSect * p_item);
void send_cv_specifier (CmmCvSpecifier * p_item);
void send_parameter_declaration_list (CmmParamSect * p_item);
void send_parameter_declaration (CmmParamItem * p_item);
void send_initializer (CmmInitializer * p_item);
void send_initializer_item (CmmInitItem * p_item);
void send_simple_initializer (CmmInitSimple * p_item);
void send_initializer_list (CmmInitList * p_item);
void send_class_specifier (CmmClassDecl * p_item);
void send_member_list (CmmDeclSect * p_item);
void send_member_item (CmmDecl * p_item);
void send_member_visibility (CmmMemberVisibility * p_item);
void send_member_declaration (CmmSimpleDecl * p_item);
void send_member_declarator (CmmInitDeclarator * p_item);
void send_base_specifier_list (CmmBaseSect * p_item);
void send_base_specifier (CmmBaseItem * p_item);
void send_ctor_initializer (CmmCtorInitializer * p_item);
void send_mem_initializer (CmmMemberInitializer * p_item);
void send_special_function (CmmSpecialFuction * p_item);
void send_template_declaration (CmmTemplateDecl * p_item);
void send_template_param_list (CmmTemplateParamSect * p_item);
void send_template_param (CmmTemplateParam * p_item);
void send_template_type_param (CmmTemplateTypeParam * p_item);
void send_template_extra_param (CmmTemplateExtraParam * p_item);
void send_template_value_param (CmmTemplateValueParam * p_item);
void send_template_arg_list (CmmTemplateArgSect * p_item);
void send_template_arg (CmmTemplateArg * p_item);
void send_template_type_arg (CmmTemplateTypeArg * p_item);
void send_template_value_arg (CmmTemplateValueArg * p_item);
void send_try_stat (CmmTryStat * p_item);
void send_handler_list (CmmHandlerSect * p_item);
void send_handler (CmmHandlerItem * p_item);
void send_exception_declaration (CmmExceptionDecl * p_item);
void send_throw_expr (CmmThrowExpr * p_item);
void send_exception_specification (CmmExceptionSpec * p_item);
void send_dir_start (string p_item);
void send_dir_end (string p_item);
void send_dir_ident (string p_item);
void send_dir_number (string p_item);
void send_dir_q_string (string p_item);
void send_dir_h_string (string p_item);
void send_include_name (CmmIncludeName * p_item);
void send_directive (CmmDirective * p_item);
void send_directive_stat (CmmDirectiveStat * p_item);
void send_namespace_extension (CmmNamespaceExtension * p_item);
void send_cluster_stat (CmmClusterStat * p_item);
void send_class_declaration (CmmClassDecl * p_item);
void send_enum_declaration (CmmEnumDecl * p_item);
void send_into_stat (CmmIntoStat * p_item);
void send_numeric_literal (string p_item);
void send_txt_token (string p_item);
void send_txt_white_space (string p_item);
void send_txt_new_line (string p_item);
void send_txt_other (string p_item);
void send_define_stat (CmmDefineStat * p_item);
void send_group_stat (CmmGroupStat * p_item);
void send_indent_stat (CmmIndentStat * p_item);
void send_when_stat (CmmWhenStat * p_item);
void send_when_item (CmmWhenItem * p_item);
void send_call_stat (CmmCallStat * p_item);
void send_space_stat (CmmSpaceStat * p_item);
void send_location_stat (CmmLocationStat * p_item);
void send_super_stat (CmmSuperStat * p_item);
void send_alt_name (AltName * p_item);
void send_alt_simple (AltSimple * p_item);
void send_alt_branch (AltBranch * p_item);
void send_alt_expression (AltExpr * p_item);
void send_alt_item (DefItem * p_item);
void send_alt_list (DefList * p_item);
void send_def_item (DefItem * p_item);
void send_def_list (DefList * p_item);
void send_log_factor (LogFactor * p_item);
void send_log_term (LogTerm * p_item);
void send_log_expression (LogExpr * p_item);
void send_log_statement (LogStat * p_item);
void send_num_statements (NumStatSect * p_item);
void send_num_statement (NumStat * p_item);
void send_num_code (NumCode * p_item);
void send_num_indent (NumIndent * p_item);
void send_num_unindent (NumUnindent * p_item);
void send_num_when (NumWhen * p_item);
void send_num_stat_alternative (NumStatAlternative * p_item);
void send_num_assign (NumAssign * p_item);
void send_num_selection (NumSelection * p_item);
void send_num_expr_alternative (NumExprAlternative * p_item);
void send_num_cond (NumCond * p_item);
void send_num_and_cond (NumAndCond * p_item);
void send_num_comp_cond (NumCompCond * p_item);

void open_simple_name (CmmSimpName * p_item);
void close_simple_name (CmmSimpName * p_item);
void open_compound_stat (CmmCompoundStat * p_item);
void close_compound_stat (CmmCompoundStat * p_item);
void open_declaration (CmmDecl * p_item);
void close_declaration (CmmDecl * p_item);
void open_constructor_declaration (CmmCtorDecl * p_item);
void close_constructor_declaration (CmmCtorDecl * p_item);
void open_function_declaration (CmmFuncDecl * p_item);
void close_function_declaration (CmmFuncDecl * p_item);
void open_member_list (CmmDeclSect * p_item);
void close_member_list (CmmDeclSect * p_item);
void open_member_item (CmmDecl * p_item);
void close_member_item (CmmDecl * p_item);

// external utilities

int getLocation ();
string getStartNote ();

void send (string s);
void sendLocation (int i);
void sendStartNote (string s);

void style_space ();
void style_no_space ();
void style_new_line ();
void style_empty_line ();
void style_no_empty_line ();
void style_indent ();
void style_unindent ();

void insert_item (CmmBasic * item);

void store_bool   (const string field, bool value);
void store_int    (const string field, int value);
void store_string (const string field, const string value);
void store_enum   (const string field, const string value, int index);
void store_ident  (const string field, const string value);
void store_item   (const string field, CmmBasic* item);

void store_location (int i);
} // end of namespace gibbon

#endif /* __PROG_CMM_HPP__ */
