
/* lex.g */

// whitespace
WS :
  ( ' '
  | '\t'
  | '\f'
  | ( '\r' '\n' // DOS
    | '\r'      // Macintosh
    | '\n'      // Unix
    )
    { newline(); }
  )
  ;

// single-line comments
SL_COMMENT :
  "//"
  ( ~('\n'|'\r') )*
  ('\n' | '\r' ('\n')? )
  { newline(); }
  ;

// multiple-line comments
ML_COMMENT :
  "/*"
  (
     ~('*'|'/'|'\r'|'\n')
    |
      ('/' ~('*')) => '/'
    |
      ('*' ~('/')) => '*'
    |
      '\r'
      (
         options { warnWhenFollowAmbig = false; }:
        '\n'
      )?
      { newline(); }
    |
      '\n'
      { newline(); }
  )*
  "*/"
  ;

// directive
DIRECTIVE :
  '#'
  { selector->push ("dir_lexer"); } // switch to lexer for directives
  ;

// character literals
CHAR_LITERAL :
  '\'' ( ESC | ~('\''|'\r'|'\n') ) '\''
  ;

// string literals
STRING_LITERAL :
  '"' ( ESC | ~('"'|'\\'|'\r'|'\n') )* '"'
  ;

// escape sequence
protected
ESC :
  '\\'
  (
      'a'
    | 'b'
    | 'f'
    | 'n'
    | 'r'
    | 't'
    | 'v'
    | '\''
    | '"'
    | '?'
    | '\\'
    |
      OCTAL_DIGIT
      (
         options { warnWhenFollowAmbig = false; }:
         OCTAL_DIGIT
         (
            options { warnWhenFollowAmbig = false; }:
            OCTAL_DIGIT
         )?
      )?
    |
      'x'
      HEX_DIGIT
      (
         options { warnWhenFollowAmbig = false; }:
         HEX_DIGIT
      )*
    |
      'u'
      HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
    |
      'U'
      HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
      HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
  )
  ;

// octal digit
protected
OCTAL_DIGIT :
  '0'..'7'
  ;

// identifier
IDENT
  options { testLiterals=true; } :
  ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
  {
      if (text[0] == 'T' && text == "TEXT_SECTION")
        selector->push ("txt_lexer"); // switch to text lexer
  }
  ;

// stencil parameter
MARK_LITERAL :
  '$' ('0'..'9') ('0'..'9')*
   ;

// a numeric literal
NUMBER :
    '0' ('x'|'X') HEX_DIGIT (HEX_DIGIT)* (INTEGER_SUFFIX)*
  | DIGIT (DIGIT)* ( '.' (DIGIT)* )? (EXPONENT)? (SUFFIX)*
  // | '.' DIGIT (DIGIT)* (EXPONENT)? (FLOAT_SUFFIX)*
  ;

// decimal digit
protected
DIGIT :
  '0'..'9'
  ;

// hexadecimal digit
protected
HEX_DIGIT :
  '0'..'9'|'A'..'F'|'a'..'f'
  ;

protected
EXPONENT :
  ('e'|'E') ('+'|'-')? DIGIT (DIGIT)*
  ;

protected
INTEGER_SUFFIX :
  'u'|'U'|'l'|'L'
  ;

protected
FLOAT_SUFFIX :
  'f'|'F'|'l'|'L'
  ;

protected
SUFFIX :
  'u'|'U'|'f'|'F'|'l'|'L'
  ;
