
/* minicpp.g */

options
{
   product_name = CMM; // output file name prefix
   product_namespace = gibbon;

   lexer_body = "lex.g";
   test_literals = true; 
   
   notes = true; // duplicate comments and spaces
   only_start_notes = true;
   
   /* 
   rule_prefix = rule_ ;
   type_prefix = type_ ;
   
   field_prefix = field_ ;
   list_prefix = queue_ ;
   internal_prefix = internal_ ;

   send_func_prefix = send_fce_ ;
   open_func_prefix = open_fce_ ;
   close_func_prefix = close_fce_ ;
   
   param_prefix = parameter_ ;
   temp_prefix = temporary_ ;
   local_prefix = local_ ;
   variable_prefix = variable_ ;

   conv_func_prefix = convert_ ;
   util_prefix = utility_ ;
   */
      
   basic_type = CmmBasic;
   expr_type = CmmExpr;

   binary_expr_type     = CmmBinaryExpr;
   binary_expr_left     = left;
   binary_expr_right    = right;
   binary_expr_selector = kind;

   unary_expr_type     = CmmUnaryExpr;
   unary_expr_param    = param;
   unary_expr_selector = kind;
}

/* --------------------------------------------------------------------- */ 

grammar_options
{
  language=Cpp;
  namespaceAntlr = "antlr::";
  namespaceStd = "std::";
  // genHashLines = false;
}

lexer_options
{
  // charVocabulary = '\0'..'\377';
  testLiterals = false;
  k = 3;
  caseSensitive = true;
  caseSensitiveLiterals = true;

  importVocab = CMMDirLexer; // connect to CMMDirLexer
  // exportVocab = CMMLexer;

  // codeGenMakeSwitchThreshold=2;
  // codeGenBitsetTestThreshold=3;
}

parser_options
{
  k = 2;
  defaultErrorHandler = false;

  // codeGenMakeSwitchThreshold=2;
  // codeGenBitsetTestThreshold=3;
}

/* --------------------------------------------------------------------- */

external_tokens // tokens defined in dirlex.h and txtlex.g
{
  "include" ;
  "if" ;
  "ifdef" ;
  "ifndef" ;
  "else" ;
  "endif" ;
  "define" ;
  "undef" ;
  "END";
}

/* --------------------------------------------------------------------- */ 

literals
{
   LPAREN     : '(' ;
   RPAREN     : ')' ;
   LBRACKET   : '[' ;
   RBRACKET   : ']' ;
   LBRACE     : '{' ;
   RBRACE     : '}' ;

   DOT        : '.'  ;
   ARROW      : "->" ;
   SCOPE      : "::" ;

   LOG_NOT    : '!' ;
   BIT_NOT    : '~' ;

   INC        : "++"  ;
   DEC        : "--"  ;

   DOT_STAR   : ".*"  ;
   ARROW_STAR : "->*" ;

   STAR       : '*' ;
   SLASH      : '/' ;
   MOD        : '%' ;

   PLUS       : '+' ;
   MINUS      : '-' ;

   SHL        : "<<" ;
   SHR        : ">>" ;

   LESS             : '<'  ;
   GREATER          : '>'  ;
   LESS_OR_EQUAL    : "<=" ;
   GREATER_OR_EQUAL : ">=" ;

   EQUAL            : "==" ;
   UNEQUAL          : "!=" ;

   BIT_AND    : '&' ;
   BIT_XOR    : '^' ;
   BIT_OR     : '|' ;

   LOG_AND    : "&&" ;
   LOG_OR     : "||" ;

   ASSIGN         : '='   ;
   MUL_ASSIGN     : "*="  ;
   DIV_ASSIGN     : "/="  ;
   MOD_ASSIGN     : "%="  ;
   PLUS_ASSIGN    : "+="  ;
   MINUS_ASSIGN   : "-="  ;
   SHL_ASSIGN     : "<<=" ;
   SHR_ASSIGN     : ">>=" ;
   BIT_AND_ASSIGN : "&="  ;
   BIT_XOR_ASSIGN : "^="  ;
   BIT_OR_ASSIGN  : "|="  ;

   QUESTION   : '?' ;
   COMMA      : ',' ;

   COLON      : ':' ;
   SEMICOLON  : ';' ;
   DOTS       : "..." ;
}

/* --------------------------------------------------------------------- */ 

// CmmAccess type shared by CmmMemberSect and CmmBaseItem

enum CmmAccess 
{
  PrivateAccess,
  ProtectedAccess,
  PublicAccess
};

struct CmmMemberVisibility
{
  CmmAccess access;
};

struct CmmBaseItem
{
  CmmAccess access;
};

// enumeration types

enum CmmSpecKind { };

struct CmmSpec
{
  CmmSpecKind kind;
};

enum CmmTypeKind { };

struct CmmType
{
  CmmTypeKind kind;
};

// extensions

struct CmmDecl
{
  string comment;

  int group;
  
  bool one_line_comment; // true => comment beginning with "//"
  
  bool insert_empty_line; // empty line before and after declaration
  bool ins_empty_line; // empty line after section
  
  bool one_line_function; // true => function declaration on one line
  bool save_newlines; // save skip_newlines variable

  bool redefine; // true => change visibility
  CmmAccess redefine_access; // change to this value

  // temporary variables for sorting members
  CmmAccess required_access;
  string group_comment; // comment before visibility specification
};

struct CmmCompoundStat
{
  CmmCompoundStat link; // outer compund statement
  
  CmmDefineSect actual_dictionary; // store define section
  CmmExpr actual_group; // group number from GROUP statement
  CmmExpr actual_indent; // indent value from INDENT statement
};

struct CmmDefineSect
{
  CmmDefineSect link; // outer define section
};

enum CmmDefineItemMode { };

struct CmmDefineItem
{
  CmmDefineItemMode mode;
};

struct CmmSuperStat
{
  CmmClassDecl cls; // outer class
};

struct CmmTextSect
{
  int keyword_column; // TEXT_SECTION column number

  CmmExpr into_param; // duplicated value from CmmIntoStat
  
  CmmExpr indent_expr; // value from indent parameter or from INDENT statement
  int shift_value; // value from SHIFT parameter
  int first_column; // first column from first line
  
  bool start_line;
  string acc; // accumulated text
  int spaces; // number of spaces after acc
};

/* --------------------------------------------------------------------- */ 

class CMMParser;

/* --------------------------------------------------------------------- */

identifier <primitive> <identifier> :
  IDENT ;

cont_name <CmmContName> <custom subst>:
  <no_space> "::" <no_space>
  (
     '~' <no_space> <set destructor_name = true>
  |
     "template" <set template_name = true>
  )? 
  id:identifier ;

simple_name <CmmSimpName> <custom subst> <custom open_close>:
   ( id: identifier | special:special_function );

id_expr <CmmNameExpr:CmmExpr> <custom subst>:
   // ( "::" <set global=true> )? // !? @@
   // ( id: identifier | special:special_function )
   head:simple_name
   ( <add> cont_name )*
   // ( template_params:template_arg_list )? // !? @@
   ;

/* --------------------------------------------------------------------- */

/* primary expression */

primary_expr <select CmmExpr>:
    literal_expr
  | this_expr
  | id_expr
  | subexpr_expr
  ;

literal_expr <CmmValueExpr:CmmExpr>:
  <set kind = ValueExp>
  value:literal;

literal <primitive>:
    NUMBER
  | CHAR_LITERAL
  | STRING_LITERAL ;

this_expr <CmmThisExpr:CmmExpr>:
  "this";

subexpr_expr <CmmSubexprExpr:CmmExpr>:
  '(' param:expr ')';

/* postfix expression */

postfix_start <select CmmExpr>:
    modern_cast_expr
  | typeid_expr
  | primary_expr ; // last - important !?

modern_cast_expr <CmmModernCastExpr:CmmExpr>:
  ( "dynamic_cast"       <set kind = DynamicCast>
  | "static_cast"        <set kind = StaticCast>
  | "const_cast"         <set kind = ConstCast>
  | "reinterpret_cast"   <set kind = ReinterpreterCast> )
  '<' <no_space> type:type_id <no_space> '>'
  '(' param:expr ')'
  ;

typeid_expr <CmmTypeidExpr:CmmExpr>:
  "typeid"
  '('
  (
     (expr) => param1:expr
  |
     param2:type_id
  )
  ')'
  ;

postfix_expr <choose CmmExpr>:
  postfix_start
  (
    index_expr |
    call_expr |
    field_expr |
    ptr_field_expr |
    post_inc_expr |
    post_dec_expr
  )* ;

index_expr <CmmIndexExpr:CmmExpr>:
  <store left:CmmExpr>
  '[' right:expr ']' ;

call_expr <CmmCallExpr:CmmExpr>:
  <store param:CmmExpr>
  '(' list:expr_list ')' ;

field_expr <CmmFieldExpr:CmmExpr>:
  <store param:CmmExpr>
  <no_space> '.' <no_space>
  ( '~' <no_space> <set destructor_name = true> |
    "template" <set template_name = true> )?
  name:id_expr ;

ptr_field_expr <CmmPtrFieldExpr:CmmExpr>:
  <store param:CmmExpr>
  <no_space> "->" <no_space>
  ( '~' <no_space> <set destructor_name = true> |
    "template" <set template_name = true> )?
  name:id_expr ;

post_inc_expr <CmmPostIncExpr:CmmExpr>:
  <store param:CmmExpr>
  "++" ;

post_dec_expr <CmmPostDecExpr:CmmExpr>:
  <store param:CmmExpr>
  "--" ;

expr_list <CmmExprList> <custom subst>:
  ( <add> assignment_expr (',' <add> assignment_expr)* )? ;

/* unary expression */

unary_expr <select CmmExpr>:
    inc_expr
  | dec_expr
  | deref_expr
  | addr_expr
  | plus_expr
  | minus_expr
  | bit_not_expr
  | log_not_expr
  | sizeof_expr
  | allocation_expr
  | deallocation_expr
  | postfix_expr ; // last - important !?

inc_expr <CmmIncExpr:CmmExpr>:
  "++" param:cast_expr;

dec_expr <CmmDecExpr:CmmExpr>:
  "--" param:cast_expr;

deref_expr <CmmDerefExpr:CmmExpr>:
  '*' param:cast_expr;

addr_expr <CmmAddrExpr:CmmExpr>:
  '&' param:cast_expr;

plus_expr <CmmPlusExpr:CmmExpr>:
  '+' param:cast_expr;

minus_expr  <CmmMinusExpr:CmmExpr>:
  '-' param:cast_expr;

bit_not_expr <CmmBitNotExpr:CmmExpr>:
  '~' param:cast_expr;

log_not_expr <CmmLogNotExpr:CmmExpr>:
  '!' param:cast_expr;

sizeof_expr <CmmSizeofExp:CmmExpr>:
  "sizeof"
  (
     (unary_expr) => param1:unary_expr
  |
     '(' param2:type_id ')'
  );

allocation_expr <CmmNewExpr:CmmExpr>:
  // ( "::" <set a_global = true> )? // !? @@
  "new"
  (
     spec:alloc_type_specifier_list
     // ptr:ptr_specifier_list // !? @@
     ( <add> allocation_array_limit )*
  |
    '(' type2:type_id ')'
  )
  ( '(' init:expr_list ')' )?
  ;

allocation_array_limit <CmmNewArrayLimit>:
  '[' value:expr ']'  ;

deallocation_expr <CmmDeleteExpr:CmmExpr>:
  // ( "::" <set a_global = true> )? // !? @@
  "delete"
  ( '[' ']' <set a_array = true> )?
  param:cast_expr ;

/* cast expression */
  
// cast_expr <return CmmExpr>:
//    unary_expr ;

cast_expr <select CmmExpr>:
    ( cast_formula ) => cast_formula // !?
  | simple:unary_expr ;

cast_formula <CmmCastFormula: CmmExpr>:
  '(' type:type_id ')' param:cast_expr ;

/* pm expression */
  
pm_expr <binary>:
  cast_expr
  ".*"  DotMemberExp
  "->*" ArrowMemverExp ;

/* arithmetic and logical expressions */

multiplicative_expr <binary>:
  pm_expr
  '*' MulExp
  '/' DivExp
  '%' ModExp ;

additive_expr <binary>:
  multiplicative_expr
  '+' AddExp
  '-' SubExp ;

shift_expr <binary>:
  additive_expr
  "<<" ShlExp
  ">>" ShrExp ;

relational_expr <binary>:
  shift_expr
  '<' LtExp
  '>' GtExp
  "<=" LeExp
  ">=" GeExp ;

equality_expr <binary>:
  relational_expr
  "==" EqExp
  "!=" NeExp ;

and_expr <binary>:
  equality_expr
  '&' BitAndExp ;

exclusive_or_expr <binary>:
  and_expr
  '^' BitXorExp ;

inclusive_or_expr <binary>:
  exclusive_or_expr
  '|' BitOrExp ;

logical_and_expr <binary>:
  inclusive_or_expr
  "&&" LogAndExp ;

logical_or_expr <binary>:
  logical_and_expr
  "||" LogOrExp ;

/* conditional expression */  

conditional_expr <choose CmmExpr>:
  logical_or_expr
  ( conditional_cont_expr )? ;

conditional_cont_expr <new CmmCondExpr:CmmExpr>:
  <store cond_expr:CmmExpr>
  '?' then_expr:expr
  ':' else_expr:conditional_expr ; // !?

/* assignment expression */

assignment_expr <select CmmExpr>:
   simple_assignment_expr |
   throw_expr;

simple_assignment_expr <binary>:
  conditional_expr // !?
  '='   AssignExp
  "+="  AddAssignExp
  "-="  SubAssignExp
  "*="  MulAssignExp
  "/="  DivAssignExp
  "%="  ModAssignExp
  "<<=" ShlAssignExp
  ">>=" ShrAssignExp
  "&="  AndAssignExp
  "^="  XorAssignExp
  "|="  OrAssignExp ;

/* expression */

expr <binary>:
  assignment_expr
  ',' CommaExp ;

/* constant expression */  

const_expr <return CmmExpr>:
  conditional_expr ;

/* --------------------------------------------------------------------- */

stat_list <CmmStatSect>:
   ( <add> stat )* ;

stat <select CmmStat>:

  <with_indentation>

     ( expression_stat ) => expression_stat |
     ( declaration_stat ) => declaration_stat |
     
     compound_stat |
     labeled_stat |
     case_stat |
     default_stat |
     empty_stat |
     if_stat |
     switch_stat |
     while_stat |
     do_stat |
     for_stat |
     break_stat |
     continue_stat |
     return_stat |
     goto_stat |
     try_stat |
     directive_stat |

     into_stat |
     define_stat |
     group_stat |
     indent_stat |
     when_stat |
     call_stat |
     space_stat |
     location_stat |
     super_stat 

  <without_indentation>
     <: additional cluster_stat :> ;

/* inner_stat and tail_stat - statements with different output indentation */

inner_stat <select CmmStat>:
  <with_new_lines>
     ( '{' ) =>  compound_stat |
  <without_indentation>
     stat;

tail_stat <select CmmStat>:
  <without_indentation>
     ( "if" ) => if_stat |
  <with_new_lines>
     ( '{' ) =>  compound_stat |
  <without_indentation>
     stat;

/* --------------------------------------------------------------------- */

declaration_stat <CmmDeclStat:CmmStat>:
  decl:block_declaration ;

labeled_stat <CmmLabeledStat:CmmStat>:
  lab:identifier ':' body:stat ;

case_stat <CmmCaseStat:CmmStat>:
  "case" case_expr:expr ':' body:inner_stat;

default_stat <CmmDefaultStat:CmmStat>:
  "default" ':' body:inner_stat;

expression_stat <CmmExprStat:CmmStat> <custom subst>:
  inner_expr:expr ';';

empty_stat <CmmEmptyStat:CmmStat>:
  ';' ;

compound_stat <CmmCompoundStat:CmmStat> <custom open_close>:
  '{'
  <no_empty_line>
  body:stat_list
  <no_empty_line>
  '}';

condition <CmmCondition>:
  // (simple_declaration) => cond_decl:simple_declaration | // !? @@
  cond_expr:expr ;


if_stat <CmmIfStat:CmmStat>:
  "if" '(' cond:condition ')' then_stat:inner_stat
  (
    <silent>
    "else" else_stat:tail_stat
  )? ;

switch_stat <CmmSwitchStat:CmmStat>:
  "switch" '(' cond:condition ')' body:inner_stat;

while_stat  <CmmWhileStat:CmmStat>:
  "while" '(' cond:condition ')' body:inner_stat; 

do_stat <CmmDoStat:CmmStat>:
  "do" body:inner_stat "while" '(' cond_expr:expr ')' ';';

for_stat <CmmForStat:CmmStat>:
  "for"
  '('
  (
     (simple_declaration) => from_decl:simple_declaration
  |
     from_expr:expr ';'
  |
     ';' <set empty_param = true>
  )
  (to_expr:expr)?
  ';'
  (cond_expr:expr)?
  ')'
  body:inner_stat;

break_stat <CmmBreakStat:CmmStat>:
  "break" ';';

continue_stat <CmmContinueStat:CmmStat>:
  "continue" ';';

return_stat <CmmReturnStat:CmmStat>:
  "return" (return_expr:expr)? ';';

goto_stat <CmmGotoStat:CmmStat>:
  "goto" goto_lab:identifier ';';

/* ---------------------------------------------------------------------- */

declaration_list <CmmDeclSect>:
  ( <add> declaration <new_line> )* ;

declaration <select CmmDecl> <custom open_close>:

    ( constructor_head ) => constructor_declaration 
  | ( function_head ) => function_declaration
  | simple_declaration
  | empty_declaration
  
  | namespace_definition
  | using_declaration
  | asm_declaration 
  
  | template_declaration
  | linkage_specification
  
  | directive
  | namespace_extension

  <: additional member_visibility :> 
  <: additional class_declaration :> 
  <: additional enum_declaration :> ;

block_declaration <select CmmDecl>:

    simple_declaration
  | empty_declaration
  | namespace_definition
  | using_declaration
  | asm_declaration ;

/* constructor declaration */
  
constructor_head <CmmCtorHead> <custom no_send>: 
  spec:constructor_decl_specifiers
  ( '~' <set destructor=true> )?

  name:declarator_id
  cont:cont_function_declarator
  
  ( ':' init: ctor_initializer )?
  ( '{' <set body=true>
  | "try" <set try_block=true>
  | ';' <set empty=true> )
  ;

constructor_declaration <CmmCtorDecl:CmmDecl> <custom open_close>:
  spec:constructor_decl_specifiers
  ( '~' <set destructor=true> )?
  decl:declarator
  (
     ( ':' init:ctor_initializer )?
     <new_line>
     statements:compound_stat
  |
     <new_line>
     "try"
     ( ':' init:ctor_initializer )?
     <new_line>
     statements:compound_stat
     handlers:handler_list
  |
     ';' <set empty=true>
  );

constructor_decl_specifiers <CmmSpecSect>:
  ( <add> constructor_decl_specifier )* ;

constructor_decl_specifier <CmmSpec>:
  "inline"   <set kind = a_inline>   |
  "virtual"  <set kind = a_virtual>  |
  "explicit" <set kind = a_explicit> ;

/* function declaration */  

function_head <CmmFuncHead> <custom no_send>:
  spec:decl_specifiers
  
  // decl:declarator
  
  ptr:ptr_specifier_list
  name:declarator_id // !?
  params:cont_function_declarator
  cont:cont_declarator_list

  (
     <new_line>
     '{' <set body=true>
  |
     <new_line>
     "try" <set try_block = true>
  |
     ';' <set empty=true>
  ) ;

function_declaration <CmmFuncDecl:CmmDecl> <custom open_close>:
  spec:decl_specifiers
  decl:declarator
  (
     <new_line>
     statements:compound_stat
  |
     <new_line>
     "try"
     statements:compound_stat
     handlers:handler_list
  |
     ';' <set empty=true>
  );

/* simple declaration */

simple_declaration <CmmSimpleDecl:CmmDecl>:
  spec:decl_specifiers
  (
     <add> init_declarator
     ( ',' <add> init_declarator )*
  )?
  ';' ;

init_declarator <CmmInitDeclarator>:
  decl:declarator
  ( init:initializer )? 
  // ( '=' init:conditional_expr )? // !?
  ;

/* empty declaration */

empty_declaration <CmmEmptyDecl:CmmDecl> <custom send>:
  ';' ;

/* ---------------------------------------------------------------------- */

/* declaration specifiers */

decl_specifiers <CmmDeclSpec>: // !?
  mode:simple_decl_specifiers

  ( name:id_expr
    ( template_args:template_arg_list )? // !?
  | 
    class_spec:class_specifier
  |
    enum_spec:enum_specifier
  |
    type:simple_type_specifiers
  )

  mode2:simple_decl_specifiers ;

simple_decl_specifiers <CmmSpecSect>:
  ( <add> simple_decl_specifier )* ;

simple_decl_specifier <CmmSpec>:
  "auto"     <set kind = a_auto>     |
  "register" <set kind = a_register> |
  "static"   <set kind = a_static>   |
  "extern"   <set kind = a_extern>   |
  "mutable"  <set kind = a_mutable>  |

  "inline"   <set kind = a_inline>   |
  "virtual"  <set kind = a_virtual>  |
  "explicit" <set kind = a_explicit> |

  "friend"   <set kind = a_friend>   |
  "typedef"  <set kind = a_typedef>  |

  "const"    <set kind = a_const>    |
  "volatile" <set kind = a_volatile> ;

simple_type_specifiers <CmmTypeSect>:
  <add> simple_type_specifier
  ( <add> simple_type_specifier )* ;

simple_type_specifier <CmmType>:
  "char"     <set kind = a_char>     |
  "wchar_t"  <set kind = a_wchar>    |
  "bool"     <set kind = a_bool>     |
  "short"    <set kind = a_short>    |
  "int"      <set kind = a_int>      |
  "long"     <set kind = a_long>     |
  "signed"   <set kind = a_signed>   |
  "unsigned" <set kind = a_unsigned> |
  "float"    <set kind = a_float>    |
  "double"   <set kind = a_double>   |
  "void"     <set kind = a_void>     ;

/* ---------------------------------------------------------------------- */

alloc_type_specifier_list <CmmAllocTypeSpecSect>:
  <add> alloc_type_specifier
  ( <add> alloc_type_specifier )* ; // !? @@

alloc_type_specifier <CmmAllocTypeSpec>:
  "const" <set a_const = true> |
  "volatile" <set a_volatile = true> |

  name:id_expr | // !?
  class_spec:class_specifier |
  enum_spec:enum_specifier |

  simple:simple_type_specifier ;

/* ---------------------------------------------------------------------- */

/* enum */

enum_specifier <CmmEnumDecl:CmmDecl>:
  "enum"
  ( name:id_expr )? // !?
  '{' items:enum_list '}' ;

enum_list <CmmEnumSect>:
  ( <add> enumerator ( ',' <add> enumerator )* )?;

enumerator <CmmEnumItem>:
  name:identifier ( '=' init:const_expr )? ;

/* namespace */

namespace_definition <CmmNamespace:CmmDecl> :
  "namespace"
  (
     '{' unnamed_body:declaration_list '}'
  |
     namespace_name:identifier // !?
     (
        '=' value:id_expr ';'
     |
        '{' body:declaration_list '}'
     )
  );

/* using */

using_declaration <CmmUsingDecl:CmmDecl>:
  "using"
  (
     "namespace" <set a_namespace = true>
  |
     "typename"  <set a_typename  = true>
  )?
  name: id_expr
  ';'
  ;

/* extern "..." */

linkage_specification <CmmExternDecl:CmmDecl>:
  "extern"
  language:string_literal
  (
    '{' decl_list:declaration_list '}'
  |
    decl:declaration
  );

string_literal <primitive>:
  STRING_LITERAL ;

/* asm declaration */

asm_declaration <CmmAsmDecl:CmmDecl>:
  "asm" '(' instruction:string_literal ')' ';' ;

/* ---------------------------------------------------------------------- */

/* type id */

type_id <CmmTypeId>:
  spec:decl_specifiers
  decl:abstract_declarator
  ;

common_type_id <CmmCommonTypeId>:
  spec:decl_specifiers
  decl:common_declarator
  ;

/* declarator */

declarator <CmmDeclarator>:
  ptr:ptr_specifier_list 
  (
     name:declarator_id
  |
     '(' inner:declarator ')'
  )
  cont:cont_declarator_list
  ;

abstract_declarator <CmmAbstractDeclarator>:
  ptr:ptr_specifier_list 
  (
     '(' inner:abstract_declarator ')' // !? @@
      cont:cont_declarator_list
  )?
  ;

common_declarator <CmmCommonDeclarator>:
  ptr:ptr_specifier_list 
  (
     (
       name:declarator_id
     |
       '(' inner:common_declarator ')'
     )
     cont:cont_declarator_list
  )?
  ;

cont_declarator_list <CmmContDeclaratorSect>:
  ( <add> cont_declarator )*
  ;

cont_declarator <select CmmContDeclarator>:
  cont_function_declarator
  | cont_array_declarator ;

cont_function_declarator <CmmContFunctionDeclarator:CmmContDeclarator>:
  '(' parameters:parameter_declaration_list ')'
  cv_spec:cv_specifier_list
  ( exception_spec:exception_specification )? ;

cont_array_declarator <CmmContArrayDeclarator:CmmContDeclarator>:
  '[' (lim:expr)? ']' ;

declarator_id <return CmmNameExpr>:
  id_expr ;

ptr_specifier_list <CmmPtrSpecifierSect>:
  ( <add> ptr_specifier )* ;

ptr_specifier <CmmPtrSpecifier>:
   '*' cv_spec:cv_specifier_list <set pointer=true> 
  | '&' <set reference=true>
//   |  (id_expr "::" '*') =>
//      name2:id_expr "::" '*'
//     cv_spec2:cv_specifier_list // !? @@
  ;

cv_specifier_list <CmmCvSpecifierSect>:
  ( <add> cv_specifier )*;

cv_specifier <CmmCvSpecifier>:
  "const" <set a_const = true> |
  "volatile" <set a_volatile = true> ;

/* parameter declarations */

parameter_declaration_list <CmmParamSect>:
  (
     <add> parameter_declaration
     ( ',' <add> parameter_declaration )*
  )? ;

parameter_declaration <CmmParamItem>:
  (
     type:common_type_id
     ( '=' init:assignment_expr )?
  |
     "..." <set dots=true> // !? @@
  );

/* initializer */

initializer <CmmInitializer>:
   '=' value:initializer_item ;

initializer_item <select CmmInitItem>:
     simple_initializer
   | initializer_list ;

simple_initializer <CmmInitSimple:CmmInitItem> <custom subst>:
    expr:assignment_expr ;

initializer_list <CmmInitList:CmmInitItem> <custom subst>:
    '{'
    <indent>
    (
      <add> initializer_item
      (
        ','
        <new_line>
        <add> initializer_item
      )*
      // ( ',' <set additional_comma=true> )? // !? @@
    )?
    <unindent>
    '}';

/* ---------------------------------------------------------------------- */

/* class */

class_specifier <CmmClassDecl:CmmDecl> :
  ( "class"  <set style = ClassStyle>
  | "struct" <set style = StructStyle>
  | "union"  <set style = UnionStyle> )
  (
    ( ':' base_list:base_specifier_list )?
     <new_line>
    '{'
     <indent>
     members:member_list
     <unindent>
    '}' 
    <set anonymous=true>
  |
     name:id_expr // !?
     // ( template_args:template_arg_list )? // !? @@
     ( 
        (':') =>
           ':' base_list:base_specifier_list
            <new_line>
           '{'
            <indent>
            members:member_list
            <unindent>
           '}' 
     |
        <new_line>
       '{'
        <indent>
        members:member_list
        <unindent>
       '}' 
     )?
  );

/* members */

member_list <CmmDeclSect> <custom open_close>:
  ( <add> member_item <new_line> )* ;

member_item <select CmmDecl> <custom open_close>:
  member_visibility |
  ( constructor_head ) => <indent> constructor_declaration <unindent> |
  ( function_head ) => <indent> function_declaration <unindent> |
  <indent> member_declaration <unindent> |
  <indent> empty_declaration <unindent> |
  <indent> using_declaration <unindent> |
  <indent> template_declaration <unindent> ;

member_visibility <CmmMemberVisibility:CmmDecl>:
  ( "private"   <set access = PrivateAccess>
  | "protected" <set access = ProtectedAccess>
  | "public"    <set access = PublicAccess> )
  <no_space> ':' ;
  
member_declaration <CmmSimpleDecl:CmmDecl>:
  spec:decl_specifiers
  (
     <add> member_declarator
     ( ',' <add> member_declarator )*
  )?
  ';' ;

member_declarator <CmmInitDeclarator>:
  decl:declarator
  ( init:initializer | ':' width:conditional_expr )? // !?
  // ( '=' init:conditional_expr | ':' width:conditional_expr )? // !?
  // ( init:initializer )? // !?
  // ( '=' init:conditional_expr )? // !?
  ;
  
/* base specification */

base_specifier_list <CmmBaseSect>:
  <add> base_specifier ( ',' <add> base_specifier )* ;

base_specifier <CmmBaseItem>:
  ("virtual" <set a_virtual = true> )?

  ( "private"   <set access = PrivateAccess>
  | "protected" <set access = ProtectedAccess>
  | "public"    <set access = PublicAccess>
  |             <set access = NoAccess> )

  // ("virtual" <set a_virtual2=true> )?

  from:id_expr 
  ( template_args:template_arg_list )? // !? 
  ;

/* constructor initializer */

ctor_initializer <CmmCtorInitializer>:
  <indent>
  <add> mem_initializer
  ( ',' <new_line> <add> mem_initializer )*
  <unindent> ;

mem_initializer <CmmMemberInitializer>:
   name:id_expr '(' params:expr_list ')' ;

/* ---------------------------------------------------------------------- */

/* special member functions */

special_function <CmmSpecialFuction>:
   "operator"
   (
     // !? @@
     // spec:type_specifier_list
     // ptr:ptr_specifier_list

     "new"    <set spec_new=true>    // ( "[" "]" <set spec_new_array=true> )?
   | "delete" <set spec_delete=true> // ( "[" "]" <set spec_delete_array=true> )?

   |  "+" <set spec_add=true>
   |  "-" <set spec_sub=true>
   |  "*" <set spec_mul=true>
   |  "/" <set spec_div=true>
   |  "%" <set spec_mod=true>
   |  "^" <set spec_xor=true>
   |  "&" <set spec_and=true>
   |  "|" <set spec_or=true>
   |  "~" <set spec_not=true>

   |  "!"  <set spec_log_not=true>
   |  "="  <set spec_eq=true>
   |  "<"  <set spec_lt=true>
   |  ">"  <set spec_gt=true>
   |  "+=" <set spec_add_assign=true>
   |  "-=" <set spec_sub_assign=true>
   |  "*=" <set spec_mul_assign=true>
   |  "/=" <set spec_div_assign=true>
   |  "%=" <set spec_mod_assign=true>

   |  "^="  <set spec_xor_assign=true>
   |  "&="  <set spec_and_assign=true>
   |  "|="  <set spec_or_assign=true>
   |  "<<"  <set spec_shl=true>
   |  ">>"  <set spec_shr=true>
   |  ">>=" <set spec_shl_assign=true>
   |  "<<=" <set spec_shr_assign=true>
   |  "=="  <set spec_eq=true>
   |  "!="  <set spec_ne=true>
   
   |  "<="  <set spec_le=true>
   |  ">="  <set spec_ge=true>
   |  "&&"  <set spec_log_and=true>
   |  "||"  <set spec_log_or=true>
   |  "++"  <set spec_inc=true>
   |  "--"  <set spec_dec=true>
   |  ","   <set spec_comma=true>
   |  "->*" <set spec_mebber_deref=true>
   |  "->"  <set spec_deref=true>
   
   |  "(" ")" <set spec_call=true>
   |  "[" "]" <set spec_index=true>
   ) ;

/* ---------------------------------------------------------------------- */

/* template declaration */

template_declaration <CmmTemplateDecl:CmmDecl>:
  ( "export" <set export_template = true > )?
  "template"
  (
     '<'
     ( params:template_param_list | <set explicit_specialization = true>)
     '>'
     <set parameter_block = true> // !?
  |
     <set explicit_instatiation = true>
  )
  body:declaration ;

template_param_list <CmmTemplateParamSect>:
  <add> template_param ( ',' <add> template_param )* ;

template_param <select CmmTemplateParam>:
    (template_extra_param) => template_extra_param // !?
  | (template_type_param) => template_type_param 
  | template_value_param ;

template_type_param <CmmTemplateTypeParam:CmmTemplateParam>:
  ( "class" <set a_class = true> |
    "typename" <set a_typename = true> )
  ( name:identifier )?
  ( '=' value:type_id )? ;

template_extra_param <CmmTemplateExtraParam:CmmTemplateParam>:
  "template" '<' params:template_param_list '>' "class" 
  ( name:identifier )?
  ( '=' value:id_expr )? ;

template_value_param <CmmTemplateValueParam:CmmTemplateParam>:
  type:common_type_id 
  ( '=' init:shift_expr )? ; // !?

/* template arguments */

template_arg_list <CmmTemplateArgSect>:
   '<' ( <add> template_arg ( ',' <add> template_arg )* )? '>';

template_arg <select CmmTemplateArg>:
   (type_id) => template_type_arg |
   template_value_arg;

template_type_arg <CmmTemplateTypeArg:CmmTemplateArg>:
   type:type_id;

template_value_arg <CmmTemplateValueArg:CmmTemplateArg>:
   value:shift_expr; // !?

/* ---------------------------------------------------------------------- */

/* try */

try_stat <CmmTryStat:CmmStat>:
  "try"
  body:compound_stat
  handlers:handler_list;

handler_list <CmmHandlerSect>:
  <add> handler (<add> handler)* ;

handler <CmmHandlerItem>: 
  "catch"
  '('
  exception:exception_declaration
  ')'
  body:compound_stat ;

exception_declaration <CmmExceptionDecl>:
    type:common_type_id 
  | '...' <set dots=true>
  ;

throw_expr <CmmThrowExpr:CmmExpr>: 
  "throw" (expr:assignment_expr) ? ;
  
exception_specification <CmmExceptionSpec>:
  "throw"
  '('
  ( <add> type_id (',' <add> type_id)* )?
  ')' ;

/* ---------------------------------------------------------------------- */

/* directives */

dir_start <primitive> <custom send>: // import foreign token
  DIRECTIVE ;

dir_end <primitive> <custom send>:
  DIR_NEW_LINE ;

dir_ident <primitive> <identifier> :
  DIR_IDENT ;

dir_number <primitive>:
  DIR_NUMBER ;

dir_q_string <primitive>:
  DIR_Q_STRING;

dir_h_string <primitive>:
  DIR_H_STRING;

include_name <CmmIncludeName> :
  include_q_str:dir_q_string |
  include_h_str:dir_h_string |
  include_id:dir_ident ; // enable substitution

directive <CmmDirective:CmmDecl>:
  d1:dir_start // store hash mark
  ( "include" <set include_directive = true> include_name:include_name |
    "if"      <set if_directive = true>      if_expr:dir_number        |
    "ifdef"   <set ifdef_directive = true>   ifdef_name:dir_ident      |
    "ifndef"  <set ifndef_directive = true>  ifndef_name:dir_ident     |
    "else"    <set else_directive = true>                              |
    "endif"   <set endif_directive = true>                             |
    "define"  <set define_directive = true>  define_name:dir_ident     |
    "undef"   <set undef_directive = true>   undef_name:dir_ident      )
  d2:dir_end;

directive_stat <CmmDirectiveStat:CmmStat>:
  d1:dir_start // store hash mark
  ( "include" <set include_directive = true> include_name:include_name |
    "if"      <set if_directive = true>      if_expr:dir_number        |
    "ifdef"   <set ifdef_directive = true>   ifdef_name:dir_ident      |
    "ifndef"  <set ifndef_directive = true>  ifndef_name:dir_ident     |
    "else"    <set else_directive = true>                              |
    "endif"   <set endif_directive = true>                             |
    "define"  <set define_directive = true>  define_name:dir_ident     |
    "undef"   <set undef_directive = true>   undef_name:dir_ident      )
  d2:dir_end;

/* ---------------------------------------------------------------------- */

/* namespace extension */

namespace_extension <CmmNamespaceExtension:CmmDecl>:
  "OPEN_NAMESPACE" <set open=true> |
  "CLOSE_NAMESPACE" <set open=false> ;

/* ---------------------------------------------------------------------- */

/* cluster statement */

cluster_stat <CmmClusterStat:CmmStat>:  
  ( <silent> body:stat_list )? ;

/* ---------------------------------------------------------------------- */

/* into */

class_declaration <return CmmClassDecl>:
   class_specifier ';' ;

enum_declaration <return CmmEnumDecl>:
   enum_specifier ';' ;

into_stat <CmmIntoStat:CmmStat> <custom send>:
  (
     "INTO"
     ( into_param:expr )?
     ( "STORE" store_param:expr )?
     <set with_into=true>
  |
     "STORE" store_param:expr 
  )   
  
  ( "GROUP" group:expr )?
  ( "COMMENT" comment:expr )?
  ( "DEFINE" def:define_part )?
  ( "PRIOR" <set prior = true> )?

  (
     "EXPRESSION" expr:expr
  |
     "EXPRESSIONS" expr_list:expr_list
  |
     "STATEMENT" stat:stat
  |
     "CODE" code:stat_list
  |
     "DECLARATION" decl:declaration
  |
     "DECLARATIONS" decl_list:declaration_list
  |
     "PARAMETERS" param_list:parameter_declaration_list 
  |
     "ENUMERATION_TYPE" enum_type:enum_declaration
  |
     "ENUMERATION_ITEMS" enum_items:enum_list
  |
     "CLASS" cls:class_declaration
  |
     "BASE" base_list:base_specifier_list
  |
     ( "PUBLIC"    <set a_public = true>    |
       "PROTECTED" <set a_protected = true> |
       "PRIVATE"   <set a_private = true>   |
       "ACCESS"    access:expr              )?
     "MEMBER" member:member_item
  |
     "MEMBERS" members:member_list 
  |
     "INITIALIZERS" ctor_init:ctor_initializer
  |
     "INITIALIZER_ITEM" init_item:initializer_item
  |
     "INITIALIZER_LIST" init_list:initializer_list
  |
     text_sect:text_section
  )

  "END";

/* text section */ 

text_section <CmmTextSect> <custom no_send> :
  ( "INDENT" indent:expr )?
  ( "SHIFT" shift:numeric_literal )?
  <invoke store_position>
  "TEXT_SECTION"
  ( <add> text_item )* ;

numeric_literal <primitive>:
    NUMBER;
    
text_item <CmmTextItem> <custom no_send> :
  token_value:    txt_token        <set token_flag = true>       |
  space_value:    txt_white_space  <set white_space_flag = true> |
  new_line_value: txt_new_line     <set new_line_flag = true>    |
  other_value:    txt_other        <set other_flag = true>       ;

txt_token <primitive> : // import foreign lexical elements
   TXT_TOKEN;

txt_white_space <primitive> :
   TXT_WHITE_SPACE;

txt_new_line <primitive> :
   TXT_NEW_LINE;

txt_other <primitive> :
   TXT_OTHER;

/* define */

define_stat <CmmDefineStat:CmmStat> <custom send> :
  "DEFINE"
  def:define_part
  "END" ;

define_part <CmmDefineSect> <custom no_send> :
  ( <add> define_item )* ;

define_item <CmmDefineItem> <custom no_send> :
  (
      name:identifier
      ":"
      ( "IDENT"     <set mode=ident_mode> |
        "TEXT"      <set mode=text_mode> |
        "STRING"    <set mode=string_mode> |
        "EXPR"      <set mode=expr_mode> |
        "EXPR_LIST" <set mode=expr_list_mode> |
        "STAT"      <set mode=stat_mode> |
        "DECL"      <set mode=decl_mode> |
        "INIT_ITEM" <set mode=init_item_mode> |
        "INIT_LIST" <set mode=init_list_mode> )
      ( '=' value:expr )?
  )
  ";" ;

/* group */

group_stat <CmmGroupStat:CmmStat> <custom send> :
  "GROUP"
  value:expr
  "END" ;
  
/* indent */

indent_stat <CmmIndentStat:CmmStat> <custom send> :
  "INDENT"
  value:expr
  "END" ;

/* when */

when_stat <CmmWhenStat:CmmStat> <custom produce>:
  "WHEN" cond_expr:expr "DO" then_body:stat_list
  ( <add> when_item )*
  ( "OTHERWISE" else_body:stat_list )?
  "END" ;

when_item <CmmWhenItem> <custom no_produce>:
  "OR_WHEN" sub_expr:expr "DO" sub_body:stat_list;

/* call */  

call_stat <CmmCallStat:CmmStat> <custom produce>:
  "CALL"
  body:stat_list
  "END" ;

/* space */

space_stat <CmmSpaceStat:CmmStat> <custom send>:
  "SPACE" ;

/* location */

location_stat <CmmLocationStat:CmmStat> <custom produce>:
  "LOCATION" '(' place:expr ')' ;

/* super */

super_stat <CmmSuperStat:CmmStat> <custom send, produce>:
  "SUPER" '::' param:cast_expr ';' ;

/* ---------------------------------------------------------------------- */

alt_name <AltName>:
   id:identifier |
   txt:string_literal
   ;

/* ---------------------------------------------------------------------- */

alt_simple <AltSimple>:
   name:alt_name |
   '[' opt:alt_expression ']' |
   '(' sub_expr:alt_expression ')'
   ;  

alt_branch <AltBranch>:
   <add> alt_simple ( <add> alt_simple )*
   ;

alt_expression <AltExpr>:
   <add> alt_branch ( '|' <add> alt_branch )*
   ;

/* ---------------------------------------------------------------------- */

alt_item <DefItem>:
   name: alt_name
   ;

alt_list <DefList>:
   ( <add> alt_item )*
   ;

/* ---------------------------------------------------------------------- */

def_item <DefItem>:
   name:alt_name
   ;

def_list <DefList>:
   <add> def_item ( ',' <add> def_item )*
   ;

/* ---------------------------------------------------------------------- */

log_factor <LogFactor>:
   name:alt_name |
   '!' not_factor:log_factor |
   '(' sub_expr:log_expression ')'
   ;

log_term <LogTerm>:
   <add> log_factor ( '&' <add> log_factor )*
   ;

log_expression <LogExpr>:
   <add> log_term ( '|' <add> log_term )*
   ;

log_statement <LogStat>:
   name:alt_name '=' expr:log_expression
   ;
   
/* ---------------------------------------------------------------------- */

num_statements <NumStatSect>:
   ( <add> num_statement )*
   ;

num_statement <select NumStat>:
   num_assign |
   num_code |
   num_when |
   num_indent |
   num_unindent
   ;

num_code <NumCode: NumStat>:
   "CODE" code:string_literal "END"
   ;

num_indent <NumIndent: NumStat>:
   "INDENT"
   ;

num_unindent <NumUnindent: NumStat>:
   "UNINDENT"
   ;

num_when <NumWhen: NumStat>:
   "WHEN" cond:num_cond "DO" when_stat:num_statements 
   ( <add> num_stat_alternative )*
   ( "OTHERWISE" otherwise_stat:num_statements )?
   "END"
   ;

num_stat_alternative <NumStatAlternative>:
   "OR_WHEN" cond:num_cond "DO" stat:num_statements
   ;
   
num_assign <NumAssign: NumStat>:
   id:identifier
   ( '(' list:expr_list ')' )?
   '='
   (expr:expr | selection:num_selection) ';'
   ;

num_selection <NumSelection>:
   "WHEN" cond:num_cond "DO" when_expr:expr 
   ( <add> num_expr_alternative )*
   "OTHERWISE" otherwise_expr:expr "END"
   ;

num_expr_alternative <NumExprAlternative>:
   "OR_WHEN" cond:num_cond "DO" expr:expr 
   ;

num_cond <NumCond>:
   <add> num_and_cond
   ( "or" <add> num_and_cond )*
   ;

num_and_cond <NumAndCond>:
   <add> num_comp_cond
   ( "and" <add> num_comp_cond )*
   ;

num_comp_cond <NumCompCond:CmmExpr>:
   left_id:identifier
   (
      (
         "=" <set equal = true>
      |
         "<>" <set not_equal = true>
      )
      (
         right_id:identifier
      |
         right_val:numeric_literal
      )
      <set relation=true>
   )?;
