
/* EG: minicpp.g -> CMM.cpp */

#include "CMM.hpp"

namespace gibbon
{

/* %% decl */

CmmMemberVisibility::CmmMemberVisibility () :
  CmmDecl (),
  access (CmmAccess (0)),
  location (),
  start_note ()
{ }

CmmBaseItem::CmmBaseItem () :
  CmmBasic (),
  access (CmmAccess (0)),
  a_virtual (false),
  from (NULL),
  template_args (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmSpec::CmmSpec () :
  CmmBasic (),
  kind (CmmSpecKind (0)),
  next (NULL),
  location (),
  start_note ()
{ }

CmmSpec::CmmSpec (CmmSpecKind p_kind) :
  CmmBasic (),
  kind (p_kind),
  next (NULL),
  location (),
  start_note ()
{ }

CmmType::CmmType () :
  CmmBasic (),
  kind (CmmTypeKind (0)),
  next (NULL),
  location (),
  start_note ()
{ }

CmmType::CmmType (CmmTypeKind p_kind) :
  CmmBasic (),
  kind (p_kind),
  next (NULL),
  location (),
  start_note ()
{ }

CmmDecl::CmmDecl () :
  CmmBasic (),
  comment (),
  group (0),
  one_line_comment (false),
  insert_empty_line (false),
  ins_empty_line (false),
  one_line_function (false),
  save_newlines (false),
  redefine (false),
  redefine_access (CmmAccess (0)),
  required_access (CmmAccess (0)),
  group_comment (),
  next (NULL)
{ }

CmmCompoundStat::CmmCompoundStat () :
  CmmStat (),
  link (NULL),
  actual_dictionary (NULL),
  actual_group (NULL),
  actual_indent (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

void CmmDefineSect::add (CmmDefineItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmDefineSect::insert_first (CmmDefineItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmDefineSect::CmmDefineSect () :
  CmmBasic (),
  link (NULL),
  first (NULL),
  last (NULL)
{ }

CmmDefineSect::CmmDefineSect (CmmDefineSect * p_link) :
  CmmBasic (),
  link (p_link),
  first (NULL),
  last (NULL)
{ }

CmmDefineItem::CmmDefineItem () :
  CmmBasic (),
  mode (CmmDefineItemMode (0)),
  name (),
  value (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmDefineItem::CmmDefineItem (CmmDefineItemMode p_mode, string p_name, CmmExpr * p_value) :
  CmmBasic (),
  mode (p_mode),
  name (p_name),
  value (p_value),
  next (NULL),
  location (),
  start_note ()
{ }

CmmSuperStat::CmmSuperStat () :
  CmmStat (),
  cls (NULL),
  param (NULL),
  location (),
  start_note ()
{ }

CmmSuperStat::CmmSuperStat (CmmClassDecl * p_cls, CmmExpr * p_param) :
  CmmStat (),
  cls (p_cls),
  param (p_param),
  location (),
  start_note ()
{ }

void CmmTextSect::add (CmmTextItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmTextSect::insert_first (CmmTextItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmTextSect::CmmTextSect () :
  CmmBasic (),
  keyword_column (0),
  into_param (NULL),
  indent_expr (NULL),
  shift_value (0),
  first_column (0),
  start_line (false),
  acc (),
  spaces (0),
  indent (NULL),
  shift (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmContName::CmmContName () :
  CmmBasic (),
  destructor_name (false),
  template_name (false),
  id (),
  next (NULL),
  location (),
  start_note ()
{ }

CmmContName::CmmContName (bool p_destructor_name, bool p_template_name, string p_id) :
  CmmBasic (),
  destructor_name (p_destructor_name),
  template_name (p_template_name),
  id (p_id),
  next (NULL),
  location (),
  start_note ()
{ }

CmmSimpName::CmmSimpName () :
  CmmBasic (),
  id (),
  special (NULL),
  location (),
  start_note ()
{ }

CmmSimpName::CmmSimpName (string p_id, CmmSpecialFuction * p_special) :
  CmmBasic (),
  id (p_id),
  special (p_special),
  location (),
  start_note ()
{ }

void CmmNameExpr::add (CmmContName * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmNameExpr::insert_first (CmmContName * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmNameExpr::CmmNameExpr () :
  CmmExpr (),
  head (NULL),
  first (NULL),
  last (NULL)
{ }

CmmNameExpr::CmmNameExpr (CmmSimpName * p_head) :
  CmmExpr (),
  head (p_head),
  first (NULL),
  last (NULL)
{ }

CmmExpr::CmmExpr () :
  CmmBasic (),
  next (NULL),
  location (),
  start_note ()
{ }

CmmValueExpr::CmmValueExpr () :
  CmmExpr (),
  kind (CmmValueExpr_kind (0)),
  value (),
  location (),
  start_note ()
{ }

CmmValueExpr::CmmValueExpr (CmmValueExpr_kind p_kind, string p_value) :
  CmmExpr (),
  kind (p_kind),
  value (p_value),
  location (),
  start_note ()
{ }

CmmThisExpr::CmmThisExpr () :
  CmmExpr (),
  location (),
  start_note ()
{ }

CmmSubexprExpr::CmmSubexprExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmSubexprExpr::CmmSubexprExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmModernCastExpr::CmmModernCastExpr () :
  CmmExpr (),
  kind (CmmModernCastExpr_kind (0)),
  type (NULL),
  param (NULL),
  location (),
  start_note ()
{ }

CmmModernCastExpr::CmmModernCastExpr (CmmModernCastExpr_kind p_kind, CmmTypeId * p_type, CmmExpr * p_param) :
  CmmExpr (),
  kind (p_kind),
  type (p_type),
  param (p_param),
  location (),
  start_note ()
{ }

CmmTypeidExpr::CmmTypeidExpr () :
  CmmExpr (),
  param1 (NULL),
  param2 (NULL),
  location (),
  start_note ()
{ }

CmmTypeidExpr::CmmTypeidExpr (CmmExpr * p_param1, CmmTypeId * p_param2) :
  CmmExpr (),
  param1 (p_param1),
  param2 (p_param2),
  location (),
  start_note ()
{ }

CmmIndexExpr::CmmIndexExpr () :
  CmmExpr (),
  left (NULL),
  right (NULL),
  location (),
  start_note ()
{ }

CmmIndexExpr::CmmIndexExpr (CmmExpr * p_left, CmmExpr * p_right) :
  CmmExpr (),
  left (p_left),
  right (p_right),
  location (),
  start_note ()
{ }

CmmCallExpr::CmmCallExpr () :
  CmmExpr (),
  param (NULL),
  list (NULL),
  location (),
  start_note ()
{ }

CmmCallExpr::CmmCallExpr (CmmExpr * p_param, CmmExprList * p_list) :
  CmmExpr (),
  param (p_param),
  list (p_list),
  location (),
  start_note ()
{ }

CmmFieldExpr::CmmFieldExpr () :
  CmmExpr (),
  param (NULL),
  destructor_name (false),
  template_name (false),
  name (NULL),
  location (),
  start_note ()
{ }

CmmPtrFieldExpr::CmmPtrFieldExpr () :
  CmmExpr (),
  param (NULL),
  destructor_name (false),
  template_name (false),
  name (NULL),
  location (),
  start_note ()
{ }

CmmPostIncExpr::CmmPostIncExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmPostIncExpr::CmmPostIncExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmPostDecExpr::CmmPostDecExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmPostDecExpr::CmmPostDecExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

void CmmExprList::add (CmmExpr * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmExprList::insert_first (CmmExpr * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmExprList::CmmExprList () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmIncExpr::CmmIncExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmIncExpr::CmmIncExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmDecExpr::CmmDecExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmDecExpr::CmmDecExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmDerefExpr::CmmDerefExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmDerefExpr::CmmDerefExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmAddrExpr::CmmAddrExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmAddrExpr::CmmAddrExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmPlusExpr::CmmPlusExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmPlusExpr::CmmPlusExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmMinusExpr::CmmMinusExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmMinusExpr::CmmMinusExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmBitNotExpr::CmmBitNotExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmBitNotExpr::CmmBitNotExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmLogNotExpr::CmmLogNotExpr () :
  CmmExpr (),
  param (NULL),
  location (),
  start_note ()
{ }

CmmLogNotExpr::CmmLogNotExpr (CmmExpr * p_param) :
  CmmExpr (),
  param (p_param),
  location (),
  start_note ()
{ }

CmmSizeofExp::CmmSizeofExp () :
  CmmExpr (),
  param1 (NULL),
  param2 (NULL),
  location (),
  start_note ()
{ }

CmmSizeofExp::CmmSizeofExp (CmmExpr * p_param1, CmmTypeId * p_param2) :
  CmmExpr (),
  param1 (p_param1),
  param2 (p_param2),
  location (),
  start_note ()
{ }

void CmmNewExpr::add (CmmNewArrayLimit * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmNewExpr::insert_first (CmmNewArrayLimit * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmNewExpr::CmmNewExpr () :
  CmmExpr (),
  spec (NULL),
  type2 (NULL),
  init (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmNewExpr::CmmNewExpr (CmmAllocTypeSpecSect * p_spec, CmmTypeId * p_type2, CmmExprList * p_init) :
  CmmExpr (),
  spec (p_spec),
  type2 (p_type2),
  init (p_init),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmNewArrayLimit::CmmNewArrayLimit () :
  CmmBasic (),
  value (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmNewArrayLimit::CmmNewArrayLimit (CmmExpr * p_value) :
  CmmBasic (),
  value (p_value),
  next (NULL),
  location (),
  start_note ()
{ }

CmmDeleteExpr::CmmDeleteExpr () :
  CmmExpr (),
  a_array (false),
  param (NULL),
  location (),
  start_note ()
{ }

CmmDeleteExpr::CmmDeleteExpr (bool p_a_array, CmmExpr * p_param) :
  CmmExpr (),
  a_array (p_a_array),
  param (p_param),
  location (),
  start_note ()
{ }

CmmCastFormula::CmmCastFormula () :
  CmmExpr (),
  type (NULL),
  param (NULL),
  location (),
  start_note ()
{ }

CmmCastFormula::CmmCastFormula (CmmTypeId * p_type, CmmExpr * p_param) :
  CmmExpr (),
  type (p_type),
  param (p_param),
  location (),
  start_note ()
{ }

CmmBinaryExpr::CmmBinaryExpr () :
  CmmExpr (),
  kind (CmmBinaryExpr_kind (0)),
  left (NULL),
  right (NULL)
{ }

CmmBinaryExpr::CmmBinaryExpr (CmmBinaryExpr_kind p_kind, CmmExpr * p_left, CmmExpr * p_right) :
  CmmExpr (),
  kind (p_kind),
  left (p_left),
  right (p_right)
{ }

CmmCondExpr::CmmCondExpr () :
  CmmExpr (),
  cond_expr (NULL),
  then_expr (NULL),
  else_expr (NULL),
  location (),
  start_note ()
{ }

CmmCondExpr::CmmCondExpr (CmmExpr * p_cond_expr, CmmExpr * p_then_expr, CmmExpr * p_else_expr) :
  CmmExpr (),
  cond_expr (p_cond_expr),
  then_expr (p_then_expr),
  else_expr (p_else_expr),
  location (),
  start_note ()
{ }

void CmmStatSect::add (CmmStat * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmStatSect::insert_first (CmmStat * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmStatSect::CmmStatSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmStat::CmmStat () :
  CmmBasic (),
  next (NULL)
{ }

CmmDeclStat::CmmDeclStat () :
  CmmStat (),
  decl (NULL)
{ }

CmmDeclStat::CmmDeclStat (CmmDecl * p_decl) :
  CmmStat (),
  decl (p_decl)
{ }

CmmLabeledStat::CmmLabeledStat () :
  CmmStat (),
  lab (),
  body (NULL),
  location (),
  start_note ()
{ }

CmmLabeledStat::CmmLabeledStat (string p_lab, CmmStat * p_body) :
  CmmStat (),
  lab (p_lab),
  body (p_body),
  location (),
  start_note ()
{ }

CmmCaseStat::CmmCaseStat () :
  CmmStat (),
  case_expr (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

CmmCaseStat::CmmCaseStat (CmmExpr * p_case_expr, CmmStat * p_body) :
  CmmStat (),
  case_expr (p_case_expr),
  body (p_body),
  location (),
  start_note ()
{ }

CmmDefaultStat::CmmDefaultStat () :
  CmmStat (),
  body (NULL),
  location (),
  start_note ()
{ }

CmmDefaultStat::CmmDefaultStat (CmmStat * p_body) :
  CmmStat (),
  body (p_body),
  location (),
  start_note ()
{ }

CmmExprStat::CmmExprStat () :
  CmmStat (),
  inner_expr (NULL),
  location (),
  start_note ()
{ }

CmmExprStat::CmmExprStat (CmmExpr * p_inner_expr) :
  CmmStat (),
  inner_expr (p_inner_expr),
  location (),
  start_note ()
{ }

CmmEmptyStat::CmmEmptyStat () :
  CmmStat (),
  location (),
  start_note ()
{ }

CmmCondition::CmmCondition () :
  CmmBasic (),
  cond_expr (NULL)
{ }

CmmCondition::CmmCondition (CmmExpr * p_cond_expr) :
  CmmBasic (),
  cond_expr (p_cond_expr)
{ }

CmmIfStat::CmmIfStat () :
  CmmStat (),
  cond (NULL),
  then_stat (NULL),
  else_stat (NULL),
  location (),
  start_note ()
{ }

CmmIfStat::CmmIfStat (CmmCondition * p_cond, CmmStat * p_then_stat, CmmStat * p_else_stat) :
  CmmStat (),
  cond (p_cond),
  then_stat (p_then_stat),
  else_stat (p_else_stat),
  location (),
  start_note ()
{ }

CmmSwitchStat::CmmSwitchStat () :
  CmmStat (),
  cond (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

CmmSwitchStat::CmmSwitchStat (CmmCondition * p_cond, CmmStat * p_body) :
  CmmStat (),
  cond (p_cond),
  body (p_body),
  location (),
  start_note ()
{ }

CmmWhileStat::CmmWhileStat () :
  CmmStat (),
  cond (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

CmmWhileStat::CmmWhileStat (CmmCondition * p_cond, CmmStat * p_body) :
  CmmStat (),
  cond (p_cond),
  body (p_body),
  location (),
  start_note ()
{ }

CmmDoStat::CmmDoStat () :
  CmmStat (),
  body (NULL),
  cond_expr (NULL),
  location (),
  start_note ()
{ }

CmmDoStat::CmmDoStat (CmmStat * p_body, CmmExpr * p_cond_expr) :
  CmmStat (),
  body (p_body),
  cond_expr (p_cond_expr),
  location (),
  start_note ()
{ }

CmmForStat::CmmForStat () :
  CmmStat (),
  from_decl (NULL),
  from_expr (NULL),
  empty_param (false),
  to_expr (NULL),
  cond_expr (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

CmmBreakStat::CmmBreakStat () :
  CmmStat (),
  location (),
  start_note ()
{ }

CmmContinueStat::CmmContinueStat () :
  CmmStat (),
  location (),
  start_note ()
{ }

CmmReturnStat::CmmReturnStat () :
  CmmStat (),
  return_expr (NULL),
  location (),
  start_note ()
{ }

CmmReturnStat::CmmReturnStat (CmmExpr * p_return_expr) :
  CmmStat (),
  return_expr (p_return_expr),
  location (),
  start_note ()
{ }

CmmGotoStat::CmmGotoStat () :
  CmmStat (),
  goto_lab (),
  location (),
  start_note ()
{ }

CmmGotoStat::CmmGotoStat (string p_goto_lab) :
  CmmStat (),
  goto_lab (p_goto_lab),
  location (),
  start_note ()
{ }

void CmmDeclSect::add (CmmDecl * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmDeclSect::insert_first (CmmDecl * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmDeclSect::CmmDeclSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmCtorHead::CmmCtorHead () :
  CmmBasic (),
  spec (NULL),
  destructor (false),
  name (NULL),
  cont (NULL),
  init (NULL),
  body (false),
  try_block (false),
  empty (false),
  location (),
  start_note ()
{ }

CmmCtorDecl::CmmCtorDecl () :
  CmmDecl (),
  spec (NULL),
  destructor (false),
  decl (NULL),
  init (NULL),
  statements (NULL),
  handlers (NULL),
  empty (false),
  location (),
  start_note ()
{ }

void CmmSpecSect::add (CmmSpec * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmSpecSect::insert_first (CmmSpec * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmSpecSect::CmmSpecSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmFuncHead::CmmFuncHead () :
  CmmBasic (),
  spec (NULL),
  ptr (NULL),
  name (NULL),
  params (NULL),
  cont (NULL),
  body (false),
  try_block (false),
  empty (false),
  location (),
  start_note ()
{ }

CmmFuncDecl::CmmFuncDecl () :
  CmmDecl (),
  spec (NULL),
  decl (NULL),
  statements (NULL),
  handlers (NULL),
  empty (false),
  location (),
  start_note ()
{ }

void CmmSimpleDecl::add (CmmInitDeclarator * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmSimpleDecl::insert_first (CmmInitDeclarator * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmSimpleDecl::CmmSimpleDecl () :
  CmmDecl (),
  spec (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmInitDeclarator::CmmInitDeclarator () :
  CmmBasic (),
  decl (NULL),
  init (NULL),
  width (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmInitDeclarator::CmmInitDeclarator (CmmDeclarator * p_decl, CmmInitializer * p_init, CmmExpr * p_width) :
  CmmBasic (),
  decl (p_decl),
  init (p_init),
  width (p_width),
  next (NULL),
  location (),
  start_note ()
{ }

CmmEmptyDecl::CmmEmptyDecl () :
  CmmDecl (),
  location (),
  start_note ()
{ }

CmmDeclSpec::CmmDeclSpec () :
  CmmBasic (),
  mode (NULL),
  name (NULL),
  template_args (NULL),
  class_spec (NULL),
  enum_spec (NULL),
  type (NULL),
  mode2 (NULL)
{ }

void CmmTypeSect::add (CmmType * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmTypeSect::insert_first (CmmType * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmTypeSect::CmmTypeSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

void CmmAllocTypeSpecSect::add (CmmAllocTypeSpec * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmAllocTypeSpecSect::insert_first (CmmAllocTypeSpec * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmAllocTypeSpecSect::CmmAllocTypeSpecSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmAllocTypeSpec::CmmAllocTypeSpec () :
  CmmBasic (),
  a_const (false),
  a_volatile (false),
  name (NULL),
  class_spec (NULL),
  enum_spec (NULL),
  simple (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmEnumDecl::CmmEnumDecl () :
  CmmDecl (),
  name (NULL),
  items (NULL),
  location (),
  start_note ()
{ }

void CmmEnumSect::add (CmmEnumItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmEnumSect::insert_first (CmmEnumItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmEnumSect::CmmEnumSect () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmEnumItem::CmmEnumItem () :
  CmmBasic (),
  name (),
  init (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmEnumItem::CmmEnumItem (string p_name, CmmExpr * p_init) :
  CmmBasic (),
  name (p_name),
  init (p_init),
  next (NULL),
  location (),
  start_note ()
{ }

CmmNamespace::CmmNamespace () :
  CmmDecl (),
  unnamed_body (NULL),
  namespace_name (),
  value (NULL),
  body (NULL),
  location (),
  start_note ()
{ }

CmmUsingDecl::CmmUsingDecl () :
  CmmDecl (),
  a_namespace (false),
  a_typename (false),
  name (NULL),
  location (),
  start_note ()
{ }

CmmExternDecl::CmmExternDecl () :
  CmmDecl (),
  language (),
  decl_list (NULL),
  decl (NULL),
  location (),
  start_note ()
{ }

CmmAsmDecl::CmmAsmDecl () :
  CmmDecl (),
  instruction (),
  location (),
  start_note ()
{ }

CmmTypeId::CmmTypeId () :
  CmmBasic (),
  spec (NULL),
  decl (NULL),
  next (NULL)
{ }

CmmTypeId::CmmTypeId (CmmDeclSpec * p_spec, CmmAbstractDeclarator * p_decl) :
  CmmBasic (),
  spec (p_spec),
  decl (p_decl),
  next (NULL)
{ }

CmmCommonTypeId::CmmCommonTypeId () :
  CmmBasic (),
  spec (NULL),
  decl (NULL)
{ }

CmmCommonTypeId::CmmCommonTypeId (CmmDeclSpec * p_spec, CmmCommonDeclarator * p_decl) :
  CmmBasic (),
  spec (p_spec),
  decl (p_decl)
{ }

CmmDeclarator::CmmDeclarator () :
  CmmBasic (),
  ptr (NULL),
  name (NULL),
  inner (NULL),
  cont (NULL),
  location (),
  start_note ()
{ }

CmmAbstractDeclarator::CmmAbstractDeclarator () :
  CmmBasic (),
  ptr (NULL),
  inner (NULL),
  cont (NULL),
  location (),
  start_note ()
{ }

CmmAbstractDeclarator::CmmAbstractDeclarator (CmmPtrSpecifierSect * p_ptr, CmmAbstractDeclarator * p_inner, CmmContDeclaratorSect * p_cont) :
  CmmBasic (),
  ptr (p_ptr),
  inner (p_inner),
  cont (p_cont),
  location (),
  start_note ()
{ }

CmmCommonDeclarator::CmmCommonDeclarator () :
  CmmBasic (),
  ptr (NULL),
  name (NULL),
  inner (NULL),
  cont (NULL),
  location (),
  start_note ()
{ }

void CmmContDeclaratorSect::add (CmmContDeclarator * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmContDeclaratorSect::insert_first (CmmContDeclarator * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmContDeclaratorSect::CmmContDeclaratorSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmContDeclarator::CmmContDeclarator () :
  CmmBasic (),
  next (NULL)
{ }

CmmContFunctionDeclarator::CmmContFunctionDeclarator () :
  CmmContDeclarator (),
  parameters (NULL),
  cv_spec (NULL),
  exception_spec (NULL),
  location (),
  start_note ()
{ }

CmmContFunctionDeclarator::CmmContFunctionDeclarator (CmmParamSect * p_parameters, CmmCvSpecifierSect * p_cv_spec, CmmExceptionSpec * p_exception_spec) :
  CmmContDeclarator (),
  parameters (p_parameters),
  cv_spec (p_cv_spec),
  exception_spec (p_exception_spec),
  location (),
  start_note ()
{ }

CmmContArrayDeclarator::CmmContArrayDeclarator () :
  CmmContDeclarator (),
  lim (NULL),
  location (),
  start_note ()
{ }

CmmContArrayDeclarator::CmmContArrayDeclarator (CmmExpr * p_lim) :
  CmmContDeclarator (),
  lim (p_lim),
  location (),
  start_note ()
{ }

void CmmPtrSpecifierSect::add (CmmPtrSpecifier * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmPtrSpecifierSect::insert_first (CmmPtrSpecifier * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmPtrSpecifierSect::CmmPtrSpecifierSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmPtrSpecifier::CmmPtrSpecifier () :
  CmmBasic (),
  cv_spec (NULL),
  pointer (false),
  reference (false),
  next (NULL),
  location (),
  start_note ()
{ }

CmmPtrSpecifier::CmmPtrSpecifier (CmmCvSpecifierSect * p_cv_spec, bool p_pointer, bool p_reference) :
  CmmBasic (),
  cv_spec (p_cv_spec),
  pointer (p_pointer),
  reference (p_reference),
  next (NULL),
  location (),
  start_note ()
{ }

void CmmCvSpecifierSect::add (CmmCvSpecifier * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmCvSpecifierSect::insert_first (CmmCvSpecifier * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmCvSpecifierSect::CmmCvSpecifierSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmCvSpecifier::CmmCvSpecifier () :
  CmmBasic (),
  a_const (false),
  a_volatile (false),
  next (NULL),
  location (),
  start_note ()
{ }

CmmCvSpecifier::CmmCvSpecifier (bool p_a_const, bool p_a_volatile) :
  CmmBasic (),
  a_const (p_a_const),
  a_volatile (p_a_volatile),
  next (NULL),
  location (),
  start_note ()
{ }

void CmmParamSect::add (CmmParamItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmParamSect::insert_first (CmmParamItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmParamSect::CmmParamSect () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmParamItem::CmmParamItem () :
  CmmBasic (),
  type (NULL),
  init (NULL),
  dots (false),
  next (NULL),
  location (),
  start_note ()
{ }

CmmParamItem::CmmParamItem (CmmCommonTypeId * p_type, CmmExpr * p_init, bool p_dots) :
  CmmBasic (),
  type (p_type),
  init (p_init),
  dots (p_dots),
  next (NULL),
  location (),
  start_note ()
{ }

CmmInitializer::CmmInitializer () :
  CmmBasic (),
  value (NULL),
  location (),
  start_note ()
{ }

CmmInitializer::CmmInitializer (CmmInitItem * p_value) :
  CmmBasic (),
  value (p_value),
  location (),
  start_note ()
{ }

CmmInitItem::CmmInitItem () :
  CmmBasic (),
  next (NULL)
{ }

CmmInitSimple::CmmInitSimple () :
  CmmInitItem (),
  expr (NULL)
{ }

CmmInitSimple::CmmInitSimple (CmmExpr * p_expr) :
  CmmInitItem (),
  expr (p_expr)
{ }

void CmmInitList::add (CmmInitItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmInitList::insert_first (CmmInitItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmInitList::CmmInitList () :
  CmmInitItem (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmClassDecl::CmmClassDecl () :
  CmmDecl (),
  style (CmmClassDecl_style (0)),
  base_list (NULL),
  members (NULL),
  anonymous (false),
  name (NULL),
  location (),
  start_note ()
{ }

void CmmBaseSect::add (CmmBaseItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmBaseSect::insert_first (CmmBaseItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmBaseSect::CmmBaseSect () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

void CmmCtorInitializer::add (CmmMemberInitializer * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmCtorInitializer::insert_first (CmmMemberInitializer * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmCtorInitializer::CmmCtorInitializer () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmMemberInitializer::CmmMemberInitializer () :
  CmmBasic (),
  name (NULL),
  params (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmMemberInitializer::CmmMemberInitializer (CmmNameExpr * p_name, CmmExprList * p_params) :
  CmmBasic (),
  name (p_name),
  params (p_params),
  next (NULL),
  location (),
  start_note ()
{ }

CmmSpecialFuction::CmmSpecialFuction () :
  CmmBasic (),
  spec_new (false),
  spec_delete (false),
  spec_add (false),
  spec_sub (false),
  spec_mul (false),
  spec_div (false),
  spec_mod (false),
  spec_xor (false),
  spec_and (false),
  spec_or (false),
  spec_not (false),
  spec_log_not (false),
  spec_eq (false),
  spec_lt (false),
  spec_gt (false),
  spec_add_assign (false),
  spec_sub_assign (false),
  spec_mul_assign (false),
  spec_div_assign (false),
  spec_mod_assign (false),
  spec_xor_assign (false),
  spec_and_assign (false),
  spec_or_assign (false),
  spec_shl (false),
  spec_shr (false),
  spec_shl_assign (false),
  spec_shr_assign (false),
  spec_ne (false),
  spec_le (false),
  spec_ge (false),
  spec_log_and (false),
  spec_log_or (false),
  spec_inc (false),
  spec_dec (false),
  spec_comma (false),
  spec_mebber_deref (false),
  spec_deref (false),
  spec_call (false),
  spec_index (false),
  location (),
  start_note ()
{ }

CmmTemplateDecl::CmmTemplateDecl () :
  CmmDecl (),
  export_template (false),
  params (NULL),
  explicit_specialization (false),
  parameter_block (false),
  explicit_instatiation (false),
  body (NULL),
  location (),
  start_note ()
{ }

void CmmTemplateParamSect::add (CmmTemplateParam * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmTemplateParamSect::insert_first (CmmTemplateParam * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmTemplateParamSect::CmmTemplateParamSect () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmTemplateParam::CmmTemplateParam () :
  CmmBasic (),
  next (NULL)
{ }

CmmTemplateTypeParam::CmmTemplateTypeParam () :
  CmmTemplateParam (),
  a_class (false),
  a_typename (false),
  name (),
  value (NULL),
  location (),
  start_note ()
{ }

CmmTemplateExtraParam::CmmTemplateExtraParam () :
  CmmTemplateParam (),
  params (NULL),
  name (),
  value (NULL),
  location (),
  start_note ()
{ }

CmmTemplateExtraParam::CmmTemplateExtraParam (CmmTemplateParamSect * p_params, string p_name, CmmNameExpr * p_value) :
  CmmTemplateParam (),
  params (p_params),
  name (p_name),
  value (p_value),
  location (),
  start_note ()
{ }

CmmTemplateValueParam::CmmTemplateValueParam () :
  CmmTemplateParam (),
  type (NULL),
  init (NULL),
  location (),
  start_note ()
{ }

CmmTemplateValueParam::CmmTemplateValueParam (CmmCommonTypeId * p_type, CmmExpr * p_init) :
  CmmTemplateParam (),
  type (p_type),
  init (p_init),
  location (),
  start_note ()
{ }

void CmmTemplateArgSect::add (CmmTemplateArg * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmTemplateArgSect::insert_first (CmmTemplateArg * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmTemplateArgSect::CmmTemplateArgSect () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmTemplateArg::CmmTemplateArg () :
  CmmBasic (),
  next (NULL)
{ }

CmmTemplateTypeArg::CmmTemplateTypeArg () :
  CmmTemplateArg (),
  type (NULL)
{ }

CmmTemplateTypeArg::CmmTemplateTypeArg (CmmTypeId * p_type) :
  CmmTemplateArg (),
  type (p_type)
{ }

CmmTemplateValueArg::CmmTemplateValueArg () :
  CmmTemplateArg (),
  value (NULL)
{ }

CmmTemplateValueArg::CmmTemplateValueArg (CmmExpr * p_value) :
  CmmTemplateArg (),
  value (p_value)
{ }

CmmTryStat::CmmTryStat () :
  CmmStat (),
  body (NULL),
  handlers (NULL),
  location (),
  start_note ()
{ }

CmmTryStat::CmmTryStat (CmmCompoundStat * p_body, CmmHandlerSect * p_handlers) :
  CmmStat (),
  body (p_body),
  handlers (p_handlers),
  location (),
  start_note ()
{ }

void CmmHandlerSect::add (CmmHandlerItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmHandlerSect::insert_first (CmmHandlerItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmHandlerSect::CmmHandlerSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

CmmHandlerItem::CmmHandlerItem () :
  CmmBasic (),
  exception (NULL),
  body (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmHandlerItem::CmmHandlerItem (CmmExceptionDecl * p_exception, CmmCompoundStat * p_body) :
  CmmBasic (),
  exception (p_exception),
  body (p_body),
  next (NULL),
  location (),
  start_note ()
{ }

CmmExceptionDecl::CmmExceptionDecl () :
  CmmBasic (),
  type (NULL),
  dots (false),
  location (),
  start_note ()
{ }

CmmExceptionDecl::CmmExceptionDecl (CmmCommonTypeId * p_type, bool p_dots) :
  CmmBasic (),
  type (p_type),
  dots (p_dots),
  location (),
  start_note ()
{ }

CmmThrowExpr::CmmThrowExpr () :
  CmmExpr (),
  expr (NULL),
  location (),
  start_note ()
{ }

CmmThrowExpr::CmmThrowExpr (CmmExpr * p_expr) :
  CmmExpr (),
  expr (p_expr),
  location (),
  start_note ()
{ }

void CmmExceptionSpec::add (CmmTypeId * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmExceptionSpec::insert_first (CmmTypeId * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmExceptionSpec::CmmExceptionSpec () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmIncludeName::CmmIncludeName () :
  CmmBasic (),
  include_q_str (),
  include_h_str (),
  include_id (),
  location (),
  start_note ()
{ }

CmmIncludeName::CmmIncludeName (string p_include_q_str, string p_include_h_str, string p_include_id) :
  CmmBasic (),
  include_q_str (p_include_q_str),
  include_h_str (p_include_h_str),
  include_id (p_include_id),
  location (),
  start_note ()
{ }

CmmDirective::CmmDirective () :
  CmmDecl (),
  d1 (),
  include_directive (false),
  include_name (NULL),
  if_directive (false),
  if_expr (),
  ifdef_directive (false),
  ifdef_name (),
  ifndef_directive (false),
  ifndef_name (),
  else_directive (false),
  endif_directive (false),
  define_directive (false),
  define_name (),
  undef_directive (false),
  undef_name (),
  d2 (),
  location (),
  start_note ()
{ }

CmmDirectiveStat::CmmDirectiveStat () :
  CmmStat (),
  d1 (),
  include_directive (false),
  include_name (NULL),
  if_directive (false),
  if_expr (),
  ifdef_directive (false),
  ifdef_name (),
  ifndef_directive (false),
  ifndef_name (),
  else_directive (false),
  endif_directive (false),
  define_directive (false),
  define_name (),
  undef_directive (false),
  undef_name (),
  d2 (),
  location (),
  start_note ()
{ }

CmmNamespaceExtension::CmmNamespaceExtension () :
  CmmDecl (),
  open (false),
  location (),
  start_note ()
{ }

CmmClusterStat::CmmClusterStat () :
  CmmStat (),
  body (NULL)
{ }

CmmClusterStat::CmmClusterStat (CmmStatSect * p_body) :
  CmmStat (),
  body (p_body)
{ }

CmmIntoStat::CmmIntoStat () :
  CmmStat (),
  into_param (NULL),
  store_param (NULL),
  with_into (false),
  group (NULL),
  comment (NULL),
  def (NULL),
  prior (false),
  expr (NULL),
  expr_list (NULL),
  stat (NULL),
  code (NULL),
  decl (NULL),
  decl_list (NULL),
  param_list (NULL),
  enum_type (NULL),
  enum_items (NULL),
  cls (NULL),
  base_list (NULL),
  a_public (false),
  a_protected (false),
  a_private (false),
  access (NULL),
  member (NULL),
  members (NULL),
  ctor_init (NULL),
  init_item (NULL),
  init_list (NULL),
  text_sect (NULL),
  location (),
  start_note ()
{ }

CmmTextItem::CmmTextItem () :
  CmmBasic (),
  token_value (),
  token_flag (false),
  space_value (),
  white_space_flag (false),
  new_line_value (),
  new_line_flag (false),
  other_value (),
  other_flag (false),
  next (NULL),
  location (),
  start_note ()
{ }

CmmDefineStat::CmmDefineStat () :
  CmmStat (),
  def (NULL),
  location (),
  start_note ()
{ }

CmmDefineStat::CmmDefineStat (CmmDefineSect * p_def) :
  CmmStat (),
  def (p_def),
  location (),
  start_note ()
{ }

CmmGroupStat::CmmGroupStat () :
  CmmStat (),
  value (NULL),
  location (),
  start_note ()
{ }

CmmGroupStat::CmmGroupStat (CmmExpr * p_value) :
  CmmStat (),
  value (p_value),
  location (),
  start_note ()
{ }

CmmIndentStat::CmmIndentStat () :
  CmmStat (),
  value (NULL),
  location (),
  start_note ()
{ }

CmmIndentStat::CmmIndentStat (CmmExpr * p_value) :
  CmmStat (),
  value (p_value),
  location (),
  start_note ()
{ }

void CmmWhenStat::add (CmmWhenItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void CmmWhenStat::insert_first (CmmWhenItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

CmmWhenStat::CmmWhenStat () :
  CmmStat (),
  cond_expr (NULL),
  then_body (NULL),
  else_body (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmWhenStat::CmmWhenStat (CmmExpr * p_cond_expr, CmmStatSect * p_then_body, CmmStatSect * p_else_body) :
  CmmStat (),
  cond_expr (p_cond_expr),
  then_body (p_then_body),
  else_body (p_else_body),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

CmmWhenItem::CmmWhenItem () :
  CmmBasic (),
  sub_expr (NULL),
  sub_body (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

CmmWhenItem::CmmWhenItem (CmmExpr * p_sub_expr, CmmStatSect * p_sub_body) :
  CmmBasic (),
  sub_expr (p_sub_expr),
  sub_body (p_sub_body),
  next (NULL),
  location (),
  start_note ()
{ }

CmmCallStat::CmmCallStat () :
  CmmStat (),
  body (NULL),
  location (),
  start_note ()
{ }

CmmCallStat::CmmCallStat (CmmStatSect * p_body) :
  CmmStat (),
  body (p_body),
  location (),
  start_note ()
{ }

CmmSpaceStat::CmmSpaceStat () :
  CmmStat (),
  location (),
  start_note ()
{ }

CmmLocationStat::CmmLocationStat () :
  CmmStat (),
  place (NULL),
  location (),
  start_note ()
{ }

CmmLocationStat::CmmLocationStat (CmmExpr * p_place) :
  CmmStat (),
  place (p_place),
  location (),
  start_note ()
{ }

AltName::AltName () :
  CmmBasic (),
  id (),
  txt (),
  location (),
  start_note ()
{ }

AltName::AltName (string p_id, string p_txt) :
  CmmBasic (),
  id (p_id),
  txt (p_txt),
  location (),
  start_note ()
{ }

AltSimple::AltSimple () :
  CmmBasic (),
  name (NULL),
  opt (NULL),
  sub_expr (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

AltSimple::AltSimple (AltName * p_name, AltExpr * p_opt, AltExpr * p_sub_expr) :
  CmmBasic (),
  name (p_name),
  opt (p_opt),
  sub_expr (p_sub_expr),
  next (NULL),
  location (),
  start_note ()
{ }

void AltBranch::add (AltSimple * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void AltBranch::insert_first (AltSimple * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

AltBranch::AltBranch () :
  CmmBasic (),
  next (NULL),
  first (NULL),
  last (NULL)
{ }

void AltExpr::add (AltBranch * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void AltExpr::insert_first (AltBranch * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

AltExpr::AltExpr () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

DefItem::DefItem () :
  CmmBasic (),
  name (NULL),
  next (NULL)
{ }

DefItem::DefItem (AltName * p_name) :
  CmmBasic (),
  name (p_name),
  next (NULL)
{ }

void DefList::add (DefItem * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void DefList::insert_first (DefItem * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

DefList::DefList () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

LogFactor::LogFactor () :
  CmmBasic (),
  name (NULL),
  not_factor (NULL),
  sub_expr (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

LogFactor::LogFactor (AltName * p_name, LogFactor * p_not_factor, LogExpr * p_sub_expr) :
  CmmBasic (),
  name (p_name),
  not_factor (p_not_factor),
  sub_expr (p_sub_expr),
  next (NULL),
  location (),
  start_note ()
{ }

void LogTerm::add (LogFactor * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void LogTerm::insert_first (LogFactor * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

LogTerm::LogTerm () :
  CmmBasic (),
  next (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

void LogExpr::add (LogTerm * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void LogExpr::insert_first (LogTerm * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

LogExpr::LogExpr () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

LogStat::LogStat () :
  CmmBasic (),
  name (NULL),
  expr (NULL),
  location (),
  start_note ()
{ }

LogStat::LogStat (AltName * p_name, LogExpr * p_expr) :
  CmmBasic (),
  name (p_name),
  expr (p_expr),
  location (),
  start_note ()
{ }

void NumStatSect::add (NumStat * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void NumStatSect::insert_first (NumStat * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

NumStatSect::NumStatSect () :
  CmmBasic (),
  first (NULL),
  last (NULL)
{ }

NumStat::NumStat () :
  CmmBasic (),
  next (NULL)
{ }

NumCode::NumCode () :
  NumStat (),
  code (),
  location (),
  start_note ()
{ }

NumCode::NumCode (string p_code) :
  NumStat (),
  code (p_code),
  location (),
  start_note ()
{ }

NumIndent::NumIndent () :
  NumStat (),
  location (),
  start_note ()
{ }

NumUnindent::NumUnindent () :
  NumStat (),
  location (),
  start_note ()
{ }

void NumWhen::add (NumStatAlternative * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void NumWhen::insert_first (NumStatAlternative * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

NumWhen::NumWhen () :
  NumStat (),
  cond (NULL),
  when_stat (NULL),
  otherwise_stat (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

NumWhen::NumWhen (NumCond * p_cond, NumStatSect * p_when_stat, NumStatSect * p_otherwise_stat) :
  NumStat (),
  cond (p_cond),
  when_stat (p_when_stat),
  otherwise_stat (p_otherwise_stat),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

NumStatAlternative::NumStatAlternative () :
  CmmBasic (),
  cond (NULL),
  stat (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

NumStatAlternative::NumStatAlternative (NumCond * p_cond, NumStatSect * p_stat) :
  CmmBasic (),
  cond (p_cond),
  stat (p_stat),
  next (NULL),
  location (),
  start_note ()
{ }

NumAssign::NumAssign () :
  NumStat (),
  id (),
  list (NULL),
  expr (NULL),
  selection (NULL),
  location (),
  start_note ()
{ }

void NumSelection::add (NumExprAlternative * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void NumSelection::insert_first (NumExprAlternative * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

NumSelection::NumSelection () :
  CmmBasic (),
  cond (NULL),
  when_expr (NULL),
  otherwise_expr (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

NumSelection::NumSelection (NumCond * p_cond, CmmExpr * p_when_expr, CmmExpr * p_otherwise_expr) :
  CmmBasic (),
  cond (p_cond),
  when_expr (p_when_expr),
  otherwise_expr (p_otherwise_expr),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

NumExprAlternative::NumExprAlternative () :
  CmmBasic (),
  cond (NULL),
  expr (NULL),
  next (NULL),
  location (),
  start_note ()
{ }

NumExprAlternative::NumExprAlternative (NumCond * p_cond, CmmExpr * p_expr) :
  CmmBasic (),
  cond (p_cond),
  expr (p_expr),
  next (NULL),
  location (),
  start_note ()
{ }

void NumCond::add (NumAndCond * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void NumCond::insert_first (NumAndCond * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

NumCond::NumCond () :
  CmmBasic (),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

void NumAndCond::add (NumCompCond * p_item)
{
  if (first == NULL)
    first = p_item;
  else
    last->next = p_item;
  last = p_item;
  p_item->next = NULL;
}

void NumAndCond::insert_first (NumCompCond * p_item)
{
  p_item->next = first;
  first = p_item;
  if (last == NULL)
    last = p_item;
}

NumAndCond::NumAndCond () :
  CmmBasic (),
  next (NULL),
  first (NULL),
  last (NULL),
  location (),
  start_note ()
{ }

NumCompCond::NumCompCond () :
  CmmExpr (),
  left_id (),
  equal (false),
  not_equal (false),
  right_id (),
  right_val (),
  relation (false),
  next (NULL),
  location (),
  start_note ()
{ }

/* %% list */

void send_identifier (string p_item)
{
  send (p_item);
}

void send_cont_name (CmmContName * p_item)
{
  assert (p_item != NULL);
  style_no_space ();
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("::");
  style_no_space ();
  if (p_item->destructor_name == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("~");
    style_no_space ();
    assert (p_item->destructor_name == true);
  }
  else if (p_item->template_name == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("template");
    assert (p_item->template_name == true);
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send_identifier (p_item->id);
}

void send_simple_name (CmmSimpName * p_item)
{
  assert (p_item != NULL);
  open_simple_name (p_item);
  if (! p_item->id.empty())
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send_identifier (p_item->id);
  }
  else if (p_item->special != NULL)
  {
    send_special_function (p_item->special);
  }
  else assert (false);
  close_simple_name (p_item);
}

void send_id_expr (CmmNameExpr * p_item)
{
  assert (p_item != NULL);
  CmmContName * p_subitem = p_item->first;
  send_simple_name (p_item->head);
  while (p_subitem != NULL)
  {
    send_cont_name (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_primary_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmValueExpr () != NULL)
  {
    send_literal_expr (p_item->conv_CmmValueExpr ());
  }
  else if (p_item->conv_CmmThisExpr () != NULL)
  {
    send_this_expr (p_item->conv_CmmThisExpr ());
  }
  else if (p_item->conv_CmmNameExpr () != NULL)
  {
    send_id_expr (p_item->conv_CmmNameExpr ());
  }
  else if (p_item->conv_CmmSubexprExpr () != NULL)
  {
    send_subexpr_expr (p_item->conv_CmmSubexprExpr ());
  }
  else assert (false);
}

void send_literal_expr (CmmValueExpr * p_item)
{
  assert (p_item != NULL);
  assert (p_item->kind == ValueExp);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_literal (p_item->value);
}

void send_literal (string p_item)
{
  send (p_item);
}

void send_this_expr (CmmThisExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("this");
}

void send_subexpr_expr (CmmSubexprExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("(");
  send_expr (p_item->param);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (")");
}

void send_postfix_start (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmModernCastExpr () != NULL)
  {
    send_modern_cast_expr (p_item->conv_CmmModernCastExpr ());
  }
  else if (p_item->conv_CmmTypeidExpr () != NULL)
  {
    send_typeid_expr (p_item->conv_CmmTypeidExpr ());
  }
  else if (p_item->conv_CmmExpr () != NULL)
  {
    send_primary_expr (p_item);
  }
  else assert (false);
}

void send_modern_cast_expr (CmmModernCastExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->kind == DynamicCast)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("dynamic_cast");
    assert (p_item->kind == DynamicCast);
  }
  else if (p_item->kind == StaticCast)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("static_cast");
    assert (p_item->kind == StaticCast);
  }
  else if (p_item->kind == ConstCast)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("const_cast");
    assert (p_item->kind == ConstCast);
  }
  else if (p_item->kind == ReinterpreterCast)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("reinterpret_cast");
    assert (p_item->kind == ReinterpreterCast);
  }
  else assert (false);
  sendStartNote (p_item->start_note [4]);
  sendLocation (p_item->location [4]);
  send ("<");
  style_no_space ();
  send_type_id (p_item->type);
  style_no_space ();
  sendStartNote (p_item->start_note [5]);
  sendLocation (p_item->location [5]);
  send (">");
  sendStartNote (p_item->start_note [6]);
  sendLocation (p_item->location [6]);
  send ("(");
  send_expr (p_item->param);
  sendStartNote (p_item->start_note [7]);
  sendLocation (p_item->location [7]);
  send (")");
}

void send_typeid_expr (CmmTypeidExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("typeid");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  if (p_item->param1 != NULL)
  {
    send_expr (p_item->param1);
  }
  else if (p_item->param2 != NULL)
  {
    send_type_id (p_item->param2);
  }
  else assert (false);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
}

void send_postfix_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmIndexExpr () != NULL)
  {
    CmmIndexExpr * p_item1 = p_item->conv_CmmIndexExpr ();
    send_postfix_expr (p_item1->left);
    send_index_expr (p_item1);
  }
  else if (p_item->conv_CmmCallExpr () != NULL)
  {
    CmmCallExpr * p_item1 = p_item->conv_CmmCallExpr ();
    send_postfix_expr (p_item1->param);
    send_call_expr (p_item1);
  }
  else if (p_item->conv_CmmFieldExpr () != NULL)
  {
    CmmFieldExpr * p_item1 = p_item->conv_CmmFieldExpr ();
    send_postfix_expr (p_item1->param);
    send_field_expr (p_item1);
  }
  else if (p_item->conv_CmmPtrFieldExpr () != NULL)
  {
    CmmPtrFieldExpr * p_item1 = p_item->conv_CmmPtrFieldExpr ();
    send_postfix_expr (p_item1->param);
    send_ptr_field_expr (p_item1);
  }
  else if (p_item->conv_CmmPostIncExpr () != NULL)
  {
    CmmPostIncExpr * p_item1 = p_item->conv_CmmPostIncExpr ();
    send_postfix_expr (p_item1->param);
    send_post_inc_expr (p_item1);
  }
  else if (p_item->conv_CmmPostDecExpr () != NULL)
  {
    CmmPostDecExpr * p_item1 = p_item->conv_CmmPostDecExpr ();
    send_postfix_expr (p_item1->param);
    send_post_dec_expr (p_item1);
  }
  else send_postfix_start (p_item);
}

void send_index_expr (CmmIndexExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("[");
  send_expr (p_item->right);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("]");
}

void send_call_expr (CmmCallExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("(");
  send_expr_list (p_item->list);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (")");
}

void send_field_expr (CmmFieldExpr * p_item)
{
  assert (p_item != NULL);
  style_no_space ();
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send (".");
  style_no_space ();
  if (p_item->destructor_name == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("~");
    style_no_space ();
    assert (p_item->destructor_name == true);
  }
  else if (p_item->template_name == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("template");
    assert (p_item->template_name == true);
  }
  send_id_expr (p_item->name);
}

void send_ptr_field_expr (CmmPtrFieldExpr * p_item)
{
  assert (p_item != NULL);
  style_no_space ();
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("->");
  style_no_space ();
  if (p_item->destructor_name == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("~");
    style_no_space ();
    assert (p_item->destructor_name == true);
  }
  else if (p_item->template_name == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("template");
    assert (p_item->template_name == true);
  }
  send_id_expr (p_item->name);
}

void send_post_inc_expr (CmmPostIncExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("++");
}

void send_post_dec_expr (CmmPostDecExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("--");
}

void send_expr_list (CmmExprList * p_item)
{
  assert (p_item != NULL);
  CmmExpr * p_subitem = p_item->first;
  if (p_subitem != NULL)
  {
    send_assignment_expr (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send (",");
      send_assignment_expr (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  assert (p_subitem == NULL);
}

void send_unary_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmIncExpr () != NULL)
  {
    send_inc_expr (p_item->conv_CmmIncExpr ());
  }
  else if (p_item->conv_CmmDecExpr () != NULL)
  {
    send_dec_expr (p_item->conv_CmmDecExpr ());
  }
  else if (p_item->conv_CmmDerefExpr () != NULL)
  {
    send_deref_expr (p_item->conv_CmmDerefExpr ());
  }
  else if (p_item->conv_CmmAddrExpr () != NULL)
  {
    send_addr_expr (p_item->conv_CmmAddrExpr ());
  }
  else if (p_item->conv_CmmPlusExpr () != NULL)
  {
    send_plus_expr (p_item->conv_CmmPlusExpr ());
  }
  else if (p_item->conv_CmmMinusExpr () != NULL)
  {
    send_minus_expr (p_item->conv_CmmMinusExpr ());
  }
  else if (p_item->conv_CmmBitNotExpr () != NULL)
  {
    send_bit_not_expr (p_item->conv_CmmBitNotExpr ());
  }
  else if (p_item->conv_CmmLogNotExpr () != NULL)
  {
    send_log_not_expr (p_item->conv_CmmLogNotExpr ());
  }
  else if (p_item->conv_CmmSizeofExp () != NULL)
  {
    send_sizeof_expr (p_item->conv_CmmSizeofExp ());
  }
  else if (p_item->conv_CmmNewExpr () != NULL)
  {
    send_allocation_expr (p_item->conv_CmmNewExpr ());
  }
  else if (p_item->conv_CmmDeleteExpr () != NULL)
  {
    send_deallocation_expr (p_item->conv_CmmDeleteExpr ());
  }
  else if (p_item->conv_CmmExpr () != NULL)
  {
    send_postfix_expr (p_item);
  }
  else assert (false);
}

void send_inc_expr (CmmIncExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("++");
  send_cast_expr (p_item->param);
}

void send_dec_expr (CmmDecExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("--");
  send_cast_expr (p_item->param);
}

void send_deref_expr (CmmDerefExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("*");
  send_cast_expr (p_item->param);
}

void send_addr_expr (CmmAddrExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("&");
  send_cast_expr (p_item->param);
}

void send_plus_expr (CmmPlusExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("+");
  send_cast_expr (p_item->param);
}

void send_minus_expr (CmmMinusExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("-");
  send_cast_expr (p_item->param);
}

void send_bit_not_expr (CmmBitNotExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("~");
  send_cast_expr (p_item->param);
}

void send_log_not_expr (CmmLogNotExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("!");
  send_cast_expr (p_item->param);
}

void send_sizeof_expr (CmmSizeofExp * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("sizeof");
  if (p_item->param1 != NULL)
  {
    send_unary_expr (p_item->param1);
  }
  else if (p_item->param2 != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("(");
    send_type_id (p_item->param2);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (")");
  }
  else assert (false);
}

void send_allocation_expr (CmmNewExpr * p_item)
{
  assert (p_item != NULL);
  CmmNewArrayLimit * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("new");
  if (p_item->spec != NULL)
  {
    send_alloc_type_specifier_list (p_item->spec);
    while (p_subitem != NULL)
    {
      send_allocation_array_limit (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  else if (p_item->type2 != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("(");
    send_type_id (p_item->type2);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (")");
  }
  else assert (false);
  if (p_item->init != NULL)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("(");
    send_expr_list (p_item->init);
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send (")");
  }
  assert (p_subitem == NULL);
}

void send_allocation_array_limit (CmmNewArrayLimit * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("[");
  send_expr (p_item->value);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("]");
}

void send_deallocation_expr (CmmDeleteExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("delete");
  if (p_item->a_array == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("[");
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("]");
    assert (p_item->a_array == true);
  }
  send_cast_expr (p_item->param);
}

void send_cast_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmCastFormula () != NULL)
  {
    send_cast_formula (p_item->conv_CmmCastFormula ());
  }
  else if (p_item->conv_CmmExpr () != NULL)
  {
    send_unary_expr (p_item);
  }
  else assert (false);
}

void send_cast_formula (CmmCastFormula * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("(");
  send_type_id (p_item->type);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (")");
  send_cast_expr (p_item->param);
}

void send_pm_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (DotMemberExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_pm_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (".*");
    send_cast_expr (p_item1->right);
  }
  else if (p_item->isBinary (ArrowMemverExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_pm_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("->*");
    send_cast_expr (p_item1->right);
  }
  else send_cast_expr (p_item);
}

void send_multiplicative_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (MulExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_multiplicative_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("*");
    send_pm_expr (p_item1->right);
  }
  else if (p_item->isBinary (DivExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_multiplicative_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("/");
    send_pm_expr (p_item1->right);
  }
  else if (p_item->isBinary (ModExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_multiplicative_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("%");
    send_pm_expr (p_item1->right);
  }
  else send_pm_expr (p_item);
}

void send_additive_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (AddExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_additive_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("+");
    send_multiplicative_expr (p_item1->right);
  }
  else if (p_item->isBinary (SubExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_additive_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("-");
    send_multiplicative_expr (p_item1->right);
  }
  else send_multiplicative_expr (p_item);
}

void send_shift_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (ShlExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_shift_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("<<");
    send_additive_expr (p_item1->right);
  }
  else if (p_item->isBinary (ShrExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_shift_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (">>");
    send_additive_expr (p_item1->right);
  }
  else send_additive_expr (p_item);
}

void send_relational_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (LtExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_relational_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("<");
    send_shift_expr (p_item1->right);
  }
  else if (p_item->isBinary (GtExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_relational_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (">");
    send_shift_expr (p_item1->right);
  }
  else if (p_item->isBinary (LeExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_relational_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("<=");
    send_shift_expr (p_item1->right);
  }
  else if (p_item->isBinary (GeExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_relational_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (">=");
    send_shift_expr (p_item1->right);
  }
  else send_shift_expr (p_item);
}

void send_equality_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (EqExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_equality_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("==");
    send_relational_expr (p_item1->right);
  }
  else if (p_item->isBinary (NeExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_equality_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("!=");
    send_relational_expr (p_item1->right);
  }
  else send_relational_expr (p_item);
}

void send_and_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (BitAndExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_and_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("&");
    send_equality_expr (p_item1->right);
  }
  else send_equality_expr (p_item);
}

void send_exclusive_or_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (BitXorExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_exclusive_or_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("^");
    send_and_expr (p_item1->right);
  }
  else send_and_expr (p_item);
}

void send_inclusive_or_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (BitOrExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_inclusive_or_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("|");
    send_exclusive_or_expr (p_item1->right);
  }
  else send_exclusive_or_expr (p_item);
}

void send_logical_and_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (LogAndExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_logical_and_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("&&");
    send_inclusive_or_expr (p_item1->right);
  }
  else send_inclusive_or_expr (p_item);
}

void send_logical_or_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (LogOrExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_logical_or_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("||");
    send_logical_and_expr (p_item1->right);
  }
  else send_logical_and_expr (p_item);
}

void send_conditional_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmCondExpr () != NULL)
  {
    CmmCondExpr * p_item1 = p_item->conv_CmmCondExpr ();
    send_logical_or_expr (p_item1->cond_expr);
    send_conditional_cont_expr (p_item1);
  }
  else send_logical_or_expr (p_item);
}

void send_conditional_cont_expr (CmmCondExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("?");
  send_expr (p_item->then_expr);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (":");
  send_conditional_expr (p_item->else_expr);
}

void send_assignment_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmExpr () != NULL)
  {
    send_simple_assignment_expr (p_item);
  }
  else if (p_item->conv_CmmThrowExpr () != NULL)
  {
    send_throw_expr (p_item->conv_CmmThrowExpr ());
  }
  else assert (false);
}

void send_simple_assignment_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (AssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (AddAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("+=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (SubAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("-=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (MulAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("*=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (DivAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("/=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (ModAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("%=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (ShlAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("<<=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (ShrAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (">>=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (AndAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("&=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (XorAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("^=");
    send_conditional_expr (p_item1->right);
  }
  else if (p_item->isBinary (OrAssignExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_simple_assignment_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send ("|=");
    send_conditional_expr (p_item1->right);
  }
  else send_conditional_expr (p_item);
}

void send_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  if (p_item->isBinary (CommaExp))
  {
    CmmBinaryExpr * p_item1 = p_item->conv_CmmBinaryExpr ();
    assert (p_item1 != NULL);
    send_expr (p_item1->left);
    sendStartNote (p_item1->start_note [0]);
    sendLocation (p_item1->location [0]);
    send (",");
    send_assignment_expr (p_item1->right);
  }
  else send_assignment_expr (p_item);
}

void send_const_expr (CmmExpr * p_item)
{
  assert (p_item != NULL);
  send_conditional_expr (p_item);
}

void send_stat_list (CmmStatSect * p_item)
{
  assert (p_item != NULL);
  CmmStat * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_stat (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_stat (CmmStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmExprStat () != NULL)
  {
    style_indent ();
    send_expression_stat (p_item->conv_CmmExprStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmDeclStat () != NULL)
  {
    style_indent ();
    send_declaration_stat (p_item->conv_CmmDeclStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmCompoundStat () != NULL)
  {
    style_indent ();
    send_compound_stat (p_item->conv_CmmCompoundStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmLabeledStat () != NULL)
  {
    style_indent ();
    send_labeled_stat (p_item->conv_CmmLabeledStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmCaseStat () != NULL)
  {
    style_indent ();
    send_case_stat (p_item->conv_CmmCaseStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmDefaultStat () != NULL)
  {
    style_indent ();
    send_default_stat (p_item->conv_CmmDefaultStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmEmptyStat () != NULL)
  {
    style_indent ();
    send_empty_stat (p_item->conv_CmmEmptyStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmIfStat () != NULL)
  {
    style_indent ();
    send_if_stat (p_item->conv_CmmIfStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmSwitchStat () != NULL)
  {
    style_indent ();
    send_switch_stat (p_item->conv_CmmSwitchStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmWhileStat () != NULL)
  {
    style_indent ();
    send_while_stat (p_item->conv_CmmWhileStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmDoStat () != NULL)
  {
    style_indent ();
    send_do_stat (p_item->conv_CmmDoStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmForStat () != NULL)
  {
    style_indent ();
    send_for_stat (p_item->conv_CmmForStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmBreakStat () != NULL)
  {
    style_indent ();
    send_break_stat (p_item->conv_CmmBreakStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmContinueStat () != NULL)
  {
    style_indent ();
    send_continue_stat (p_item->conv_CmmContinueStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmReturnStat () != NULL)
  {
    style_indent ();
    send_return_stat (p_item->conv_CmmReturnStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmGotoStat () != NULL)
  {
    style_indent ();
    send_goto_stat (p_item->conv_CmmGotoStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmTryStat () != NULL)
  {
    style_indent ();
    send_try_stat (p_item->conv_CmmTryStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmDirectiveStat () != NULL)
  {
    style_indent ();
    send_directive_stat (p_item->conv_CmmDirectiveStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmIntoStat () != NULL)
  {
    style_indent ();
    send_into_stat (p_item->conv_CmmIntoStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmDefineStat () != NULL)
  {
    style_indent ();
    send_define_stat (p_item->conv_CmmDefineStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmGroupStat () != NULL)
  {
    style_indent ();
    send_group_stat (p_item->conv_CmmGroupStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmIndentStat () != NULL)
  {
    style_indent ();
    send_indent_stat (p_item->conv_CmmIndentStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmWhenStat () != NULL)
  {
    style_indent ();
    send_when_stat (p_item->conv_CmmWhenStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmCallStat () != NULL)
  {
    style_indent ();
    send_call_stat (p_item->conv_CmmCallStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmSpaceStat () != NULL)
  {
    style_indent ();
    send_space_stat (p_item->conv_CmmSpaceStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmLocationStat () != NULL)
  {
    style_indent ();
    send_location_stat (p_item->conv_CmmLocationStat ());
    style_unindent ();
  }
  else if (p_item->conv_CmmSuperStat () != NULL)
  {
    style_indent ();
    send_super_stat (p_item->conv_CmmSuperStat ());
    style_unindent ();
  }
  // additional
  else if (p_item->conv_CmmClusterStat () != NULL)
  {
    send_cluster_stat (p_item->conv_CmmClusterStat ());
  }
  else assert (false);
}

void send_inner_stat (CmmStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmCompoundStat () != NULL)
  {
    style_new_line ();
    send_compound_stat (p_item->conv_CmmCompoundStat ());
    style_new_line ();
  }
  else if (p_item->conv_CmmStat () != NULL)
  {
    send_stat (p_item);
  }
  else assert (false);
}

void send_tail_stat (CmmStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmIfStat () != NULL)
  {
    send_if_stat (p_item->conv_CmmIfStat ());
  }
  else if (p_item->conv_CmmCompoundStat () != NULL)
  {
    style_new_line ();
    send_compound_stat (p_item->conv_CmmCompoundStat ());
    style_new_line ();
  }
  else if (p_item->conv_CmmStat () != NULL)
  {
    send_stat (p_item);
  }
  else assert (false);
}

void send_declaration_stat (CmmDeclStat * p_item)
{
  assert (p_item != NULL);
  send_block_declaration (p_item->decl);
}

void send_labeled_stat (CmmLabeledStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_identifier (p_item->lab);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (":");
  send_stat (p_item->body);
}

void send_case_stat (CmmCaseStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("case");
  send_expr (p_item->case_expr);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (":");
  send_inner_stat (p_item->body);
}

void send_default_stat (CmmDefaultStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("default");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (":");
  send_inner_stat (p_item->body);
}

void send_expression_stat (CmmExprStat * p_item)
{
  assert (p_item != NULL);
  send_expr (p_item->inner_expr);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send (";");
}

void send_empty_stat (CmmEmptyStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send (";");
}

void send_compound_stat (CmmCompoundStat * p_item)
{
  assert (p_item != NULL);
  open_compound_stat (p_item);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("{");
  style_no_empty_line ();
  send_stat_list (p_item->body);
  style_no_empty_line ();
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("}");
  close_compound_stat (p_item);
}

void send_condition (CmmCondition * p_item)
{
  assert (p_item != NULL);
  send_expr (p_item->cond_expr);
}

void send_if_stat (CmmIfStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("if");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  send_condition (p_item->cond);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
  send_inner_stat (p_item->then_stat);
  if (p_item->else_stat != NULL)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("else");
    send_tail_stat (p_item->else_stat);
  }
}

void send_switch_stat (CmmSwitchStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("switch");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  send_condition (p_item->cond);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
  send_inner_stat (p_item->body);
}

void send_while_stat (CmmWhileStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("while");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  send_condition (p_item->cond);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
  send_inner_stat (p_item->body);
}

void send_do_stat (CmmDoStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("do");
  send_inner_stat (p_item->body);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("while");
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send ("(");
  send_expr (p_item->cond_expr);
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (")");
  sendStartNote (p_item->start_note [4]);
  sendLocation (p_item->location [4]);
  send (";");
}

void send_for_stat (CmmForStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("for");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  if (p_item->from_decl != NULL)
  {
    send_simple_declaration (p_item->from_decl);
  }
  else if (p_item->from_expr != NULL)
  {
    send_expr (p_item->from_expr);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (";");
  }
  else if (p_item->empty_param == true)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send (";");
    assert (p_item->empty_param == true);
  }
  else assert (false);
  if (p_item->to_expr != NULL)
  {
    send_expr (p_item->to_expr);
  }
  sendStartNote (p_item->start_note [4]);
  sendLocation (p_item->location [4]);
  send (";");
  if (p_item->cond_expr != NULL)
  {
    send_expr (p_item->cond_expr);
  }
  sendStartNote (p_item->start_note [5]);
  sendLocation (p_item->location [5]);
  send (")");
  send_inner_stat (p_item->body);
}

void send_break_stat (CmmBreakStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("break");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (";");
}

void send_continue_stat (CmmContinueStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("continue");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (";");
}

void send_return_stat (CmmReturnStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("return");
  if (p_item->return_expr != NULL)
  {
    send_expr (p_item->return_expr);
  }
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (";");
}

void send_goto_stat (CmmGotoStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("goto");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send_identifier (p_item->goto_lab);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (";");
}

void send_declaration_list (CmmDeclSect * p_item)
{
  assert (p_item != NULL);
  CmmDecl * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_declaration (p_subitem);
    p_subitem = p_subitem->next;
    style_new_line ();
  }
  assert (p_subitem == NULL);
}

void send_declaration (CmmDecl * p_item)
{
  assert (p_item != NULL);
  open_declaration (p_item);
  if (p_item->conv_CmmCtorDecl () != NULL)
  {
    send_constructor_declaration (p_item->conv_CmmCtorDecl ());
  }
  else if (p_item->conv_CmmFuncDecl () != NULL)
  {
    send_function_declaration (p_item->conv_CmmFuncDecl ());
  }
  else if (p_item->conv_CmmSimpleDecl () != NULL)
  {
    send_simple_declaration (p_item->conv_CmmSimpleDecl ());
  }
  else if (p_item->conv_CmmEmptyDecl () != NULL)
  {
    send_empty_declaration (p_item->conv_CmmEmptyDecl ());
  }
  else if (p_item->conv_CmmNamespace () != NULL)
  {
    send_namespace_definition (p_item->conv_CmmNamespace ());
  }
  else if (p_item->conv_CmmUsingDecl () != NULL)
  {
    send_using_declaration (p_item->conv_CmmUsingDecl ());
  }
  else if (p_item->conv_CmmAsmDecl () != NULL)
  {
    send_asm_declaration (p_item->conv_CmmAsmDecl ());
  }
  else if (p_item->conv_CmmTemplateDecl () != NULL)
  {
    send_template_declaration (p_item->conv_CmmTemplateDecl ());
  }
  else if (p_item->conv_CmmExternDecl () != NULL)
  {
    send_linkage_specification (p_item->conv_CmmExternDecl ());
  }
  else if (p_item->conv_CmmDirective () != NULL)
  {
    send_directive (p_item->conv_CmmDirective ());
  }
  else if (p_item->conv_CmmNamespaceExtension () != NULL)
  {
    send_namespace_extension (p_item->conv_CmmNamespaceExtension ());
  }
  // additional
  else if (p_item->conv_CmmMemberVisibility () != NULL)
  {
    send_member_visibility (p_item->conv_CmmMemberVisibility ());
  }
  // additional
  else if (p_item->conv_CmmClassDecl () != NULL)
  {
    send_class_declaration (p_item->conv_CmmClassDecl ());
  }
  // additional
  else if (p_item->conv_CmmEnumDecl () != NULL)
  {
    send_enum_declaration (p_item->conv_CmmEnumDecl ());
  }
  else assert (false);
  close_declaration (p_item);
}

void send_block_declaration (CmmDecl * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmSimpleDecl () != NULL)
  {
    send_simple_declaration (p_item->conv_CmmSimpleDecl ());
  }
  else if (p_item->conv_CmmEmptyDecl () != NULL)
  {
    send_empty_declaration (p_item->conv_CmmEmptyDecl ());
  }
  else if (p_item->conv_CmmNamespace () != NULL)
  {
    send_namespace_definition (p_item->conv_CmmNamespace ());
  }
  else if (p_item->conv_CmmUsingDecl () != NULL)
  {
    send_using_declaration (p_item->conv_CmmUsingDecl ());
  }
  else if (p_item->conv_CmmAsmDecl () != NULL)
  {
    send_asm_declaration (p_item->conv_CmmAsmDecl ());
  }
  else assert (false);
}

/*
void send_constructor_head (CmmCtorHead * p_item)
{
  assert (p_item != NULL);
  send_constructor_decl_specifiers (p_item->spec);
  if (p_item->destructor == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("~");
    assert (p_item->destructor == true);
  }
  send_declarator_id (p_item->name);
  send_cont_function_declarator (p_item->cont);
  if (p_item->init != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send (":");
    send_ctor_initializer (p_item->init);
  }
  if (p_item->body == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("{");
    assert (p_item->body == true);
  }
  else if (p_item->try_block == true)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("try");
    assert (p_item->try_block == true);
  }
  else if (p_item->empty == true)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send (";");
    assert (p_item->empty == true);
  }
  else assert (false);
}
*/

void send_constructor_declaration (CmmCtorDecl * p_item)
{
  assert (p_item != NULL);
  open_constructor_declaration (p_item);
  send_constructor_decl_specifiers (p_item->spec);
  if (p_item->destructor == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("~");
    assert (p_item->destructor == true);
  }
  send_declarator (p_item->decl);
  if (p_item->statements != NULL)
  {
    if (p_item->init != NULL)
    {
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send (":");
      send_ctor_initializer (p_item->init);
    }
    style_new_line ();
    send_compound_stat (p_item->statements);
  }
  else if (p_item->statements != NULL && p_item->handlers != NULL)
  {
    style_new_line ();
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("try");
    if (p_item->init != NULL)
    {
      sendStartNote (p_item->start_note [3]);
      sendLocation (p_item->location [3]);
      send (":");
      send_ctor_initializer (p_item->init);
    }
    style_new_line ();
    send_compound_stat (p_item->statements);
    send_handler_list (p_item->handlers);
  }
  else if (p_item->empty == true)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send (";");
    assert (p_item->empty == true);
  }
  else assert (false);
  close_constructor_declaration (p_item);
}

void send_constructor_decl_specifiers (CmmSpecSect * p_item)
{
  assert (p_item != NULL);
  CmmSpec * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_constructor_decl_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_constructor_decl_specifier (CmmSpec * p_item)
{
  assert (p_item != NULL);
  if (p_item->kind == a_inline)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("inline");
    assert (p_item->kind == a_inline);
  }
  else if (p_item->kind == a_virtual)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("virtual");
    assert (p_item->kind == a_virtual);
  }
  else if (p_item->kind == a_explicit)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("explicit");
    assert (p_item->kind == a_explicit);
  }
  else assert (false);
}

/*
void send_function_head (CmmFuncHead * p_item)
{
  assert (p_item != NULL);
  send_decl_specifiers (p_item->spec);
  send_ptr_specifier_list (p_item->ptr);
  send_declarator_id (p_item->name);
  send_cont_function_declarator (p_item->params);
  send_cont_declarator_list (p_item->cont);
  if (p_item->body == true)
  {
    style_new_line ();
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("{");
    assert (p_item->body == true);
  }
  else if (p_item->try_block == true)
  {
    style_new_line ();
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("try");
    assert (p_item->try_block == true);
  }
  else if (p_item->empty == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (";");
    assert (p_item->empty == true);
  }
  else assert (false);
}
*/

void send_function_declaration (CmmFuncDecl * p_item)
{
  assert (p_item != NULL);
  open_function_declaration (p_item);
  send_decl_specifiers (p_item->spec);
  send_declarator (p_item->decl);
  if (p_item->statements != NULL)
  {
    style_new_line ();
    send_compound_stat (p_item->statements);
  }
  else if (p_item->statements != NULL && p_item->handlers != NULL)
  {
    style_new_line ();
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("try");
    send_compound_stat (p_item->statements);
    send_handler_list (p_item->handlers);
  }
  else if (p_item->empty == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send (";");
    assert (p_item->empty == true);
  }
  else assert (false);
  close_function_declaration (p_item);
}

void send_simple_declaration (CmmSimpleDecl * p_item)
{
  assert (p_item != NULL);
  CmmInitDeclarator * p_subitem = p_item->first;
  send_decl_specifiers (p_item->spec);
  if (p_subitem != NULL)
  {
    send_init_declarator (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send (",");
      send_init_declarator (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (";");
  assert (p_subitem == NULL);
}

void send_init_declarator (CmmInitDeclarator * p_item)
{
  assert (p_item != NULL);
  send_declarator (p_item->decl);
  if (p_item->init != NULL)
  {
    send_initializer (p_item->init);
  }
}

/*
void send_empty_declaration (CmmEmptyDecl * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send (";");
}
*/

void send_decl_specifiers (CmmDeclSpec * p_item)
{
  assert (p_item != NULL);
  send_simple_decl_specifiers (p_item->mode);
  if (p_item->name != NULL)
  {
    send_id_expr (p_item->name);
    if (p_item->template_args != NULL)
    {
      send_template_arg_list (p_item->template_args);
    }
  }
  else if (p_item->class_spec != NULL)
  {
    send_class_specifier (p_item->class_spec);
  }
  else if (p_item->enum_spec != NULL)
  {
    send_enum_specifier (p_item->enum_spec);
  }
  else if (p_item->type != NULL)
  {
    send_simple_type_specifiers (p_item->type);
  }
  else assert (false);
  send_simple_decl_specifiers (p_item->mode2);
}

void send_simple_decl_specifiers (CmmSpecSect * p_item)
{
  assert (p_item != NULL);
  CmmSpec * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_simple_decl_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_simple_decl_specifier (CmmSpec * p_item)
{
  assert (p_item != NULL);
  if (p_item->kind == a_auto)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("auto");
    assert (p_item->kind == a_auto);
  }
  else if (p_item->kind == a_register)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("register");
    assert (p_item->kind == a_register);
  }
  else if (p_item->kind == a_static)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("static");
    assert (p_item->kind == a_static);
  }
  else if (p_item->kind == a_extern)
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("extern");
    assert (p_item->kind == a_extern);
  }
  else if (p_item->kind == a_mutable)
  {
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send ("mutable");
    assert (p_item->kind == a_mutable);
  }
  else if (p_item->kind == a_inline)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("inline");
    assert (p_item->kind == a_inline);
  }
  else if (p_item->kind == a_virtual)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("virtual");
    assert (p_item->kind == a_virtual);
  }
  else if (p_item->kind == a_explicit)
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("explicit");
    assert (p_item->kind == a_explicit);
  }
  else if (p_item->kind == a_friend)
  {
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send ("friend");
    assert (p_item->kind == a_friend);
  }
  else if (p_item->kind == a_typedef)
  {
    sendStartNote (p_item->start_note [12]);
    sendLocation (p_item->location [12]);
    send ("typedef");
    assert (p_item->kind == a_typedef);
  }
  else if (p_item->kind == a_const)
  {
    sendStartNote (p_item->start_note [13]);
    sendLocation (p_item->location [13]);
    send ("const");
    assert (p_item->kind == a_const);
  }
  else if (p_item->kind == a_volatile)
  {
    sendStartNote (p_item->start_note [14]);
    sendLocation (p_item->location [14]);
    send ("volatile");
    assert (p_item->kind == a_volatile);
  }
  else assert (false);
}

void send_simple_type_specifiers (CmmTypeSect * p_item)
{
  assert (p_item != NULL);
  CmmType * p_subitem = p_item->first;
  send_simple_type_specifier (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    send_simple_type_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_simple_type_specifier (CmmType * p_item)
{
  assert (p_item != NULL);
  if (p_item->kind == a_char)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("char");
    assert (p_item->kind == a_char);
  }
  else if (p_item->kind == a_wchar)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("wchar_t");
    assert (p_item->kind == a_wchar);
  }
  else if (p_item->kind == a_bool)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("bool");
    assert (p_item->kind == a_bool);
  }
  else if (p_item->kind == a_short)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("short");
    assert (p_item->kind == a_short);
  }
  else if (p_item->kind == a_int)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("int");
    assert (p_item->kind == a_int);
  }
  else if (p_item->kind == a_long)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("long");
    assert (p_item->kind == a_long);
  }
  else if (p_item->kind == a_signed)
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("signed");
    assert (p_item->kind == a_signed);
  }
  else if (p_item->kind == a_unsigned)
  {
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send ("unsigned");
    assert (p_item->kind == a_unsigned);
  }
  else if (p_item->kind == a_float)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("float");
    assert (p_item->kind == a_float);
  }
  else if (p_item->kind == a_double)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("double");
    assert (p_item->kind == a_double);
  }
  else if (p_item->kind == a_void)
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("void");
    assert (p_item->kind == a_void);
  }
  else assert (false);
}

void send_alloc_type_specifier_list (CmmAllocTypeSpecSect * p_item)
{
  assert (p_item != NULL);
  CmmAllocTypeSpec * p_subitem = p_item->first;
  send_alloc_type_specifier (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    send_alloc_type_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_alloc_type_specifier (CmmAllocTypeSpec * p_item)
{
  assert (p_item != NULL);
  if (p_item->a_const == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("const");
    assert (p_item->a_const == true);
  }
  else if (p_item->a_volatile == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("volatile");
    assert (p_item->a_volatile == true);
  }
  else if (p_item->name != NULL)
  {
    send_id_expr (p_item->name);
  }
  else if (p_item->class_spec != NULL)
  {
    send_class_specifier (p_item->class_spec);
  }
  else if (p_item->enum_spec != NULL)
  {
    send_enum_specifier (p_item->enum_spec);
  }
  else if (p_item->simple != NULL)
  {
    send_simple_type_specifier (p_item->simple);
  }
  else assert (false);
}

void send_enum_specifier (CmmEnumDecl * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("enum");
  if (p_item->name != NULL)
  {
    send_id_expr (p_item->name);
  }
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("{");
  send_enum_list (p_item->items);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send ("}");
}

void send_enum_list (CmmEnumSect * p_item)
{
  assert (p_item != NULL);
  CmmEnumItem * p_subitem = p_item->first;
  if (p_subitem != NULL)
  {
    send_enumerator (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send (",");
      send_enumerator (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  assert (p_subitem == NULL);
}

void send_enumerator (CmmEnumItem * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_identifier (p_item->name);
  if (p_item->init != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("=");
    send_const_expr (p_item->init);
  }
}

void send_namespace_definition (CmmNamespace * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("namespace");
  if (p_item->unnamed_body != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("{");
    send_declaration_list (p_item->unnamed_body);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("}");
  }
  else if (! p_item->namespace_name.empty())
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send_identifier (p_item->namespace_name);
    if (p_item->value != NULL)
    {
      sendStartNote (p_item->start_note [4]);
      sendLocation (p_item->location [4]);
      send ("=");
      send_id_expr (p_item->value);
      sendStartNote (p_item->start_note [5]);
      sendLocation (p_item->location [5]);
      send (";");
    }
    else if (p_item->body != NULL)
    {
      sendStartNote (p_item->start_note [6]);
      sendLocation (p_item->location [6]);
      send ("{");
      send_declaration_list (p_item->body);
      sendStartNote (p_item->start_note [7]);
      sendLocation (p_item->location [7]);
      send ("}");
    }
    else assert (false);
  }
  else assert (false);
}

void send_using_declaration (CmmUsingDecl * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("using");
  if (p_item->a_namespace == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("namespace");
    assert (p_item->a_namespace == true);
  }
  else if (p_item->a_typename == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("typename");
    assert (p_item->a_typename == true);
  }
  send_id_expr (p_item->name);
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (";");
}

void send_linkage_specification (CmmExternDecl * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("extern");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send_string_literal (p_item->language);
  if (p_item->decl_list != NULL)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("{");
    send_declaration_list (p_item->decl_list);
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("}");
  }
  else if (p_item->decl != NULL)
  {
    send_declaration (p_item->decl);
  }
  else assert (false);
}

void send_string_literal (string p_item)
{
  send (p_item);
}

void send_asm_declaration (CmmAsmDecl * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("asm");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send_string_literal (p_item->instruction);
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (")");
  sendStartNote (p_item->start_note [4]);
  sendLocation (p_item->location [4]);
  send (";");
}

void send_type_id (CmmTypeId * p_item)
{
  assert (p_item != NULL);
  send_decl_specifiers (p_item->spec);
  send_abstract_declarator (p_item->decl);
}

void send_common_type_id (CmmCommonTypeId * p_item)
{
  assert (p_item != NULL);
  send_decl_specifiers (p_item->spec);
  send_common_declarator (p_item->decl);
}

void send_declarator (CmmDeclarator * p_item)
{
  assert (p_item != NULL);
  send_ptr_specifier_list (p_item->ptr);
  if (p_item->name != NULL)
  {
    send_declarator_id (p_item->name);
  }
  else if (p_item->inner != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("(");
    send_declarator (p_item->inner);
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send (")");
  }
  else assert (false);
  send_cont_declarator_list (p_item->cont);
}

void send_abstract_declarator (CmmAbstractDeclarator * p_item)
{
  assert (p_item != NULL);
  send_ptr_specifier_list (p_item->ptr);
  if (p_item->inner != NULL && p_item->cont != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("(");
    send_abstract_declarator (p_item->inner);
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send (")");
    send_cont_declarator_list (p_item->cont);
  }
}

void send_common_declarator (CmmCommonDeclarator * p_item)
{
  assert (p_item != NULL);
  send_ptr_specifier_list (p_item->ptr);
  if (p_item->cont != NULL)
  {
    if (p_item->name != NULL)
    {
      send_declarator_id (p_item->name);
    }
    else if (p_item->inner != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send ("(");
      send_common_declarator (p_item->inner);
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send (")");
    }
    else assert (false);
    send_cont_declarator_list (p_item->cont);
  }
}

void send_cont_declarator_list (CmmContDeclaratorSect * p_item)
{
  assert (p_item != NULL);
  CmmContDeclarator * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_cont_declarator (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_cont_declarator (CmmContDeclarator * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmContFunctionDeclarator () != NULL)
  {
    send_cont_function_declarator (p_item->conv_CmmContFunctionDeclarator ());
  }
  else if (p_item->conv_CmmContArrayDeclarator () != NULL)
  {
    send_cont_array_declarator (p_item->conv_CmmContArrayDeclarator ());
  }
  else assert (false);
}

void send_cont_function_declarator (CmmContFunctionDeclarator * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("(");
  send_parameter_declaration_list (p_item->parameters);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (")");
  send_cv_specifier_list (p_item->cv_spec);
  if (p_item->exception_spec != NULL)
  {
    send_exception_specification (p_item->exception_spec);
  }
}

void send_cont_array_declarator (CmmContArrayDeclarator * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("[");
  if (p_item->lim != NULL)
  {
    send_expr (p_item->lim);
  }
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("]");
}

void send_declarator_id (CmmNameExpr * p_item)
{
  assert (p_item != NULL);
  send_id_expr (p_item);
}

void send_ptr_specifier_list (CmmPtrSpecifierSect * p_item)
{
  assert (p_item != NULL);
  CmmPtrSpecifier * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_ptr_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_ptr_specifier (CmmPtrSpecifier * p_item)
{
  assert (p_item != NULL);
  if (p_item->pointer == true && p_item->cv_spec != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("*");
    send_cv_specifier_list (p_item->cv_spec);
    assert (p_item->pointer == true);
  }
  else if (p_item->reference == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("&");
    assert (p_item->reference == true);
  }
  else assert (false);
}

void send_cv_specifier_list (CmmCvSpecifierSect * p_item)
{
  assert (p_item != NULL);
  CmmCvSpecifier * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_cv_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_cv_specifier (CmmCvSpecifier * p_item)
{
  assert (p_item != NULL);
  if (p_item->a_const == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("const");
    assert (p_item->a_const == true);
  }
  else if (p_item->a_volatile == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("volatile");
    assert (p_item->a_volatile == true);
  }
  else assert (false);
}

void send_parameter_declaration_list (CmmParamSect * p_item)
{
  assert (p_item != NULL);
  CmmParamItem * p_subitem = p_item->first;
  if (p_subitem != NULL)
  {
    send_parameter_declaration (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send (",");
      send_parameter_declaration (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  assert (p_subitem == NULL);
}

void send_parameter_declaration (CmmParamItem * p_item)
{
  assert (p_item != NULL);
  if (p_item->type != NULL)
  {
    send_common_type_id (p_item->type);
    if (p_item->init != NULL)
    {
      sendStartNote (p_item->start_note [0]);
      sendLocation (p_item->location [0]);
      send ("=");
      send_assignment_expr (p_item->init);
    }
  }
  else if (p_item->dots == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("...");
    assert (p_item->dots == true);
  }
  else assert (false);
}

void send_initializer (CmmInitializer * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("=");
  send_initializer_item (p_item->value);
}

void send_initializer_item (CmmInitItem * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmInitSimple () != NULL)
  {
    send_simple_initializer (p_item->conv_CmmInitSimple ());
  }
  else if (p_item->conv_CmmInitList () != NULL)
  {
    send_initializer_list (p_item->conv_CmmInitList ());
  }
  else assert (false);
}

void send_simple_initializer (CmmInitSimple * p_item)
{
  assert (p_item != NULL);
  send_assignment_expr (p_item->expr);
}

void send_initializer_list (CmmInitList * p_item)
{
  assert (p_item != NULL);
  CmmInitItem * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("{");
  style_indent ();
  if (p_subitem != NULL)
  {
    send_initializer_item (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send (",");
      style_new_line ();
      send_initializer_item (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  style_unindent ();
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send ("}");
  assert (p_subitem == NULL);
}

void send_class_specifier (CmmClassDecl * p_item)
{
  assert (p_item != NULL);
  if (p_item->style == ClassStyle)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("class");
    assert (p_item->style == ClassStyle);
  }
  else if (p_item->style == StructStyle)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("struct");
    assert (p_item->style == StructStyle);
  }
  else if (p_item->style == UnionStyle)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("union");
    assert (p_item->style == UnionStyle);
  }
  else assert (false);
  if (p_item->anonymous == true && p_item->members != NULL)
  {
    if (p_item->base_list != NULL)
    {
      sendStartNote (p_item->start_note [3]);
      sendLocation (p_item->location [3]);
      send (":");
      send_base_specifier_list (p_item->base_list);
    }
    style_new_line ();
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("{");
    style_indent ();
    send_member_list (p_item->members);
    style_unindent ();
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("}");
    assert (p_item->anonymous == true);
  }
  else if (p_item->name != NULL)
  {
    send_id_expr (p_item->name);
    if (p_item->base_list != NULL && p_item->members != NULL)
    {
      sendStartNote (p_item->start_note [6]);
      sendLocation (p_item->location [6]);
      send (":");
      send_base_specifier_list (p_item->base_list);
      style_new_line ();
      sendStartNote (p_item->start_note [7]);
      sendLocation (p_item->location [7]);
      send ("{");
      style_indent ();
      send_member_list (p_item->members);
      style_unindent ();
      sendStartNote (p_item->start_note [8]);
      sendLocation (p_item->location [8]);
      send ("}");
    }
    else if (p_item->members != NULL)
    {
      style_new_line ();
      sendStartNote (p_item->start_note [9]);
      sendLocation (p_item->location [9]);
      send ("{");
      style_indent ();
      send_member_list (p_item->members);
      style_unindent ();
      sendStartNote (p_item->start_note [10]);
      sendLocation (p_item->location [10]);
      send ("}");
    }
  }
  else assert (false);
}

void send_member_list (CmmDeclSect * p_item)
{
  assert (p_item != NULL);
  open_member_list (p_item);
  CmmDecl * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_member_item (p_subitem);
    p_subitem = p_subitem->next;
    style_new_line ();
  }
  assert (p_subitem == NULL);
  close_member_list (p_item);
}

void send_member_item (CmmDecl * p_item)
{
  assert (p_item != NULL);
  open_member_item (p_item);
  if (p_item->conv_CmmMemberVisibility () != NULL)
  {
    send_member_visibility (p_item->conv_CmmMemberVisibility ());
  }
  else if (p_item->conv_CmmCtorDecl () != NULL)
  {
    style_indent ();
    send_constructor_declaration (p_item->conv_CmmCtorDecl ());
    style_unindent ();
  }
  else if (p_item->conv_CmmFuncDecl () != NULL)
  {
    style_indent ();
    send_function_declaration (p_item->conv_CmmFuncDecl ());
    style_unindent ();
  }
  else if (p_item->conv_CmmSimpleDecl () != NULL)
  {
    style_indent ();
    send_member_declaration (p_item->conv_CmmSimpleDecl ());
    style_unindent ();
  }
  else if (p_item->conv_CmmEmptyDecl () != NULL)
  {
    style_indent ();
    send_empty_declaration (p_item->conv_CmmEmptyDecl ());
    style_unindent ();
  }
  else if (p_item->conv_CmmUsingDecl () != NULL)
  {
    style_indent ();
    send_using_declaration (p_item->conv_CmmUsingDecl ());
    style_unindent ();
  }
  else if (p_item->conv_CmmTemplateDecl () != NULL)
  {
    style_indent ();
    send_template_declaration (p_item->conv_CmmTemplateDecl ());
    style_unindent ();
  }
  else assert (false);
  close_member_item (p_item);
}

void send_member_visibility (CmmMemberVisibility * p_item)
{
  assert (p_item != NULL);
  if (p_item->access == PrivateAccess)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("private");
    assert (p_item->access == PrivateAccess);
  }
  else if (p_item->access == ProtectedAccess)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("protected");
    assert (p_item->access == ProtectedAccess);
  }
  else if (p_item->access == PublicAccess)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("public");
    assert (p_item->access == PublicAccess);
  }
  else assert (false);
  style_no_space ();
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (":");
}

void send_member_declaration (CmmSimpleDecl * p_item)
{
  assert (p_item != NULL);
  CmmInitDeclarator * p_subitem = p_item->first;
  send_decl_specifiers (p_item->spec);
  if (p_subitem != NULL)
  {
    send_member_declarator (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [2]);
      sendLocation (p_item->location [2]);
      send (",");
      send_member_declarator (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (";");
  assert (p_subitem == NULL);
}

void send_member_declarator (CmmInitDeclarator * p_item)
{
  assert (p_item != NULL);
  send_declarator (p_item->decl);
  if (p_item->init != NULL)
  {
    send_initializer (p_item->init);
  }
  else if (p_item->width != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send (":");
    send_conditional_expr (p_item->width);
  }
}

void send_base_specifier_list (CmmBaseSect * p_item)
{
  assert (p_item != NULL);
  CmmBaseItem * p_subitem = p_item->first;
  send_base_specifier (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send (",");
    send_base_specifier (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_base_specifier (CmmBaseItem * p_item)
{
  assert (p_item != NULL);
  if (p_item->a_virtual == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("virtual");
    assert (p_item->a_virtual == true);
  }
  if (p_item->access == PrivateAccess)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("private");
    assert (p_item->access == PrivateAccess);
  }
  else if (p_item->access == ProtectedAccess)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("protected");
    assert (p_item->access == ProtectedAccess);
  }
  else if (p_item->access == PublicAccess)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("public");
    assert (p_item->access == PublicAccess);
  }
  else if (p_item->access == NoAccess)
  {
    assert (p_item->access == NoAccess);
  }
  else assert (false);
  send_id_expr (p_item->from);
  if (p_item->template_args != NULL)
  {
    send_template_arg_list (p_item->template_args);
  }
}

void send_ctor_initializer (CmmCtorInitializer * p_item)
{
  assert (p_item != NULL);
  CmmMemberInitializer * p_subitem = p_item->first;
  style_indent ();
  send_mem_initializer (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send (",");
    style_new_line ();
    send_mem_initializer (p_subitem);
    p_subitem = p_subitem->next;
  }
  style_unindent ();
  assert (p_subitem == NULL);
}

void send_mem_initializer (CmmMemberInitializer * p_item)
{
  assert (p_item != NULL);
  send_id_expr (p_item->name);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("(");
  send_expr_list (p_item->params);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (")");
}

void send_special_function (CmmSpecialFuction * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("operator");
  if (p_item->spec_new == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("new");
    assert (p_item->spec_new == true);
  }
  else if (p_item->spec_delete == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("delete");
    assert (p_item->spec_delete == true);
  }
  else if (p_item->spec_add == true)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("+");
    assert (p_item->spec_add == true);
  }
  else if (p_item->spec_sub == true)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("-");
    assert (p_item->spec_sub == true);
  }
  else if (p_item->spec_mul == true)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("*");
    assert (p_item->spec_mul == true);
  }
  else if (p_item->spec_div == true)
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("/");
    assert (p_item->spec_div == true);
  }
  else if (p_item->spec_mod == true)
  {
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send ("%");
    assert (p_item->spec_mod == true);
  }
  else if (p_item->spec_xor == true)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("^");
    assert (p_item->spec_xor == true);
  }
  else if (p_item->spec_and == true)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("&");
    assert (p_item->spec_and == true);
  }
  else if (p_item->spec_or == true)
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("|");
    assert (p_item->spec_or == true);
  }
  else if (p_item->spec_not == true)
  {
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send ("~");
    assert (p_item->spec_not == true);
  }
  else if (p_item->spec_log_not == true)
  {
    sendStartNote (p_item->start_note [12]);
    sendLocation (p_item->location [12]);
    send ("!");
    assert (p_item->spec_log_not == true);
  }
  else if (p_item->spec_eq == true)
  {
    sendStartNote (p_item->start_note [13]);
    sendLocation (p_item->location [13]);
    send ("=");
    assert (p_item->spec_eq == true);
  }
  else if (p_item->spec_lt == true)
  {
    sendStartNote (p_item->start_note [14]);
    sendLocation (p_item->location [14]);
    send ("<");
    assert (p_item->spec_lt == true);
  }
  else if (p_item->spec_gt == true)
  {
    sendStartNote (p_item->start_note [15]);
    sendLocation (p_item->location [15]);
    send (">");
    assert (p_item->spec_gt == true);
  }
  else if (p_item->spec_add_assign == true)
  {
    sendStartNote (p_item->start_note [16]);
    sendLocation (p_item->location [16]);
    send ("+=");
    assert (p_item->spec_add_assign == true);
  }
  else if (p_item->spec_sub_assign == true)
  {
    sendStartNote (p_item->start_note [17]);
    sendLocation (p_item->location [17]);
    send ("-=");
    assert (p_item->spec_sub_assign == true);
  }
  else if (p_item->spec_mul_assign == true)
  {
    sendStartNote (p_item->start_note [18]);
    sendLocation (p_item->location [18]);
    send ("*=");
    assert (p_item->spec_mul_assign == true);
  }
  else if (p_item->spec_div_assign == true)
  {
    sendStartNote (p_item->start_note [19]);
    sendLocation (p_item->location [19]);
    send ("/=");
    assert (p_item->spec_div_assign == true);
  }
  else if (p_item->spec_mod_assign == true)
  {
    sendStartNote (p_item->start_note [20]);
    sendLocation (p_item->location [20]);
    send ("%=");
    assert (p_item->spec_mod_assign == true);
  }
  else if (p_item->spec_xor_assign == true)
  {
    sendStartNote (p_item->start_note [21]);
    sendLocation (p_item->location [21]);
    send ("^=");
    assert (p_item->spec_xor_assign == true);
  }
  else if (p_item->spec_and_assign == true)
  {
    sendStartNote (p_item->start_note [22]);
    sendLocation (p_item->location [22]);
    send ("&=");
    assert (p_item->spec_and_assign == true);
  }
  else if (p_item->spec_or_assign == true)
  {
    sendStartNote (p_item->start_note [23]);
    sendLocation (p_item->location [23]);
    send ("|=");
    assert (p_item->spec_or_assign == true);
  }
  else if (p_item->spec_shl == true)
  {
    sendStartNote (p_item->start_note [24]);
    sendLocation (p_item->location [24]);
    send ("<<");
    assert (p_item->spec_shl == true);
  }
  else if (p_item->spec_shr == true)
  {
    sendStartNote (p_item->start_note [25]);
    sendLocation (p_item->location [25]);
    send (">>");
    assert (p_item->spec_shr == true);
  }
  else if (p_item->spec_shl_assign == true)
  {
    sendStartNote (p_item->start_note [26]);
    sendLocation (p_item->location [26]);
    send (">>=");
    assert (p_item->spec_shl_assign == true);
  }
  else if (p_item->spec_shr_assign == true)
  {
    sendStartNote (p_item->start_note [27]);
    sendLocation (p_item->location [27]);
    send ("<<=");
    assert (p_item->spec_shr_assign == true);
  }
  else if (p_item->spec_eq == true)
  {
    sendStartNote (p_item->start_note [28]);
    sendLocation (p_item->location [28]);
    send ("==");
    assert (p_item->spec_eq == true);
  }
  else if (p_item->spec_ne == true)
  {
    sendStartNote (p_item->start_note [29]);
    sendLocation (p_item->location [29]);
    send ("!=");
    assert (p_item->spec_ne == true);
  }
  else if (p_item->spec_le == true)
  {
    sendStartNote (p_item->start_note [30]);
    sendLocation (p_item->location [30]);
    send ("<=");
    assert (p_item->spec_le == true);
  }
  else if (p_item->spec_ge == true)
  {
    sendStartNote (p_item->start_note [31]);
    sendLocation (p_item->location [31]);
    send (">=");
    assert (p_item->spec_ge == true);
  }
  else if (p_item->spec_log_and == true)
  {
    sendStartNote (p_item->start_note [32]);
    sendLocation (p_item->location [32]);
    send ("&&");
    assert (p_item->spec_log_and == true);
  }
  else if (p_item->spec_log_or == true)
  {
    sendStartNote (p_item->start_note [33]);
    sendLocation (p_item->location [33]);
    send ("||");
    assert (p_item->spec_log_or == true);
  }
  else if (p_item->spec_inc == true)
  {
    sendStartNote (p_item->start_note [34]);
    sendLocation (p_item->location [34]);
    send ("++");
    assert (p_item->spec_inc == true);
  }
  else if (p_item->spec_dec == true)
  {
    sendStartNote (p_item->start_note [35]);
    sendLocation (p_item->location [35]);
    send ("--");
    assert (p_item->spec_dec == true);
  }
  else if (p_item->spec_comma == true)
  {
    sendStartNote (p_item->start_note [36]);
    sendLocation (p_item->location [36]);
    send (",");
    assert (p_item->spec_comma == true);
  }
  else if (p_item->spec_mebber_deref == true)
  {
    sendStartNote (p_item->start_note [37]);
    sendLocation (p_item->location [37]);
    send ("->*");
    assert (p_item->spec_mebber_deref == true);
  }
  else if (p_item->spec_deref == true)
  {
    sendStartNote (p_item->start_note [38]);
    sendLocation (p_item->location [38]);
    send ("->");
    assert (p_item->spec_deref == true);
  }
  else if (p_item->spec_call == true)
  {
    sendStartNote (p_item->start_note [39]);
    sendLocation (p_item->location [39]);
    send ("(");
    sendStartNote (p_item->start_note [40]);
    sendLocation (p_item->location [40]);
    send (")");
    assert (p_item->spec_call == true);
  }
  else if (p_item->spec_index == true)
  {
    sendStartNote (p_item->start_note [41]);
    sendLocation (p_item->location [41]);
    send ("[");
    sendStartNote (p_item->start_note [42]);
    sendLocation (p_item->location [42]);
    send ("]");
    assert (p_item->spec_index == true);
  }
  else assert (false);
}

void send_template_declaration (CmmTemplateDecl * p_item)
{
  assert (p_item != NULL);
  if (p_item->export_template == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("export");
    assert (p_item->export_template == true);
  }
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("template");
  if (p_item->parameter_block == true)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("<");
    if (p_item->params != NULL)
    {
      send_template_param_list (p_item->params);
    }
    else if (p_item->explicit_specialization == true)
    {
      assert (p_item->explicit_specialization == true);
    }
    else assert (false);
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send (">");
    assert (p_item->parameter_block == true);
  }
  else if (p_item->explicit_instatiation == true)
  {
    assert (p_item->explicit_instatiation == true);
  }
  else assert (false);
  send_declaration (p_item->body);
}

void send_template_param_list (CmmTemplateParamSect * p_item)
{
  assert (p_item != NULL);
  CmmTemplateParam * p_subitem = p_item->first;
  send_template_param (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send (",");
    send_template_param (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_template_param (CmmTemplateParam * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmTemplateExtraParam () != NULL)
  {
    send_template_extra_param (p_item->conv_CmmTemplateExtraParam ());
  }
  else if (p_item->conv_CmmTemplateTypeParam () != NULL)
  {
    send_template_type_param (p_item->conv_CmmTemplateTypeParam ());
  }
  else if (p_item->conv_CmmTemplateValueParam () != NULL)
  {
    send_template_value_param (p_item->conv_CmmTemplateValueParam ());
  }
  else assert (false);
}

void send_template_type_param (CmmTemplateTypeParam * p_item)
{
  assert (p_item != NULL);
  if (p_item->a_class == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("class");
    assert (p_item->a_class == true);
  }
  else if (p_item->a_typename == true)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("typename");
    assert (p_item->a_typename == true);
  }
  else assert (false);
  if (! p_item->name.empty())
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send_identifier (p_item->name);
  }
  if (p_item->value != NULL)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("=");
    send_type_id (p_item->value);
  }
}

void send_template_extra_param (CmmTemplateExtraParam * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("template");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("<");
  send_template_param_list (p_item->params);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (">");
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("class");
  if (! p_item->name.empty())
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send_identifier (p_item->name);
  }
  if (p_item->value != NULL)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("=");
    send_id_expr (p_item->value);
  }
}

void send_template_value_param (CmmTemplateValueParam * p_item)
{
  assert (p_item != NULL);
  send_common_type_id (p_item->type);
  if (p_item->init != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("=");
    send_shift_expr (p_item->init);
  }
}

void send_template_arg_list (CmmTemplateArgSect * p_item)
{
  assert (p_item != NULL);
  CmmTemplateArg * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("<");
  if (p_subitem != NULL)
  {
    send_template_arg (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send (",");
      send_template_arg (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (">");
  assert (p_subitem == NULL);
}

void send_template_arg (CmmTemplateArg * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_CmmTemplateTypeArg () != NULL)
  {
    send_template_type_arg (p_item->conv_CmmTemplateTypeArg ());
  }
  else if (p_item->conv_CmmTemplateValueArg () != NULL)
  {
    send_template_value_arg (p_item->conv_CmmTemplateValueArg ());
  }
  else assert (false);
}

void send_template_type_arg (CmmTemplateTypeArg * p_item)
{
  assert (p_item != NULL);
  send_type_id (p_item->type);
}

void send_template_value_arg (CmmTemplateValueArg * p_item)
{
  assert (p_item != NULL);
  send_shift_expr (p_item->value);
}

void send_try_stat (CmmTryStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("try");
  send_compound_stat (p_item->body);
  send_handler_list (p_item->handlers);
}

void send_handler_list (CmmHandlerSect * p_item)
{
  assert (p_item != NULL);
  CmmHandlerItem * p_subitem = p_item->first;
  send_handler (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    send_handler (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_handler (CmmHandlerItem * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("catch");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  send_exception_declaration (p_item->exception);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
  send_compound_stat (p_item->body);
}

void send_exception_declaration (CmmExceptionDecl * p_item)
{
  assert (p_item != NULL);
  if (p_item->type != NULL)
  {
    send_common_type_id (p_item->type);
  }
  else if (p_item->dots == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("...");
    assert (p_item->dots == true);
  }
  else assert (false);
}

void send_throw_expr (CmmThrowExpr * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("throw");
  if (p_item->expr != NULL)
  {
    send_assignment_expr (p_item->expr);
  }
}

void send_exception_specification (CmmExceptionSpec * p_item)
{
  assert (p_item != NULL);
  CmmTypeId * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("throw");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  if (p_subitem != NULL)
  {
    send_type_id (p_subitem);
    p_subitem = p_subitem->next;
    while (p_subitem != NULL)
    {
      sendStartNote (p_item->start_note [2]);
      sendLocation (p_item->location [2]);
      send (",");
      send_type_id (p_subitem);
      p_subitem = p_subitem->next;
    }
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (")");
  assert (p_subitem == NULL);
}

/*
void send_dir_start (string p_item)
{
  send (p_item);
}
*/

/*
void send_dir_end (string p_item)
{
  send (p_item);
}
*/

void send_dir_ident (string p_item)
{
  send (p_item);
}

void send_dir_number (string p_item)
{
  send (p_item);
}

void send_dir_q_string (string p_item)
{
  send (p_item);
}

void send_dir_h_string (string p_item)
{
  send (p_item);
}

void send_include_name (CmmIncludeName * p_item)
{
  assert (p_item != NULL);
  if (! p_item->include_q_str.empty())
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send_dir_q_string (p_item->include_q_str);
  }
  else if (! p_item->include_h_str.empty())
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send_dir_h_string (p_item->include_h_str);
  }
  else if (! p_item->include_id.empty())
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send_dir_ident (p_item->include_id);
  }
  else assert (false);
}

void send_directive (CmmDirective * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_dir_start (p_item->d1);
  if (p_item->include_directive == true && p_item->include_name != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("include");
    assert (p_item->include_directive == true);
    send_include_name (p_item->include_name);
  }
  else if (p_item->if_directive == true && ! p_item->if_expr.empty())
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("if");
    assert (p_item->if_directive == true);
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send_dir_number (p_item->if_expr);
  }
  else if (p_item->ifdef_directive == true && ! p_item->ifdef_name.empty())
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("ifdef");
    assert (p_item->ifdef_directive == true);
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send_dir_ident (p_item->ifdef_name);
  }
  else if (p_item->ifndef_directive == true && ! p_item->ifndef_name.empty())
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("ifndef");
    assert (p_item->ifndef_directive == true);
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send_dir_ident (p_item->ifndef_name);
  }
  else if (p_item->else_directive == true)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("else");
    assert (p_item->else_directive == true);
  }
  else if (p_item->endif_directive == true)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("endif");
    assert (p_item->endif_directive == true);
  }
  else if (p_item->define_directive == true && ! p_item->define_name.empty())
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("define");
    assert (p_item->define_directive == true);
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send_dir_ident (p_item->define_name);
  }
  else if (p_item->undef_directive == true && ! p_item->undef_name.empty())
  {
    sendStartNote (p_item->start_note [12]);
    sendLocation (p_item->location [12]);
    send ("undef");
    assert (p_item->undef_directive == true);
    sendStartNote (p_item->start_note [13]);
    sendLocation (p_item->location [13]);
    send_dir_ident (p_item->undef_name);
  }
  else assert (false);
  sendStartNote (p_item->start_note [14]);
  sendLocation (p_item->location [14]);
  send_dir_end (p_item->d2);
}

void send_directive_stat (CmmDirectiveStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_dir_start (p_item->d1);
  if (p_item->include_directive == true && p_item->include_name != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("include");
    assert (p_item->include_directive == true);
    send_include_name (p_item->include_name);
  }
  else if (p_item->if_directive == true && ! p_item->if_expr.empty())
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("if");
    assert (p_item->if_directive == true);
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send_dir_number (p_item->if_expr);
  }
  else if (p_item->ifdef_directive == true && ! p_item->ifdef_name.empty())
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("ifdef");
    assert (p_item->ifdef_directive == true);
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send_dir_ident (p_item->ifdef_name);
  }
  else if (p_item->ifndef_directive == true && ! p_item->ifndef_name.empty())
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("ifndef");
    assert (p_item->ifndef_directive == true);
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send_dir_ident (p_item->ifndef_name);
  }
  else if (p_item->else_directive == true)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("else");
    assert (p_item->else_directive == true);
  }
  else if (p_item->endif_directive == true)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("endif");
    assert (p_item->endif_directive == true);
  }
  else if (p_item->define_directive == true && ! p_item->define_name.empty())
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("define");
    assert (p_item->define_directive == true);
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send_dir_ident (p_item->define_name);
  }
  else if (p_item->undef_directive == true && ! p_item->undef_name.empty())
  {
    sendStartNote (p_item->start_note [12]);
    sendLocation (p_item->location [12]);
    send ("undef");
    assert (p_item->undef_directive == true);
    sendStartNote (p_item->start_note [13]);
    sendLocation (p_item->location [13]);
    send_dir_ident (p_item->undef_name);
  }
  else assert (false);
  sendStartNote (p_item->start_note [14]);
  sendLocation (p_item->location [14]);
  send_dir_end (p_item->d2);
}

void send_namespace_extension (CmmNamespaceExtension * p_item)
{
  assert (p_item != NULL);
  if (p_item->open == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("OPEN_NAMESPACE");
    assert (p_item->open == true);
  }
  else if (p_item->open == false)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("CLOSE_NAMESPACE");
    assert (p_item->open == false);
  }
  else assert (false);
}

void send_cluster_stat (CmmClusterStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->body != NULL)
  {
    send_stat_list (p_item->body);
  }
}

void send_class_declaration (CmmClassDecl * p_item)
{
  assert (p_item != NULL);
  send_class_specifier (p_item);
  sendStartNote (p_item->start_note [11]);
  sendLocation (p_item->location [11]);
  send (";");
}

void send_enum_declaration (CmmEnumDecl * p_item)
{
  assert (p_item != NULL);
  send_enum_specifier (p_item);
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send (";");
}

/*
void send_into_stat (CmmIntoStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->with_into == true)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("INTO");
    if (p_item->into_param != NULL)
    {
      send_expr (p_item->into_param);
    }
    if (p_item->store_param != NULL)
    {
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send ("STORE");
      send_expr (p_item->store_param);
    }
    assert (p_item->with_into == true);
  }
  else if (p_item->store_param != NULL)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("STORE");
    send_expr (p_item->store_param);
  }
  else assert (false);
  if (p_item->group != NULL)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("GROUP");
    send_expr (p_item->group);
  }
  if (p_item->comment != NULL)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("COMMENT");
    send_expr (p_item->comment);
  }
  if (p_item->def != NULL)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("DEFINE");
    assert (false); // no_send
  }
  if (p_item->prior == true)
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("PRIOR");
    assert (p_item->prior == true);
  }
  if (p_item->expr != NULL)
  {
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send ("EXPRESSION");
    send_expr (p_item->expr);
  }
  else if (p_item->expr_list != NULL)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("EXPRESSIONS");
    send_expr_list (p_item->expr_list);
  }
  else if (p_item->stat != NULL)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("STATEMENT");
    send_stat (p_item->stat);
  }
  else if (p_item->code != NULL)
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("CODE");
    send_stat_list (p_item->code);
  }
  else if (p_item->decl != NULL)
  {
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send ("DECLARATION");
    send_declaration (p_item->decl);
  }
  else if (p_item->decl_list != NULL)
  {
    sendStartNote (p_item->start_note [12]);
    sendLocation (p_item->location [12]);
    send ("DECLARATIONS");
    send_declaration_list (p_item->decl_list);
  }
  else if (p_item->param_list != NULL)
  {
    sendStartNote (p_item->start_note [13]);
    sendLocation (p_item->location [13]);
    send ("PARAMETERS");
    send_parameter_declaration_list (p_item->param_list);
  }
  else if (p_item->enum_type != NULL)
  {
    sendStartNote (p_item->start_note [14]);
    sendLocation (p_item->location [14]);
    send ("ENUMERATION_TYPE");
    send_enum_declaration (p_item->enum_type);
  }
  else if (p_item->enum_items != NULL)
  {
    sendStartNote (p_item->start_note [15]);
    sendLocation (p_item->location [15]);
    send ("ENUMERATION_ITEMS");
    send_enum_list (p_item->enum_items);
  }
  else if (p_item->cls != NULL)
  {
    sendStartNote (p_item->start_note [16]);
    sendLocation (p_item->location [16]);
    send ("CLASS");
    send_class_declaration (p_item->cls);
  }
  else if (p_item->base_list != NULL)
  {
    sendStartNote (p_item->start_note [17]);
    sendLocation (p_item->location [17]);
    send ("BASE");
    send_base_specifier_list (p_item->base_list);
  }
  else if (p_item->member != NULL)
  {
    if (p_item->a_public == true)
    {
      sendStartNote (p_item->start_note [18]);
      sendLocation (p_item->location [18]);
      send ("PUBLIC");
      assert (p_item->a_public == true);
    }
    else if (p_item->a_protected == true)
    {
      sendStartNote (p_item->start_note [19]);
      sendLocation (p_item->location [19]);
      send ("PROTECTED");
      assert (p_item->a_protected == true);
    }
    else if (p_item->a_private == true)
    {
      sendStartNote (p_item->start_note [20]);
      sendLocation (p_item->location [20]);
      send ("PRIVATE");
      assert (p_item->a_private == true);
    }
    else if (p_item->access != NULL)
    {
      sendStartNote (p_item->start_note [21]);
      sendLocation (p_item->location [21]);
      send ("ACCESS");
      send_expr (p_item->access);
    }
    sendStartNote (p_item->start_note [22]);
    sendLocation (p_item->location [22]);
    send ("MEMBER");
    send_member_item (p_item->member);
  }
  else if (p_item->members != NULL)
  {
    sendStartNote (p_item->start_note [23]);
    sendLocation (p_item->location [23]);
    send ("MEMBERS");
    send_member_list (p_item->members);
  }
  else if (p_item->ctor_init != NULL)
  {
    sendStartNote (p_item->start_note [24]);
    sendLocation (p_item->location [24]);
    send ("INITIALIZERS");
    send_ctor_initializer (p_item->ctor_init);
  }
  else if (p_item->init_item != NULL)
  {
    sendStartNote (p_item->start_note [25]);
    sendLocation (p_item->location [25]);
    send ("INITIALIZER_ITEM");
    send_initializer_item (p_item->init_item);
  }
  else if (p_item->init_list != NULL)
  {
    sendStartNote (p_item->start_note [26]);
    sendLocation (p_item->location [26]);
    send ("INITIALIZER_LIST");
    send_initializer_list (p_item->init_list);
  }
  else if (p_item->text_sect != NULL)
  {
    assert (false); // no_send
  }
  else assert (false);
  sendStartNote (p_item->start_note [27]);
  sendLocation (p_item->location [27]);
  send ("END");
}
*/

/*
void send_text_section (CmmTextSect * p_item)
{
  assert (p_item != NULL);
  CmmTextItem * p_subitem = p_item->first;
  if (p_item->indent != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("INDENT");
    send_expr (p_item->indent);
  }
  if (! p_item->shift.empty())
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("SHIFT");
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send_numeric_literal (p_item->shift);
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("TEXT_SECTION");
  while (p_subitem != NULL)
  {
    assert (false); // no_send
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}
*/

void send_numeric_literal (string p_item)
{
  send (p_item);
}

/*
void send_text_item (CmmTextItem * p_item)
{
  assert (p_item != NULL);
  if (p_item->token_flag == true && ! p_item->token_value.empty())
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send_txt_token (p_item->token_value);
    assert (p_item->token_flag == true);
  }
  else if (p_item->white_space_flag == true && ! p_item->space_value.empty())
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send_txt_white_space (p_item->space_value);
    assert (p_item->white_space_flag == true);
  }
  else if (p_item->new_line_flag == true && ! p_item->new_line_value.empty())
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send_txt_new_line (p_item->new_line_value);
    assert (p_item->new_line_flag == true);
  }
  else if (p_item->other_flag == true && ! p_item->other_value.empty())
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send_txt_other (p_item->other_value);
    assert (p_item->other_flag == true);
  }
  else assert (false);
}
*/

void send_txt_token (string p_item)
{
  send (p_item);
}

void send_txt_white_space (string p_item)
{
  send (p_item);
}

void send_txt_new_line (string p_item)
{
  send (p_item);
}

void send_txt_other (string p_item)
{
  send (p_item);
}

/*
void send_define_stat (CmmDefineStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("DEFINE");
  assert (false); // no_send
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("END");
}
*/

/*
void send_define_part (CmmDefineSect * p_item)
{
  assert (p_item != NULL);
  CmmDefineItem * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    assert (false); // no_send
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}
*/

/*
void send_define_item (CmmDefineItem * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_identifier (p_item->name);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send (":");
  if (p_item->mode == ident_mode)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("IDENT");
    assert (p_item->mode == ident_mode);
  }
  else if (p_item->mode == text_mode)
  {
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send ("TEXT");
    assert (p_item->mode == text_mode);
  }
  else if (p_item->mode == string_mode)
  {
    sendStartNote (p_item->start_note [4]);
    sendLocation (p_item->location [4]);
    send ("STRING");
    assert (p_item->mode == string_mode);
  }
  else if (p_item->mode == expr_mode)
  {
    sendStartNote (p_item->start_note [5]);
    sendLocation (p_item->location [5]);
    send ("EXPR");
    assert (p_item->mode == expr_mode);
  }
  else if (p_item->mode == expr_list_mode)
  {
    sendStartNote (p_item->start_note [6]);
    sendLocation (p_item->location [6]);
    send ("EXPR_LIST");
    assert (p_item->mode == expr_list_mode);
  }
  else if (p_item->mode == stat_mode)
  {
    sendStartNote (p_item->start_note [7]);
    sendLocation (p_item->location [7]);
    send ("STAT");
    assert (p_item->mode == stat_mode);
  }
  else if (p_item->mode == decl_mode)
  {
    sendStartNote (p_item->start_note [8]);
    sendLocation (p_item->location [8]);
    send ("DECL");
    assert (p_item->mode == decl_mode);
  }
  else if (p_item->mode == init_item_mode)
  {
    sendStartNote (p_item->start_note [9]);
    sendLocation (p_item->location [9]);
    send ("INIT_ITEM");
    assert (p_item->mode == init_item_mode);
  }
  else if (p_item->mode == init_list_mode)
  {
    sendStartNote (p_item->start_note [10]);
    sendLocation (p_item->location [10]);
    send ("INIT_LIST");
    assert (p_item->mode == init_list_mode);
  }
  else assert (false);
  if (p_item->value != NULL)
  {
    sendStartNote (p_item->start_note [11]);
    sendLocation (p_item->location [11]);
    send ("=");
    send_expr (p_item->value);
  }
  sendStartNote (p_item->start_note [12]);
  sendLocation (p_item->location [12]);
  send (";");
}
*/

/*
void send_group_stat (CmmGroupStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("GROUP");
  send_expr (p_item->value);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("END");
}
*/

/*
void send_indent_stat (CmmIndentStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("INDENT");
  send_expr (p_item->value);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("END");
}
*/

void send_when_stat (CmmWhenStat * p_item)
{
  assert (p_item != NULL);
  CmmWhenItem * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("WHEN");
  send_expr (p_item->cond_expr);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_stat_list (p_item->then_body);
  while (p_subitem != NULL)
  {
    send_when_item (p_subitem);
    p_subitem = p_subitem->next;
  }
  if (p_item->else_body != NULL)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("OTHERWISE");
    send_stat_list (p_item->else_body);
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("END");
  assert (p_subitem == NULL);
}

void send_when_item (CmmWhenItem * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("OR_WHEN");
  send_expr (p_item->sub_expr);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_stat_list (p_item->sub_body);
}

void send_call_stat (CmmCallStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("CALL");
  send_stat_list (p_item->body);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("END");
}

/*
void send_space_stat (CmmSpaceStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("SPACE");
}
*/

void send_location_stat (CmmLocationStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("LOCATION");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("(");
  send_expr (p_item->place);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (")");
}

/*
void send_super_stat (CmmSuperStat * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("SUPER");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("::");
  send_cast_expr (p_item->param);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send (";");
}
*/

void send_alt_name (AltName * p_item)
{
  assert (p_item != NULL);
  if (! p_item->id.empty())
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send_identifier (p_item->id);
  }
  else if (! p_item->txt.empty())
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send_string_literal (p_item->txt);
  }
  else assert (false);
}

void send_alt_simple (AltSimple * p_item)
{
  assert (p_item != NULL);
  if (p_item->name != NULL)
  {
    send_alt_name (p_item->name);
  }
  else if (p_item->opt != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("[");
    send_alt_expression (p_item->opt);
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("]");
  }
  else if (p_item->sub_expr != NULL)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("(");
    send_alt_expression (p_item->sub_expr);
    sendStartNote (p_item->start_note [3]);
    sendLocation (p_item->location [3]);
    send (")");
  }
  else assert (false);
}

void send_alt_branch (AltBranch * p_item)
{
  assert (p_item != NULL);
  AltSimple * p_subitem = p_item->first;
  send_alt_simple (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    send_alt_simple (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_alt_expression (AltExpr * p_item)
{
  assert (p_item != NULL);
  AltBranch * p_subitem = p_item->first;
  send_alt_branch (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("|");
    send_alt_branch (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_alt_item (DefItem * p_item)
{
  assert (p_item != NULL);
  send_alt_name (p_item->name);
}

void send_alt_list (DefList * p_item)
{
  assert (p_item != NULL);
  DefItem * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_alt_item (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_def_item (DefItem * p_item)
{
  assert (p_item != NULL);
  send_alt_name (p_item->name);
}

void send_def_list (DefList * p_item)
{
  assert (p_item != NULL);
  DefItem * p_subitem = p_item->first;
  send_def_item (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send (",");
    send_def_item (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_log_factor (LogFactor * p_item)
{
  assert (p_item != NULL);
  if (p_item->name != NULL)
  {
    send_alt_name (p_item->name);
  }
  else if (p_item->not_factor != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("!");
    send_log_factor (p_item->not_factor);
  }
  else if (p_item->sub_expr != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("(");
    send_log_expression (p_item->sub_expr);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (")");
  }
  else assert (false);
}

void send_log_term (LogTerm * p_item)
{
  assert (p_item != NULL);
  LogFactor * p_subitem = p_item->first;
  send_log_factor (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("&");
    send_log_factor (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_log_expression (LogExpr * p_item)
{
  assert (p_item != NULL);
  LogTerm * p_subitem = p_item->first;
  send_log_term (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("|");
    send_log_term (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_log_statement (LogStat * p_item)
{
  assert (p_item != NULL);
  send_alt_name (p_item->name);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("=");
  send_log_expression (p_item->expr);
}

void send_num_statements (NumStatSect * p_item)
{
  assert (p_item != NULL);
  NumStat * p_subitem = p_item->first;
  while (p_subitem != NULL)
  {
    send_num_statement (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_num_statement (NumStat * p_item)
{
  assert (p_item != NULL);
  if (p_item->conv_NumAssign () != NULL)
  {
    send_num_assign (p_item->conv_NumAssign ());
  }
  else if (p_item->conv_NumCode () != NULL)
  {
    send_num_code (p_item->conv_NumCode ());
  }
  else if (p_item->conv_NumWhen () != NULL)
  {
    send_num_when (p_item->conv_NumWhen ());
  }
  else if (p_item->conv_NumIndent () != NULL)
  {
    send_num_indent (p_item->conv_NumIndent ());
  }
  else if (p_item->conv_NumUnindent () != NULL)
  {
    send_num_unindent (p_item->conv_NumUnindent ());
  }
  else assert (false);
}

void send_num_code (NumCode * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("CODE");
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send_string_literal (p_item->code);
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send ("END");
}

void send_num_indent (NumIndent * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("INDENT");
}

void send_num_unindent (NumUnindent * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("UNINDENT");
}

void send_num_when (NumWhen * p_item)
{
  assert (p_item != NULL);
  NumStatAlternative * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("WHEN");
  send_num_cond (p_item->cond);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_num_statements (p_item->when_stat);
  while (p_subitem != NULL)
  {
    send_num_stat_alternative (p_subitem);
    p_subitem = p_subitem->next;
  }
  if (p_item->otherwise_stat != NULL)
  {
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send ("OTHERWISE");
    send_num_statements (p_item->otherwise_stat);
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("END");
  assert (p_subitem == NULL);
}

void send_num_stat_alternative (NumStatAlternative * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("OR_WHEN");
  send_num_cond (p_item->cond);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_num_statements (p_item->stat);
}

void send_num_assign (NumAssign * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_identifier (p_item->id);
  if (p_item->list != NULL)
  {
    sendStartNote (p_item->start_note [1]);
    sendLocation (p_item->location [1]);
    send ("(");
    send_expr_list (p_item->list);
    sendStartNote (p_item->start_note [2]);
    sendLocation (p_item->location [2]);
    send (")");
  }
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("=");
  if (p_item->expr != NULL)
  {
    send_expr (p_item->expr);
  }
  else if (p_item->selection != NULL)
  {
    send_num_selection (p_item->selection);
  }
  else assert (false);
  sendStartNote (p_item->start_note [4]);
  sendLocation (p_item->location [4]);
  send (";");
}

void send_num_selection (NumSelection * p_item)
{
  assert (p_item != NULL);
  NumExprAlternative * p_subitem = p_item->first;
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("WHEN");
  send_num_cond (p_item->cond);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_expr (p_item->when_expr);
  while (p_subitem != NULL)
  {
    send_num_expr_alternative (p_subitem);
    p_subitem = p_subitem->next;
  }
  sendStartNote (p_item->start_note [2]);
  sendLocation (p_item->location [2]);
  send ("OTHERWISE");
  send_expr (p_item->otherwise_expr);
  sendStartNote (p_item->start_note [3]);
  sendLocation (p_item->location [3]);
  send ("END");
  assert (p_subitem == NULL);
}

void send_num_expr_alternative (NumExprAlternative * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send ("OR_WHEN");
  send_num_cond (p_item->cond);
  sendStartNote (p_item->start_note [1]);
  sendLocation (p_item->location [1]);
  send ("DO");
  send_expr (p_item->expr);
}

void send_num_cond (NumCond * p_item)
{
  assert (p_item != NULL);
  NumAndCond * p_subitem = p_item->first;
  send_num_and_cond (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("or");
    send_num_and_cond (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_num_and_cond (NumAndCond * p_item)
{
  assert (p_item != NULL);
  NumCompCond * p_subitem = p_item->first;
  send_num_comp_cond (p_subitem);
  p_subitem = p_subitem->next;
  while (p_subitem != NULL)
  {
    sendStartNote (p_item->start_note [0]);
    sendLocation (p_item->location [0]);
    send ("and");
    send_num_comp_cond (p_subitem);
    p_subitem = p_subitem->next;
  }
  assert (p_subitem == NULL);
}

void send_num_comp_cond (NumCompCond * p_item)
{
  assert (p_item != NULL);
  sendStartNote (p_item->start_note [0]);
  sendLocation (p_item->location [0]);
  send_identifier (p_item->left_id);
  if (p_item->relation == true)
  {
    if (p_item->equal == true)
    {
      sendStartNote (p_item->start_note [1]);
      sendLocation (p_item->location [1]);
      send ("=");
      assert (p_item->equal == true);
    }
    else if (p_item->not_equal == true)
    {
      sendStartNote (p_item->start_note [2]);
      sendLocation (p_item->location [2]);
      send ("<>");
      assert (p_item->not_equal == true);
    }
    else assert (false);
    if (! p_item->right_id.empty())
    {
      sendStartNote (p_item->start_note [3]);
      sendLocation (p_item->location [3]);
      send_identifier (p_item->right_id);
    }
    else if (! p_item->right_val.empty())
    {
      sendStartNote (p_item->start_note [4]);
      sendLocation (p_item->location [4]);
      send_numeric_literal (p_item->right_val);
    }
    else assert (false);
    assert (p_item->relation == true);
  }
}

/* %% tree */

string conv_CmmAccess (CmmAccess p_item);
string conv_CmmSpecKind (CmmSpecKind p_item);
string conv_CmmTypeKind (CmmTypeKind p_item);
string conv_CmmDefineItemMode (CmmDefineItemMode p_item);
string conv_CmmValueExpr_kind (CmmValueExpr_kind p_item);
string conv_CmmModernCastExpr_kind (CmmModernCastExpr_kind p_item);
string conv_CmmBinaryExpr_kind (CmmBinaryExpr_kind p_item);
string conv_CmmClassDecl_style (CmmClassDecl_style p_item);

void CmmMemberVisibility::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_enum ("access", conv_CmmAccess (this->access), this->access);
}

void CmmBaseItem::produce_attributes ()
{
  store_location (location [0]);
  store_enum ("access", conv_CmmAccess (this->access), this->access);
  store_bool ("a_virtual", this->a_virtual);
  store_item ("from", this->from);
  store_item ("template_args", this->template_args);
}

void CmmSpec::produce_attributes ()
{
  store_location (location [0]);
  store_enum ("kind", conv_CmmSpecKind (this->kind), this->kind);
}

void CmmType::produce_attributes ()
{
  store_location (location [0]);
  store_enum ("kind", conv_CmmTypeKind (this->kind), this->kind);
}

void CmmDecl::produce_attributes ()
{
  store_string ("comment", this->comment);
  store_int ("group", this->group);
  store_bool ("one_line_comment", this->one_line_comment);
  store_bool ("insert_empty_line", this->insert_empty_line);
  store_bool ("ins_empty_line", this->ins_empty_line);
  store_bool ("one_line_function", this->one_line_function);
  store_bool ("save_newlines", this->save_newlines);
  store_bool ("redefine", this->redefine);
  store_enum ("redefine_access", conv_CmmAccess (this->redefine_access), this->redefine_access);
  store_enum ("required_access", conv_CmmAccess (this->required_access), this->required_access);
  store_string ("group_comment", this->group_comment);
}

void CmmCompoundStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("link", this->link);
  store_item ("actual_dictionary", this->actual_dictionary);
  store_item ("actual_group", this->actual_group);
  store_item ("actual_indent", this->actual_indent);
  store_item ("body", this->body);
}

void CmmDefineSect::produce_attributes ()
{
  store_item ("link", this->link);
}

void CmmDefineSect::produce_sequence ()
{
  CmmDefineItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmDefineItem::produce_attributes ()
{
  store_location (location [0]);
  store_enum ("mode", conv_CmmDefineItemMode (this->mode), this->mode);
  store_ident ("name", this->name);
  store_item ("value", this->value);
}

void CmmSuperStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("cls", this->cls);
  store_item ("param", this->param);
}

void CmmTextSect::produce_attributes ()
{
  store_location (location [0]);
  store_int ("keyword_column", this->keyword_column);
  store_item ("into_param", this->into_param);
  store_item ("indent_expr", this->indent_expr);
  store_int ("shift_value", this->shift_value);
  store_int ("first_column", this->first_column);
  store_bool ("start_line", this->start_line);
  store_string ("acc", this->acc);
  store_int ("spaces", this->spaces);
  store_item ("indent", this->indent);
  store_string ("shift", this->shift);
}

void CmmTextSect::produce_sequence ()
{
  CmmTextItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmContName::produce_attributes ()
{
  store_location (location [0]);
  store_bool ("destructor_name", this->destructor_name);
  store_bool ("template_name", this->template_name);
  store_ident ("id", this->id);
}

void CmmSimpName::produce_attributes ()
{
  store_location (location [0]);
  store_ident ("id", this->id);
  store_item ("special", this->special);
}

void CmmNameExpr::produce_attributes ()
{
  CmmExpr::produce_attributes ();
  store_item ("head", this->head);
}

void CmmNameExpr::produce_sequence ()
{
  CmmContName * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmValueExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_enum ("kind", conv_CmmValueExpr_kind (this->kind), this->kind);
  store_string ("value", this->value);
}

void CmmSubexprExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmModernCastExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_enum ("kind", conv_CmmModernCastExpr_kind (this->kind), this->kind);
  store_item ("type", this->type);
  store_item ("param", this->param);
}

void CmmTypeidExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param1", this->param1);
  store_item ("param2", this->param2);
}

void CmmIndexExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("left", this->left);
  store_item ("right", this->right);
}

void CmmCallExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
  store_item ("list", this->list);
}

void CmmFieldExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
  store_bool ("destructor_name", this->destructor_name);
  store_bool ("template_name", this->template_name);
  store_item ("name", this->name);
}

void CmmPtrFieldExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
  store_bool ("destructor_name", this->destructor_name);
  store_bool ("template_name", this->template_name);
  store_item ("name", this->name);
}

void CmmPostIncExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmPostDecExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmExprList::produce_sequence ()
{
  CmmExpr * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmIncExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmDecExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmDerefExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmAddrExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmPlusExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmMinusExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmBitNotExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmLogNotExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param", this->param);
}

void CmmSizeofExp::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("param1", this->param1);
  store_item ("param2", this->param2);
}

void CmmNewExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("spec", this->spec);
  store_item ("type2", this->type2);
  store_item ("init", this->init);
}

void CmmNewExpr::produce_sequence ()
{
  CmmNewArrayLimit * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmNewArrayLimit::produce_attributes ()
{
  store_location (location [0]);
  store_item ("value", this->value);
}

void CmmDeleteExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_bool ("a_array", this->a_array);
  store_item ("param", this->param);
}

void CmmCastFormula::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("type", this->type);
  store_item ("param", this->param);
}

void CmmBinaryExpr::produce_attributes ()
{
  CmmExpr::produce_attributes ();
  store_enum ("kind", conv_CmmBinaryExpr_kind (this->kind), this->kind);
  store_item ("left", this->left);
  store_item ("right", this->right);
}

void CmmCondExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("cond_expr", this->cond_expr);
  store_item ("then_expr", this->then_expr);
  store_item ("else_expr", this->else_expr);
}

void CmmStatSect::produce_sequence ()
{
  CmmStat * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmDeclStat::produce_attributes ()
{
  CmmStat::produce_attributes ();
  store_item ("decl", this->decl);
}

void CmmLabeledStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_ident ("lab", this->lab);
  store_item ("body", this->body);
}

void CmmCaseStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("case_expr", this->case_expr);
  store_item ("body", this->body);
}

void CmmDefaultStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("body", this->body);
}

void CmmExprStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("inner_expr", this->inner_expr);
}

void CmmCondition::produce_attributes ()
{
  store_item ("cond_expr", this->cond_expr);
}

void CmmIfStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("cond", this->cond);
  store_item ("then_stat", this->then_stat);
  store_item ("else_stat", this->else_stat);
}

void CmmSwitchStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("cond", this->cond);
  store_item ("body", this->body);
}

void CmmWhileStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("cond", this->cond);
  store_item ("body", this->body);
}

void CmmDoStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("body", this->body);
  store_item ("cond_expr", this->cond_expr);
}

void CmmForStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("from_decl", this->from_decl);
  store_item ("from_expr", this->from_expr);
  store_bool ("empty_param", this->empty_param);
  store_item ("to_expr", this->to_expr);
  store_item ("cond_expr", this->cond_expr);
  store_item ("body", this->body);
}

void CmmReturnStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("return_expr", this->return_expr);
}

void CmmGotoStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_ident ("goto_lab", this->goto_lab);
}

void CmmDeclSect::produce_sequence ()
{
  CmmDecl * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmCtorHead::produce_attributes ()
{
  store_location (location [0]);
  store_item ("spec", this->spec);
  store_bool ("destructor", this->destructor);
  store_item ("name", this->name);
  store_item ("cont", this->cont);
  store_item ("init", this->init);
  store_bool ("body", this->body);
  store_bool ("try_block", this->try_block);
  store_bool ("empty", this->empty);
}

void CmmCtorDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_item ("spec", this->spec);
  store_bool ("destructor", this->destructor);
  store_item ("decl", this->decl);
  store_item ("init", this->init);
  store_item ("statements", this->statements);
  store_item ("handlers", this->handlers);
  store_bool ("empty", this->empty);
}

void CmmSpecSect::produce_sequence ()
{
  CmmSpec * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmFuncHead::produce_attributes ()
{
  store_location (location [0]);
  store_item ("spec", this->spec);
  store_item ("ptr", this->ptr);
  store_item ("name", this->name);
  store_item ("params", this->params);
  store_item ("cont", this->cont);
  store_bool ("body", this->body);
  store_bool ("try_block", this->try_block);
  store_bool ("empty", this->empty);
}

void CmmFuncDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_item ("spec", this->spec);
  store_item ("decl", this->decl);
  store_item ("statements", this->statements);
  store_item ("handlers", this->handlers);
  store_bool ("empty", this->empty);
}

void CmmSimpleDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_item ("spec", this->spec);
}

void CmmSimpleDecl::produce_sequence ()
{
  CmmInitDeclarator * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmInitDeclarator::produce_attributes ()
{
  store_location (location [0]);
  store_item ("decl", this->decl);
  store_item ("init", this->init);
  store_item ("width", this->width);
}

void CmmDeclSpec::produce_attributes ()
{
  store_item ("mode", this->mode);
  store_item ("name", this->name);
  store_item ("template_args", this->template_args);
  store_item ("class_spec", this->class_spec);
  store_item ("enum_spec", this->enum_spec);
  store_item ("type", this->type);
  store_item ("mode2", this->mode2);
}

void CmmTypeSect::produce_sequence ()
{
  CmmType * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmAllocTypeSpecSect::produce_sequence ()
{
  CmmAllocTypeSpec * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmAllocTypeSpec::produce_attributes ()
{
  store_location (location [0]);
  store_bool ("a_const", this->a_const);
  store_bool ("a_volatile", this->a_volatile);
  store_item ("name", this->name);
  store_item ("class_spec", this->class_spec);
  store_item ("enum_spec", this->enum_spec);
  store_item ("simple", this->simple);
}

void CmmEnumDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_item ("name", this->name);
  store_item ("items", this->items);
}

void CmmEnumSect::produce_sequence ()
{
  CmmEnumItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmEnumItem::produce_attributes ()
{
  store_location (location [0]);
  store_ident ("name", this->name);
  store_item ("init", this->init);
}

void CmmNamespace::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_item ("unnamed_body", this->unnamed_body);
  store_ident ("namespace_name", this->namespace_name);
  store_item ("value", this->value);
  store_item ("body", this->body);
}

void CmmUsingDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_bool ("a_namespace", this->a_namespace);
  store_bool ("a_typename", this->a_typename);
  store_item ("name", this->name);
}

void CmmExternDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_string ("language", this->language);
  store_item ("decl_list", this->decl_list);
  store_item ("decl", this->decl);
}

void CmmAsmDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_string ("instruction", this->instruction);
}

void CmmTypeId::produce_attributes ()
{
  store_item ("spec", this->spec);
  store_item ("decl", this->decl);
}

void CmmCommonTypeId::produce_attributes ()
{
  store_item ("spec", this->spec);
  store_item ("decl", this->decl);
}

void CmmDeclarator::produce_attributes ()
{
  store_location (location [0]);
  store_item ("ptr", this->ptr);
  store_item ("name", this->name);
  store_item ("inner", this->inner);
  store_item ("cont", this->cont);
}

void CmmAbstractDeclarator::produce_attributes ()
{
  store_location (location [0]);
  store_item ("ptr", this->ptr);
  store_item ("inner", this->inner);
  store_item ("cont", this->cont);
}

void CmmCommonDeclarator::produce_attributes ()
{
  store_location (location [0]);
  store_item ("ptr", this->ptr);
  store_item ("name", this->name);
  store_item ("inner", this->inner);
  store_item ("cont", this->cont);
}

void CmmContDeclaratorSect::produce_sequence ()
{
  CmmContDeclarator * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmContFunctionDeclarator::produce_attributes ()
{
  store_location (location [0]);
  CmmContDeclarator::produce_attributes ();
  store_item ("parameters", this->parameters);
  store_item ("cv_spec", this->cv_spec);
  store_item ("exception_spec", this->exception_spec);
}

void CmmContArrayDeclarator::produce_attributes ()
{
  store_location (location [0]);
  CmmContDeclarator::produce_attributes ();
  store_item ("lim", this->lim);
}

void CmmPtrSpecifierSect::produce_sequence ()
{
  CmmPtrSpecifier * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmPtrSpecifier::produce_attributes ()
{
  store_location (location [0]);
  store_item ("cv_spec", this->cv_spec);
  store_bool ("pointer", this->pointer);
  store_bool ("reference", this->reference);
}

void CmmCvSpecifierSect::produce_sequence ()
{
  CmmCvSpecifier * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmCvSpecifier::produce_attributes ()
{
  store_location (location [0]);
  store_bool ("a_const", this->a_const);
  store_bool ("a_volatile", this->a_volatile);
}

void CmmParamSect::produce_sequence ()
{
  CmmParamItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmParamItem::produce_attributes ()
{
  store_location (location [0]);
  store_item ("type", this->type);
  store_item ("init", this->init);
  store_bool ("dots", this->dots);
}

void CmmInitializer::produce_attributes ()
{
  store_location (location [0]);
  store_item ("value", this->value);
}

void CmmInitSimple::produce_attributes ()
{
  CmmInitItem::produce_attributes ();
  store_item ("expr", this->expr);
}

void CmmInitList::produce_sequence ()
{
  CmmInitItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmClassDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_enum ("style", conv_CmmClassDecl_style (this->style), this->style);
  store_item ("base_list", this->base_list);
  store_item ("members", this->members);
  store_bool ("anonymous", this->anonymous);
  store_item ("name", this->name);
}

void CmmBaseSect::produce_sequence ()
{
  CmmBaseItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmCtorInitializer::produce_sequence ()
{
  CmmMemberInitializer * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmMemberInitializer::produce_attributes ()
{
  store_location (location [0]);
  store_item ("name", this->name);
  store_item ("params", this->params);
}

void CmmSpecialFuction::produce_attributes ()
{
  store_location (location [0]);
  store_bool ("spec_new", this->spec_new);
  store_bool ("spec_delete", this->spec_delete);
  store_bool ("spec_add", this->spec_add);
  store_bool ("spec_sub", this->spec_sub);
  store_bool ("spec_mul", this->spec_mul);
  store_bool ("spec_div", this->spec_div);
  store_bool ("spec_mod", this->spec_mod);
  store_bool ("spec_xor", this->spec_xor);
  store_bool ("spec_and", this->spec_and);
  store_bool ("spec_or", this->spec_or);
  store_bool ("spec_not", this->spec_not);
  store_bool ("spec_log_not", this->spec_log_not);
  store_bool ("spec_eq", this->spec_eq);
  store_bool ("spec_lt", this->spec_lt);
  store_bool ("spec_gt", this->spec_gt);
  store_bool ("spec_add_assign", this->spec_add_assign);
  store_bool ("spec_sub_assign", this->spec_sub_assign);
  store_bool ("spec_mul_assign", this->spec_mul_assign);
  store_bool ("spec_div_assign", this->spec_div_assign);
  store_bool ("spec_mod_assign", this->spec_mod_assign);
  store_bool ("spec_xor_assign", this->spec_xor_assign);
  store_bool ("spec_and_assign", this->spec_and_assign);
  store_bool ("spec_or_assign", this->spec_or_assign);
  store_bool ("spec_shl", this->spec_shl);
  store_bool ("spec_shr", this->spec_shr);
  store_bool ("spec_shl_assign", this->spec_shl_assign);
  store_bool ("spec_shr_assign", this->spec_shr_assign);
  store_bool ("spec_ne", this->spec_ne);
  store_bool ("spec_le", this->spec_le);
  store_bool ("spec_ge", this->spec_ge);
  store_bool ("spec_log_and", this->spec_log_and);
  store_bool ("spec_log_or", this->spec_log_or);
  store_bool ("spec_inc", this->spec_inc);
  store_bool ("spec_dec", this->spec_dec);
  store_bool ("spec_comma", this->spec_comma);
  store_bool ("spec_mebber_deref", this->spec_mebber_deref);
  store_bool ("spec_deref", this->spec_deref);
  store_bool ("spec_call", this->spec_call);
  store_bool ("spec_index", this->spec_index);
}

void CmmTemplateDecl::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_bool ("export_template", this->export_template);
  store_item ("params", this->params);
  store_bool ("explicit_specialization", this->explicit_specialization);
  store_bool ("parameter_block", this->parameter_block);
  store_bool ("explicit_instatiation", this->explicit_instatiation);
  store_item ("body", this->body);
}

void CmmTemplateParamSect::produce_sequence ()
{
  CmmTemplateParam * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmTemplateTypeParam::produce_attributes ()
{
  store_location (location [0]);
  CmmTemplateParam::produce_attributes ();
  store_bool ("a_class", this->a_class);
  store_bool ("a_typename", this->a_typename);
  store_ident ("name", this->name);
  store_item ("value", this->value);
}

void CmmTemplateExtraParam::produce_attributes ()
{
  store_location (location [0]);
  CmmTemplateParam::produce_attributes ();
  store_item ("params", this->params);
  store_ident ("name", this->name);
  store_item ("value", this->value);
}

void CmmTemplateValueParam::produce_attributes ()
{
  store_location (location [0]);
  CmmTemplateParam::produce_attributes ();
  store_item ("type", this->type);
  store_item ("init", this->init);
}

void CmmTemplateArgSect::produce_sequence ()
{
  CmmTemplateArg * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmTemplateTypeArg::produce_attributes ()
{
  CmmTemplateArg::produce_attributes ();
  store_item ("type", this->type);
}

void CmmTemplateValueArg::produce_attributes ()
{
  CmmTemplateArg::produce_attributes ();
  store_item ("value", this->value);
}

void CmmTryStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("body", this->body);
  store_item ("handlers", this->handlers);
}

void CmmHandlerSect::produce_sequence ()
{
  CmmHandlerItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmHandlerItem::produce_attributes ()
{
  store_location (location [0]);
  store_item ("exception", this->exception);
  store_item ("body", this->body);
}

void CmmExceptionDecl::produce_attributes ()
{
  store_location (location [0]);
  store_item ("type", this->type);
  store_bool ("dots", this->dots);
}

void CmmThrowExpr::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_item ("expr", this->expr);
}

void CmmExceptionSpec::produce_sequence ()
{
  CmmTypeId * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void CmmIncludeName::produce_attributes ()
{
  store_location (location [0]);
  store_string ("include_q_str", this->include_q_str);
  store_string ("include_h_str", this->include_h_str);
  store_ident ("include_id", this->include_id);
}

void CmmDirective::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_string ("d1", this->d1);
  store_bool ("include_directive", this->include_directive);
  store_item ("include_name", this->include_name);
  store_bool ("if_directive", this->if_directive);
  store_string ("if_expr", this->if_expr);
  store_bool ("ifdef_directive", this->ifdef_directive);
  store_ident ("ifdef_name", this->ifdef_name);
  store_bool ("ifndef_directive", this->ifndef_directive);
  store_ident ("ifndef_name", this->ifndef_name);
  store_bool ("else_directive", this->else_directive);
  store_bool ("endif_directive", this->endif_directive);
  store_bool ("define_directive", this->define_directive);
  store_ident ("define_name", this->define_name);
  store_bool ("undef_directive", this->undef_directive);
  store_ident ("undef_name", this->undef_name);
  store_string ("d2", this->d2);
}

void CmmDirectiveStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_string ("d1", this->d1);
  store_bool ("include_directive", this->include_directive);
  store_item ("include_name", this->include_name);
  store_bool ("if_directive", this->if_directive);
  store_string ("if_expr", this->if_expr);
  store_bool ("ifdef_directive", this->ifdef_directive);
  store_ident ("ifdef_name", this->ifdef_name);
  store_bool ("ifndef_directive", this->ifndef_directive);
  store_ident ("ifndef_name", this->ifndef_name);
  store_bool ("else_directive", this->else_directive);
  store_bool ("endif_directive", this->endif_directive);
  store_bool ("define_directive", this->define_directive);
  store_ident ("define_name", this->define_name);
  store_bool ("undef_directive", this->undef_directive);
  store_ident ("undef_name", this->undef_name);
  store_string ("d2", this->d2);
}

void CmmNamespaceExtension::produce_attributes ()
{
  store_location (location [0]);
  CmmDecl::produce_attributes ();
  store_bool ("open", this->open);
}

void CmmClusterStat::produce_attributes ()
{
  CmmStat::produce_attributes ();
  store_item ("body", this->body);
}

void CmmIntoStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("into_param", this->into_param);
  store_item ("store_param", this->store_param);
  store_bool ("with_into", this->with_into);
  store_item ("group", this->group);
  store_item ("comment", this->comment);
  store_item ("def", this->def);
  store_bool ("prior", this->prior);
  store_item ("expr", this->expr);
  store_item ("expr_list", this->expr_list);
  store_item ("stat", this->stat);
  store_item ("code", this->code);
  store_item ("decl", this->decl);
  store_item ("decl_list", this->decl_list);
  store_item ("param_list", this->param_list);
  store_item ("enum_type", this->enum_type);
  store_item ("enum_items", this->enum_items);
  store_item ("cls", this->cls);
  store_item ("base_list", this->base_list);
  store_bool ("a_public", this->a_public);
  store_bool ("a_protected", this->a_protected);
  store_bool ("a_private", this->a_private);
  store_item ("access", this->access);
  store_item ("member", this->member);
  store_item ("members", this->members);
  store_item ("ctor_init", this->ctor_init);
  store_item ("init_item", this->init_item);
  store_item ("init_list", this->init_list);
  store_item ("text_sect", this->text_sect);
}

void CmmTextItem::produce_attributes ()
{
  store_location (location [0]);
  store_string ("token_value", this->token_value);
  store_bool ("token_flag", this->token_flag);
  store_string ("space_value", this->space_value);
  store_bool ("white_space_flag", this->white_space_flag);
  store_string ("new_line_value", this->new_line_value);
  store_bool ("new_line_flag", this->new_line_flag);
  store_string ("other_value", this->other_value);
  store_bool ("other_flag", this->other_flag);
}

void CmmDefineStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("def", this->def);
}

void CmmGroupStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("value", this->value);
}

void CmmIndentStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("value", this->value);
}

void CmmWhenStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("cond_expr", this->cond_expr);
  store_item ("then_body", this->then_body);
  store_item ("else_body", this->else_body);
}

void CmmWhenStat::produce_sequence ()
{
  CmmWhenItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

/*
void CmmWhenItem::produce_attributes ()
{
  store_location (location [0]);
  store_item ("sub_expr", this->sub_expr);
  store_item ("sub_body", this->sub_body);
}
*/

void CmmCallStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("body", this->body);
}

void CmmLocationStat::produce_attributes ()
{
  store_location (location [0]);
  CmmStat::produce_attributes ();
  store_item ("place", this->place);
}

void AltName::produce_attributes ()
{
  store_location (location [0]);
  store_ident ("id", this->id);
  store_string ("txt", this->txt);
}

void AltSimple::produce_attributes ()
{
  store_location (location [0]);
  store_item ("name", this->name);
  store_item ("opt", this->opt);
  store_item ("sub_expr", this->sub_expr);
}

void AltBranch::produce_sequence ()
{
  AltSimple * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void AltExpr::produce_sequence ()
{
  AltBranch * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void DefItem::produce_attributes ()
{
  store_item ("name", this->name);
}

void DefList::produce_sequence ()
{
  DefItem * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void LogFactor::produce_attributes ()
{
  store_location (location [0]);
  store_item ("name", this->name);
  store_item ("not_factor", this->not_factor);
  store_item ("sub_expr", this->sub_expr);
}

void LogTerm::produce_sequence ()
{
  LogFactor * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void LogExpr::produce_sequence ()
{
  LogTerm * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void LogStat::produce_attributes ()
{
  store_location (location [0]);
  store_item ("name", this->name);
  store_item ("expr", this->expr);
}

void NumStatSect::produce_sequence ()
{
  NumStat * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void NumCode::produce_attributes ()
{
  store_location (location [0]);
  NumStat::produce_attributes ();
  store_string ("code", this->code);
}

void NumWhen::produce_attributes ()
{
  store_location (location [0]);
  NumStat::produce_attributes ();
  store_item ("cond", this->cond);
  store_item ("when_stat", this->when_stat);
  store_item ("otherwise_stat", this->otherwise_stat);
}

void NumWhen::produce_sequence ()
{
  NumStatAlternative * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void NumStatAlternative::produce_attributes ()
{
  store_location (location [0]);
  store_item ("cond", this->cond);
  store_item ("stat", this->stat);
}

void NumAssign::produce_attributes ()
{
  store_location (location [0]);
  NumStat::produce_attributes ();
  store_ident ("id", this->id);
  store_item ("list", this->list);
  store_item ("expr", this->expr);
  store_item ("selection", this->selection);
}

void NumSelection::produce_attributes ()
{
  store_location (location [0]);
  store_item ("cond", this->cond);
  store_item ("when_expr", this->when_expr);
  store_item ("otherwise_expr", this->otherwise_expr);
}

void NumSelection::produce_sequence ()
{
  NumExprAlternative * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void NumExprAlternative::produce_attributes ()
{
  store_location (location [0]);
  store_item ("cond", this->cond);
  store_item ("expr", this->expr);
}

void NumCond::produce_sequence ()
{
  NumAndCond * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void NumAndCond::produce_sequence ()
{
  NumCompCond * subitem = this->first;
  while (subitem != NULL)
  {
    insert_item (subitem);
    subitem = subitem->next;
  }
}

void NumCompCond::produce_attributes ()
{
  store_location (location [0]);
  CmmExpr::produce_attributes ();
  store_ident ("left_id", this->left_id);
  store_bool ("equal", this->equal);
  store_bool ("not_equal", this->not_equal);
  store_ident ("right_id", this->right_id);
  store_string ("right_val", this->right_val);
  store_bool ("relation", this->relation);
}

/* %% conv */

string conv_CmmAccess (CmmAccess p_item)
{
  switch (p_item)
  {
    case PrivateAccess:
      return "PrivateAccess"; 
      break;
    case ProtectedAccess:
      return "ProtectedAccess"; 
      break;
    case PublicAccess:
      return "PublicAccess"; 
      break;
    case NoAccess:
      return "NoAccess"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmSpecKind (CmmSpecKind p_item)
{
  switch (p_item)
  {
    case a_inline:
      return "a_inline"; 
      break;
    case a_virtual:
      return "a_virtual"; 
      break;
    case a_explicit:
      return "a_explicit"; 
      break;
    case a_auto:
      return "a_auto"; 
      break;
    case a_register:
      return "a_register"; 
      break;
    case a_static:
      return "a_static"; 
      break;
    case a_extern:
      return "a_extern"; 
      break;
    case a_mutable:
      return "a_mutable"; 
      break;
    case a_friend:
      return "a_friend"; 
      break;
    case a_typedef:
      return "a_typedef"; 
      break;
    case a_const:
      return "a_const"; 
      break;
    case a_volatile:
      return "a_volatile"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmTypeKind (CmmTypeKind p_item)
{
  switch (p_item)
  {
    case a_char:
      return "a_char"; 
      break;
    case a_wchar:
      return "a_wchar"; 
      break;
    case a_bool:
      return "a_bool"; 
      break;
    case a_short:
      return "a_short"; 
      break;
    case a_int:
      return "a_int"; 
      break;
    case a_long:
      return "a_long"; 
      break;
    case a_signed:
      return "a_signed"; 
      break;
    case a_unsigned:
      return "a_unsigned"; 
      break;
    case a_float:
      return "a_float"; 
      break;
    case a_double:
      return "a_double"; 
      break;
    case a_void:
      return "a_void"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmDefineItemMode (CmmDefineItemMode p_item)
{
  switch (p_item)
  {
    case ident_mode:
      return "ident_mode"; 
      break;
    case text_mode:
      return "text_mode"; 
      break;
    case string_mode:
      return "string_mode"; 
      break;
    case expr_mode:
      return "expr_mode"; 
      break;
    case expr_list_mode:
      return "expr_list_mode"; 
      break;
    case stat_mode:
      return "stat_mode"; 
      break;
    case decl_mode:
      return "decl_mode"; 
      break;
    case init_item_mode:
      return "init_item_mode"; 
      break;
    case init_list_mode:
      return "init_list_mode"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmValueExpr_kind (CmmValueExpr_kind p_item)
{
  switch (p_item)
  {
    case ValueExp:
      return "ValueExp"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmModernCastExpr_kind (CmmModernCastExpr_kind p_item)
{
  switch (p_item)
  {
    case DynamicCast:
      return "DynamicCast"; 
      break;
    case StaticCast:
      return "StaticCast"; 
      break;
    case ConstCast:
      return "ConstCast"; 
      break;
    case ReinterpreterCast:
      return "ReinterpreterCast"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmBinaryExpr_kind (CmmBinaryExpr_kind p_item)
{
  switch (p_item)
  {
    case DotMemberExp:
      return "DotMemberExp"; 
      break;
    case ArrowMemverExp:
      return "ArrowMemverExp"; 
      break;
    case MulExp:
      return "MulExp"; 
      break;
    case DivExp:
      return "DivExp"; 
      break;
    case ModExp:
      return "ModExp"; 
      break;
    case AddExp:
      return "AddExp"; 
      break;
    case SubExp:
      return "SubExp"; 
      break;
    case ShlExp:
      return "ShlExp"; 
      break;
    case ShrExp:
      return "ShrExp"; 
      break;
    case LtExp:
      return "LtExp"; 
      break;
    case GtExp:
      return "GtExp"; 
      break;
    case LeExp:
      return "LeExp"; 
      break;
    case GeExp:
      return "GeExp"; 
      break;
    case EqExp:
      return "EqExp"; 
      break;
    case NeExp:
      return "NeExp"; 
      break;
    case BitAndExp:
      return "BitAndExp"; 
      break;
    case BitXorExp:
      return "BitXorExp"; 
      break;
    case BitOrExp:
      return "BitOrExp"; 
      break;
    case LogAndExp:
      return "LogAndExp"; 
      break;
    case LogOrExp:
      return "LogOrExp"; 
      break;
    case AssignExp:
      return "AssignExp"; 
      break;
    case AddAssignExp:
      return "AddAssignExp"; 
      break;
    case SubAssignExp:
      return "SubAssignExp"; 
      break;
    case MulAssignExp:
      return "MulAssignExp"; 
      break;
    case DivAssignExp:
      return "DivAssignExp"; 
      break;
    case ModAssignExp:
      return "ModAssignExp"; 
      break;
    case ShlAssignExp:
      return "ShlAssignExp"; 
      break;
    case ShrAssignExp:
      return "ShrAssignExp"; 
      break;
    case AndAssignExp:
      return "AndAssignExp"; 
      break;
    case XorAssignExp:
      return "XorAssignExp"; 
      break;
    case OrAssignExp:
      return "OrAssignExp"; 
      break;
    case CommaExp:
      return "CommaExp"; 
      break;
    default: 
      assert (false);
  }
}

string conv_CmmClassDecl_style (CmmClassDecl_style p_item)
{
  switch (p_item)
  {
    case ClassStyle:
      return "ClassStyle"; 
      break;
    case StructStyle:
      return "StructStyle"; 
      break;
    case UnionStyle:
      return "UnionStyle"; 
      break;
    default: 
      assert (false);
  }
}

} // end of namespace gibbon
